#!/bin/ksh
#=======================================================================#
# Licensed Materials - Property of IBM                                  #
# 5724-C04                                                              #
# Copyright IBM Corporation 2006-2014 All Rights Reserved               #
#=======================================================================#
# db2access.sh - This script queries the TEPS data base (in DB2) to     #
#    determine whether the user has the correct authorizations.         #
#-----------------------------------------------------------------------#
# SYNTAX: db2access.sh <db2inst> <tepsdb> <tepsuser> <tepspassfile>     #
#                                                                       #
#    Where:                                                             #
#       <db2inst> is the DB2 instance name. Example: "db2inst1".        #
#       <tepsdb> is the name of the TEPS data base. Example: "TEPS".    #
#       <tepsuser> is the name of the login userid for the TEPS data    #
#          base. Example: "itmuser".                                    #
#       <tepspassfile> is the name of a file that contains the password #
#          for <tepsuser>. The file contains a single record in the     #
#          form "DB2PW=password".                                       #
#                                                                       #
# RETURN CODES:                                                         #
#    0 - The user has the correct access authorizations.                #
#    1 - The user does not have the correct access authorizations or    #
#        db2access.sh was unable to determine access authorizations.    #
#                                                                       #
# NOTE: This script employs its own trace mechanism that is enabled by  #
#    the environment variable setting TRACE_LEVEL=DEBUG_MAX. If this    #
#    setting is in effect, a file "/tmp/db2access-<PID>.log" is left    #
#    with details as to what happened for that particular run. We can't #
#    use the standard ITMSH_TRACE/itmsh_trace_main because the Java     #
#    caller eats the trace output.                                      #
#=======================================================================#

#-----------------------------------------------------------------------#
# Initialization.                                                       #
#-----------------------------------------------------------------------#
unalias -a;                                 # avert alias interference  #

db2access_rc=0;                             # 0=access ok, 1=bad        #
db2access_log="/tmp/db2access-${$}.log";    # log/trace output          #
db2access_tmp="/tmp/db2access-${$}.tmp";    # private temporary file    #
unset DB2OPTIONS;                           # prevent funny stuff       #

print "db2access args: ${@}" > "${db2access_log}";

#-----------------------------------------------------------------------#
# Load the db2 environment for the instance specified.                  #
#-----------------------------------------------------------------------#
db2home=$(eval print "~$1")
ksh -c ". \"${db2home}/sqllib/profile.env\"" 2>> "${db2access_log}";
if (( ${?} == 0 )); then
   . "${db2home}/sqllib/profile.env";
   ksh -c ". \"${db2home}/sqllib/db2profile\"" 2>> "${db2access_log}";
   if (( ${?} == 0 )); then
      . "${db2home}/sqllib/db2profile";
   else
      db2access_rc=1;
   fi;
else
   db2access_rc=1;
fi;

#-----------------------------------------------------------------------#
# Connect to the TEPS data base. Note that we pass the CONNECT command  #
# via a temporary file that only we can read/write. We could pass this  #
# directly through the db2 incantation like we do select below. But,    #
# the password would then be visible to the public in ps output.        #
#-----------------------------------------------------------------------#
if (( ${db2access_rc} == 0 )); then
   DB2PW=$(grep "^DB2PW=" "${4}" 2>> "${db2access_log}" | cut -d"=" -f2);
   
   if [[ "${DB2PW}" = "" ]]; then
      print "Error: Password not found in ${4}." >> "${db2access_log}";
      db2access_rc=1;                       # no password == no connect #
   else
      touch "${db2access_tmp}";             # make the temp file        #
      chmod 600 "${db2access_tmp}";         # make it private to us     #
      db2Connect="CONNECT TO ${2} USER ${3} USING";
      print "${db2Connect} <password_hidden>" >> "${db2access_log}"; 
      print "${db2Connect} ${DB2PW}"          >> "${db2access_tmp}"; 
      
      db2 -f "${db2access_tmp}" >> "${db2access_log}" 2>&1;
      db2RC=${?};
      rm -rf "${db2access_tmp}" > /dev/null 2>&1;
      print "db2 connect rc=${db2RC}" >> "${db2access_log}";
      
      if (( ${db2RC} != 0 )); then
         db2access_rc=1;                    # connect failed            #
      fi;                                   # (if(db2RC!=0))            #
   fi;                                      # (else(DB2PW=""))          #
fi;                                         # (if(db2access_rc==0))     #

#-----------------------------------------------------------------------#
# Get authorization information from the data base. The result we're    #
# looking for is ZERO rows. We're asking db2 to tell us the things for  #
# which the user is NOT authorized. So, any row output means there is   #
# a problem. We record the output to the log/trace file, but the return #
# code from db2 tells us the important part:                            #
#                                                                       #
#    1 means no rows returned (this is what we want)                    #
#    0 means some rows returned (this is bad)                           #
#    2+ means db2 failed for some reason (this is really bad).          #
#-----------------------------------------------------------------------#
if (( ${db2access_rc} == 0 )); then
   authTEST="'DBADM','CREATETAB','BINDADD','CONNECT','CREATE_NOT_FENCED_ROUTINE','IMPLICIT_SCHEMA','LOAD','CREATE_EXTERNAL_ROUTINE','QUIESCE_CONNECT'";
   typeset -u authUSER="${3}";
   
   db2Select="SELECT AUTHORITY";
   db2Select="${db2Select} FROM TABLE (SYSPROC.AUTH_LIST_AUTHORITIES_FOR_AUTHID ('${authUSER}','U'))";
   db2Select="${db2Select} WHERE D_USER!='Y' AND AUTHORITY IN (${authTEST})";
   print "${db2Select}" >> "${db2access_log}";
   db2 -x "${db2Select}" >> "${db2access_log}" 2>&1;
   db2RC=${?};
   print "db2 select rc=${db2RC}" >> "${db2access_log}";

   if (( ${db2RC} == 1 )); then             # db2 returned no rows...   #
      db2access_rc=0;                       # ...means access is ok     #
   elif (( ${db2RC} == 0 )); then           # db2 returned some rows... #
      db2access_rc=1;                       # ...means access is bad    #
   elif (( ${db2RC} > 1 )); then            # db2 select failed...      #
      db2access_rc=1;                       # ...assume access is bad   #
   fi;                                      # (elif(db2RC...))          #
fi;                                         # (if(db2access_rc==0))     #

#-----------------------------------------------------------------------#
# Finalize the log and return results to caller.                        #
#-----------------------------------------------------------------------#
if [[ "${TRACE_LEVEL}" != "DEBUG_MAX" ]]; then   # are we tracing?      #
   rm -rf "${db2access_log}" > /dev/null 2>&1;   # nope, nuke the log   #
else
   print "db2access rc: ${db2access_rc}" >> "${db2access_log}";
fi;

exit ${db2access_rc};
