#!/bin/ksh

# plugin type to execute
PLUGIN_TYPE=config

# plugin action to execute
PLUGIN_ACTION=ondemand

# teps code
PC=cq

# default error return code
RCE=5

# fail_exit - exit with error
# params: message [return_code]
fail_exit()
{
  msg=$1
  rc=$RCE
  [[ -n "$2" ]] && rc=$2
  log $msg
  exit_info "$msg" $rc
}

# exit and display information
# params: message return_code
exit_info()
{
  if [[ -n "$LOGFILE" ]];
  then
    case $2 in
      0 ) msg="All operations completed successfully" ;;
      1 ) msg="No applications found for processing" ;;
      2 ) msg="Failed deployment/redeployment operation" ;;
      3 ) msg="Failed start/restart operation" ;;
      4 ) msg="Deployable application does not have starting/restarting command parameters" ;;
      5 ) msg="General error" ;;
    esac
    rctext="$2,$msg,$(basename $LOGFILE 2>/dev/null)"
    log "Exiting with text = $rctext"
    log "Return code = $2"
  else
    rctext="$2,$1,"
  fi  
  print "$rctext"
  exit $2
}

# log - log message to file
# params: message
log()
{
  [[ -n "$LOGFILE" ]] && print "$(date +%m.%d.%Y\ %T) $1" >> $LOGFILE	
}

# -----------------------------------------------
# main code starts here

# set itm installation directory
CANDLEHOME=$(cd $(dirname $0 2>/dev/null) ; dirname $(pwd) 2>/dev/null)

# verify itm installation directory
[[ ! -d "$CANDLEHOME" ]] && fail_exit "Cannot set CANDLEHOME environment variable"

[[ $(print "$CANDLEHOME" | wc -w) -ne 1 ]] && fail_exit "Blanks are not permitted in install_dir"

# define log file
LOGFILE="$CANDLEHOME/logs/deploysdewas_$(date +%m%d%y%H%M%S).log" 

log "$0 script started"

# initialize and execute dynarch.shl script

CurRegPath=$CANDLEHOME/registry
ArchTbl=$CurRegPath/archdsc.tbl
DynShl=$CANDLEHOME/bin/dynarch.shl

. $DynShl $ArchTbl 2>/dev/null

log "System architecture = $cArch"

# get base architecture li6263 -> li
baseArch=${cArch%%[0-9]*}

# check if teps is installed

pcVer=$(ls $CANDLEHOME/registry/$PC$baseArch*.ver 2>/dev/null)
[[ -z "$pcVer" ]] && fail_exit "TEPS not installed, exiting..."

# check if ewas is installed

iwVer=$(ls $CANDLEHOME/registry/iw$baseArch*.ver 2>/dev/null)
[[ -z "$iwVer" ]] && fail_exit "eWAS not installed, exiting..."

# get teps binary architecture

setBinLibArchs $cArch $PC

log "TEPS architecture = $binArch"

# set path to teps .ver file

pcVer=$CANDLEHOME/registry/$PC$binArch.ver

# get teps version

vrmf=$(grep "^VRMF *=" $pcVer | cut -d"=" -f2 | tr -d '\r' )

log "TEPS version = $vrmf"

# determine what jre to use
  
useJreArch=$cArch

# in case we are running platform lx we need to modify java arch prerequisite

if [[ "$baseArch" = "lx" ]]; 
then
  saveLang=$LANG
  export LANG=C
  cArchJava=$(grep "^preReqALL = jr" $pcVer 2>/dev/null | cut -d"=" -f2 | cut -d "|" -f1 | cut -c 4-)
  export LANG=$saveLang
  #use cArchJava only if component has jre prereq
  [ -n "$cArchJava" ] && useJreArch=$cArchJava
fi

setJavaEnv $useJreArch

log "JRE executable = $JREEXE, classpath = $CLASSPATH"

# plugin execution output file

resultfile=$CANDLEHOME/tmp/ewas_upd.out

# remove result file
rm -rf $resultfile > /dev/null 2>&1
[[ $? -eq 0 ]] || fail_exit "Unable to remove old result file $resultfile"

# execute plugin processing

rc=0

log "Executing $PC plugin $PLUGIN_TYPE.$PLUGIN_ACTION, result will be saved in $resultfile file"

$JREEXE -classpath $CLASSPATH $SAFE_MODE -DGlobalDbgLevel="DEBUG_MAX" -DInstallRASConfig="$CANDLEHOME/config/ITMConfigRAS.properties" ITMinstall.InstallComponentPluginCall $PLUGIN_TYPE $PLUGIN_ACTION $CANDLEHOME $cArch $PC $binArch $vrmf > /dev/null 2>&1

if [[ $? -eq 0 ]] && [[ -f $resultfile ]];
then
  rc=$(awk 'NR==1{print $1}' $resultfile 2>/dev/null)  
  log "Result file $resultfile found after $PC plugin $PLUGIN_TYPE.$PLUGIN_ACTION execution, return code = $rc"
else
  rc=2
  log "Result file $resultfile not found after $PC plugin $PLUGIN_TYPE.$PLUGIN_ACTION execution, return code = $rc"
fi

exit_info "$0 finished" $rc