#!/bin/ksh
# ****************************************************************************
# * Licensed Materials - Property of IBM
# * 
# * OCO Source Materials - IBM Confidential
# * 5724-C04
# *
# * Copyright IBM Corporation 2007-2014 All Rights Reserved             //IV65374
# *
# * The source code for this program is not published or other-
# * wise divested of its trade secrets, irrespective of what has
# * been deposited with the U.S. Copyright Office.
# *****************************************************************************

### settings
RC_OK=0
RC_DEFAULT=1
RC_OLDER=2
RC_COPY_FAIL=3
RC_MANIFEST_FAIL=4
RC_PARAM_FAIL=5
RC_COPY_TEMS_FAIL=6

# set this variable to enable debug information
DEBUG=

### functions

# exit with error
# $1 - message text
# $2 - return code - optional, default is RC_DEFAULT
function fail_exit
{
  loginfo "$1"  
  rc=$RC_DEFAULT
  [[ -n "$2" ]] && rc=$2
  print "$rc,$1,$LOGFILE"
  exit $rc
}

# cleanup selected directory and exit with error
# $1 - directory to remove
# $2 - message text
# $3 - return code - optional, default is RC_DEFAULT
function fail_exit_cleanup
{
  rm -rf $1 > /dev/null 2>&1
  fail_exit "$2" $3
}

# print utility usage
# $1 - program name
function print_usage
{
  print "support installation: $1 -h installation_directory -d sda_support_packages_directory -p product_code -t support_type [ -f ] [ -b backup_directory ]"
  print "restore backup: $1 -h installation_directory -r backup_directory -p product_code -t support_type"
  exit $RC_PARAM_FAIL
}

# log information to file and if debug enabled on screen
# $1 - message text
function loginfo
{
  [[ -n "$DEBUG" ]] && print "$1"
  [[ -n "$LOG" ]] && print "$(date +%m.%d.%Y\ %T) $1" >> $LOG
}

# gets product binary architecture
# $1 - product code
# $2 - operatin system code
function getbinarch
{
  typeset pc=$1
  typeset os=$2
  typeset verfile=$(ls $CANDLEHOME/registry/$pc$os*.ver 2>/dev/null)
  verfile=${verfile##*/}
  typeset binarch=${verfile%.ver}
  binarch=${binarch#$pc}
  print $binarch 
}

# gets tems server name
function gettemsname
{
  temsconfig=$(ls $CANDLEHOME/config/*_ms_*.config 2>/dev/null)
  for cfgfile in $temsconfig
  do
    cfgfile=${cfgfile%.config}
    name=${cfgfile##*_ms_}
    [[ -d "$CANDLEHOME/tables/$name" ]] && print $name && break
  done
}

# $1 - source file name
# return 1 if argument is product file otherwise return 0
function is_product_file
{
  srcfile=$1

  [[ $srcfile = tables/cicatrsq/rb/* ]] && return 1 
  [[ $srcfile = /tables/cicatrsq/rb/* ]] && return 1
  [[ $srcfile = ${CANDLEHOME}/tables/cicatrsq/rb/* ]] && return 1

  [[ $srcfile = */k${PC}*.atr ]] && return 1
  [[ $srcfile = */k${PC}*.cat ]] && return 1

  [[ $srcfile = */k${PC}atr00 ]] && return 1
  [[ $srcfile = */k${PC}cat00 ]] && return 1

  [[ $srcfile = */k${PC}.baroc ]] && return 1
  [[ $srcfile = */k${PC}.map ]] && return 1

  [[ $srcfile = */k${PC}util.bin ]] && return 1
  [[ $srcfile = */k${PC}oslc.xml ]] && return 1  

  # for product cp copy all files
  [[ ${PC} = cp ]] && return 1

  return 0
}

# appends the "installDate" record to the specified version file
# $1 - version file name
function append_install_date_record
{
  typeset verfile=$1

  [ -z "$( whoami 2>/dev/null )" ] && alias whoami='id | cut -d"(" -f2 | cut -d")" -f1'
  typeset userName=$( whoami )

  typeset currentDate=$(get_epoch);                                     #IV65374

  # the "installDate" record has the following format:
  # installDate = file_owner|date_installed|, where date_installed is in milliseconds
  typeset installDate="installDate = $userName|${currentDate}000|"

  printf "\n%s\n" "$installDate" 2>/dev/null >> "$verfile"
}

function get_epoch { #--------------------------------------------------#IV65374
   # Return epoch time (seconds since 1970-01-01 00:00:00 UTC):         #IV65374
   # - Solaris doesn't provide 'date +%s' and its awk is old. So, we    #IV65374
   #   try nawk and fall back to truss/awk of the simple date command.  #IV65374
   # - HP-UX doesn't proclaim to provide 'date +%s'. Its awk is new, so #IV65374
   #   we try that first and fall back to 'date +%s' (which sometimes   #IV65374
   #   works).                                                          #IV65374
   # - AIX and Linux provide 'date +%s' and it seems to work            #IV65374
   #   reliably. Both provide a new awk, so we have that as a fallback  #IV65374
   #   though we don't expect to ever use it.                           #IV65374
   #--------------------------------------------------------------------#IV65374
   typeset epochtime;                                                   #IV65374
   case "${ostype}" in                                                  #IV65374
      "SunOS")                                                          #IV65374
         epochtime="0000000000";                                        #IV65374
         if [[ ! -z "$(whence nawk 2>/dev/null)" ]]; then               #IV65374
            epochtime=$(nawk 'BEGIN {srand(); printf("%d\n", srand())}'); #IV65374
         elif [[ ! -z "$(whence truss 2>/dev/null)" ]]; then            #IV65374
            epochtime=$(truss date 2>&1 | awk '/^time/ {print $NF}' );  #IV65374
         fi;                                                            #IV65374
      ;;                                                                #IV65374
      "HP-UX")                                                          #IV65374
         epochtime=$(awk 'BEGIN {srand(); printf("%d\n", srand())}');   #IV65374
         [ -z "${epochtime}" ] && epochtime=$( date +%s );              #IV65374
      ;;                                                                #IV65374
      *)                                                                #IV65374
         epochtime=$(date +%s);                                         #IV65374
         [ -z "${epochtime}" ] && epochtime=$(awk 'BEGIN {srand(); printf("%d\n", srand())}'); #IV65374
   esac;                                                                #IV65374
   print "${epochtime}";                                                #IV65374
}                                                                       #IV65374
                                                                        #IV65374
### main code starts here

CANDLEHOME=""
PC=""
SDADIR=""
SDATYPE=""
FORCEOPT=""
BACKUPDIR=""
RESTOREDIR=""
LOGFILE="installsdsupport_$(date +%m%d%y%H%M%S).log"   
   
while getopts ":h:d:p:t:r:b:fx" OPTS
do case $OPTS in
    b) BACKUPDIR=$OPTARG     
       ;;
    d) SDADIR=$OPTARG
       ;;
    f) FORCEOPT="-f"        
       ;;
    h) CANDLEHOME=$OPTARG
       ;;
    p) PC=$OPTARG
       ;;
    r) RESTOREDIR=$OPTARG
       ;;
    t) SDATYPE=$OPTARG
       ;;
    x) DEBUG=yes
       ;;
    ?) print_usage "$0"
       ;;
  esac
done

[[ $# -lt 8 ]] && print_usage "$0"

# check if candlehome directory exists
[[ -d "$CANDLEHOME" ]] || fail_exit "Directory $CANDLEHOME does not exist" $RC_PARAM_FAIL

# check if candlehome directory is valid
[[ -f "$CANDLEHOME/registry/cienv.ver" ]] || fail_exit "Invalid ITM installation directory" $RC_PARAM_FAIL

LOG="$CANDLEHOME/logs/$LOGFILE"

# check if product code length is 2
[[ ${#PC} -eq 2 ]] || fail_exit "Invalid product code"

# check if sda support type is valid
case "$SDATYPE" in
  tms) serverpc=ms;;
  tps) serverpc=cq;;
  tpw) serverpc=cw;;
  *) fail_exit "Invalid support type $SDATYPE - use tms for tems or tps for teps and tpw for tep browser" $RC_PARAM_FAIL;;
esac

ostype=$(uname -s)

# short part of restore functionality
if [[ -n "$RESTOREDIR" ]];
then
  loginfo "## Restoring backup - $0 $*"
  [[ -d "$RESTOREDIR" ]] || fail_exit "Invalid backup directory $RESTOREDIR" $RC_PARAM_FAIL
  restorefile=$RESTOREDIR/${PC}${SDATYPE}_support.tar  
  [[ -f "$restorefile" ]] || fail_exit "Unable to find backup file $restorefile"
  loginfo "Executing command 'tar tf $restorefile' to verify backup file"
  eval "tar tf $restorefile >> $LOG 2>&1"
  [[ $? -eq 0 ]] || fail_exit "Invalid backup file $restorefile"
  loginfo "Extracting files from backup file $restorefile"
  [[ "$ostype" = "Linux" ]] && taropt="P"
  eval "tar xvf$taropt $restorefile >> $LOG 2>&1"
  [[ $? -eq 0 ]] || fail_exit "Unable to extract files from archive $restorefile"
  loginfo "Files restored from backup archive successfully"
  if [[ "$serverpc" = "ms" ]];
  then
    loginfo "Copying files to TEMS tables directories"
    temsname=$(gettemsname)
    loginfo "TEMS name - $temsname"      
    [[ -z "$temsname" ]] && fail_exit "Unable to determine TEMS name"
    for file in `tar tf$taropt $restorefile 2>/dev/null`
    do
      if [[ -n $( print $file | grep /tables/cicatrsq/ | grep -v /SQLLIB/ ) ]];
      then
        is_product_file ${file}
        if [[ $? -eq 1 ]] then
          filedst=$( print $file | sed 's/cicatrsq/'${temsname}'/' )        
          mkdir -p $(dirname $filedst) 2>/dev/null        
          loginfo "Copying file $file to $filedst"
          cp -f $file $filedst 2>/dev/null
          [[ $? -eq 0 ]] || fail_exit_cleanup $tmpdir "Unable to copy file $file to $filedst" $RC_COPY_TEMS_FAIL
        else 
          loginfo "Skipping file $file"
        fi
      fi
    done
  fi
  print "$RC_OK,Files restored from backup archive successfully,$LOGFILE"
  exit $RC_OK
fi

# main code for support installation

loginfo "## Starting support installation - $0 $*"

sdatypeup=$(print $SDATYPE | tr '[a-z]' '[A-Z]')
pcup=$(print $PC | tr '[a-z]' '[A-Z]')

sdamanifest=K${pcup}MSMAN.txt
sdamanifestpath=$SDADIR/$sdamanifest

sdafile=K${pcup}JS${sdatypeup}.jar
sdafilepath=$SDADIR/$sdafile

# check if sda package exists in sda directory
[[ -f "$sdafilepath" ]] || fail_exit "Unable to find file $sdafilepath"

# check if sda support directory is valid
[[ -f "$sdamanifestpath" ]] || fail_exit "Unable to find manifest file $sdamanifestpath" $RC_MANIFEST_FAIL

# set umask if secure mode is enabled
SecureFile=$CANDLEHOME/registry/SecureMode
export SecureFile

[[ -r "$SecureFile" ]] && commongroup=$(cat $SecureFile | cut -d: -f3)
if [[ -n "$commongroup" && "$commongroup" != "@none@" ]];
then
  umask 002 # tighten permissions for common group
else
  umask 022 # tighten permissions in general
fi

# get operating system arch
ArchTbl=$CANDLEHOME/registry/archdsc.tbl
DynShl=$CANDLEHOME/bin/dynarch.shl
. $DynShl $ArchTbl 2>/dev/null
os=$(print $cArch|cut -c1-2)

loginfo "System architecture - $cArch"

# check if required server component is installed
[[ -z $(ls $CANDLEHOME/registry/$serverpc$os*.ver 2>/dev/null) ]] && fail_exit "Required server component $serverpc is not installed"

verfile=$PC$SDATYPE.ver

# check if this is upgrade or pristine installation
if [[ -f "$CANDLEHOME/registry/$verfile" ]]
then
 isupgrade=yes
fi

if [[ -n "$FORCEOPT" || -z "$isupgrade" ]];
then
  # always install when there support is not installer or -f flag was given
  install=yes
else
  # compare versions
  curVRMF=$(grep "^VRMF *=" $CANDLEHOME/registry/$verfile | cut -d"=" -f2 | tr -d '\r' )
  curVer=${curVRMF##+( )}
  loginfo "Support version from $CANDLEHOME/registry/$verfile - $curVer"
  newVRMF=$(grep "^${SDATYPE}_package_vrmf *=" $sdamanifestpath | cut -d"=" -f2 | tr -d '\r' )
  newVer=${newVRMF##+( )}
  loginfo "Support version from $sdamanifestpath - $newVer"
  [[ newVer -gt curVer ]] && install=yes      
fi

if [[ -n "$install" ]];
then
  # start support installation
  loginfo "Installing $SDATYPE support for $PC"

  # check if backup directory exists
  if [[ -n "$BACKUPDIR" && ! -d "$BACKUPDIR" ]];
  then
    # create backup directory if it doesn't exist
    mkdir -p $BACKUPDIR > /dev/null 2>&1
    [[ $? -eq 0 ]] || fail_exit "Unable to create backup directory $BACKUPDIR"
  fi

  tmpdir=$CANDLEHOME/tmp/$sdafile.$$
  listfile=$PC${SDATYPE}_support_unix.txt

  # get server components binarch
  serverbinarch=$(getbinarch $serverpc $os)
  loginfo "$serverpc binarch = $serverbinarch"
  [[ -z "$serverbinarch" ]] && fail_exit "Unable to determine server component $serverpc architecture"
  
  case "$serverpc" in
    "cq")# if server component is teps, get eclipse help server binarch    
         # get ehs binarch from cq .ver file
         kfbinarch=$(grep "^preReqALL = kf" $CANDLEHOME/registry/cq$serverbinarch.ver 2>/dev/null | cut -d"=" -f2 | cut -d "|" -f1 | cut -c 4-)
         loginfo "EHS binarch - $kfbinarch"
         [[ -z "$kfbinarch" ]] && fail_exit "Unable to determine Eclipse Help Server architecture";;
    "ms")# if server component is tems, get tems name
         temsname=$(gettemsname)
         loginfo "TEMS name - $temsname"      
         [[ -z "$temsname" ]] && fail_exit "Unable to determine TEMS name";;
  esac         
  
  # create temporary directory to unpack archive
  loginfo "Creating temporary directory $tmpdir"
  mkdir -p $tmpdir 2>/dev/null
  [[ -w "$tmpdir" ]] || fail_exit "Unable to write to $tmpdir directory"

  # find java executable
  setJavaEnv $cArch
  [[ -x "$JAVA_HOME/bin/java" ]] || fail_exit "Unable to find JRE in $JAVA_HOME required for support installation"

  # extract support package to temporary directory
  loginfo "Extracting archive $sdafilepath to temporary directory $tmpdir"     
  classpath=$CANDLEHOME/classes/install_common.jar:$CANDLEHOME/classes/InstallRAS.jar:$CANDLEHOME/classes/jlog.jar    
  $JAVA_HOME/bin/java -cp $classpath com.ibm.tivoli.monitoring.install.utils.ZipUtil $sdafilepath $tmpdir >> $LOG 2>&1
  rc=$?
  [[ $rc -eq 0 ]] || fail_exit_cleanup $tmpdir "Unable to extract archive $sdafilepath to directory $tmpdir - return code $rc"
 
  loginfo "List of files in temporary directory $tmpdir"
        
  [[ -n "$DEBUG" ]] && ls -l $tmpdir 2>/dev/null
  ls -l $tmpdir >> $LOG

  # if exists, append "installDate" record to the version file
  if [[ -f $tmpdir/$verfile ]];
  then
    loginfo "Appending \"installDate\" record to the version file - \"$tmpdir/$verfile\""
    append_install_date_record "$tmpdir/$verfile"
  fi

  [[ -f $tmpdir/$listfile ]] || fail_exit_cleanup $tmpdir "Unable to find file $listfile"
  
  # read files list
  loginfo "Reading files list from $tmpdir/$listfile"
  while read line
  do
    # get rid of windows end of line character
    line=$(print $line | grep -v ^package_ | tr -d '\r' )
    [[ -z "$line" ]] && continue
    loginfo "> $line"
    files="$files $(print $line | cut -d"=" -f1)"
    dirs="$dirs $(print $line | cut -d"=" -f2)"    
  done < "$tmpdir/$listfile"
  
  set -A sdafiles $files
  set -A sdafilesdst $dirs

  unset dirs

  [[ ${#sdafiles[*]} -eq 0 ]] && fail_exit_cleanup $tmpdir "Unable to get list of files to copy from $tmpdir/$listfile" $RC_COPY_FAIL

  # add binarch to destination directory if needed  
  for filedst in ${sdafilesdst[@]}
  do
    # add server component binarch
    [[ -n "$(print $filedst | grep ^$serverpc/)" ]] && filedst="$serverbinarch/$filedst"
    # if teps is installed, add also binarch for kf component
    [[ "$serverpc" = "cq" ]] && [[ -n "$(print $filedst | grep ^kf/)" ]] && filedst="$kfbinarch/$filedst"
    dirs="$dirs $filedst"
  done

  set -A sdafilesdst $dirs

  # verify that existing files can be overwritten
  for file in ${sdafilesdst[@]}
  do        
    filedst=$CANDLEHOME/$file
    [[ -f "$filedst" ]] && [[ ! -w "$filedst" ]] && wfiles="$wfiles $filedst"
  done  
      
  [[ -n "$wfiles" ]] && fail_exit_cleanup $tmpdir "Not enough permissions to overwrite file(s) -  $wfiles"    

  # create backup file
  if [[ -n "$BACKUPDIR" ]];
  then
    backupfile=$BACKUPDIR/${PC}${SDATYPE}_support.tar
    loginfo "Creating backup file $backupfile"
    # remove existing directory
    rm -f $backupfile > /dev/null 2>&1
    placeholder=$CANDLEHOME/tmp/placeholder.${SDATYPE}
    print "Backup of $PC support for $serverpc made on $(date)" > $placeholder 2>/dev/null
    # use option P with tar to save absolute path
    [[ "$ostype" = "Linux" ]] && taropt="P"
    # create backup file
    tar cf$taropt $backupfile $placeholder > /dev/null 2>&1
    rc=$?
    rm -f $placeholder 2>/dev/null
    [[ $rc -eq 0 ]] || fail_exit_cleanup $tmpdir "Unable to create backup file $backupfile"
    # add files to backup archive
    for file in ${sdafilesdst[@]}
    do        
      filedst=$CANDLEHOME/$file
      # if destination file exists add it to backup file
      if [[ -f "$filedst" ]];
      then
        loginfo "Adding file $filedst to archive $backupfile"
        tar rf$taropt $backupfile $filedst >/dev/null 2>&1
        [[ $? -eq 0 ]] || fail_exit_cleanup $tmpdir "Unable to make backup of file $filedst"
      fi
    done
  fi

  # create backup of xml files in TEPS tmsdla directory
  if [[ "$serverpc" = "cq" ]];
  then
    for file in `ls $CANDLEHOME/$serverbinarch/cq/bin/tmsdla/k${PC}*.xml 2>/dev/null`
    do
      loginfo "Copying file $file to $file.bak"
      cp -f $file $file.bak 2>/dev/null
      [[ $? -eq 0 ]] || loginfo "WARNING: Unable to make backup of $file"
    done
  fi

  # copy files
  typeset -i i=0
  while [[ $i -lt ${#sdafiles[*]} ]]
  do
    file=$tmpdir/${sdafiles[$i]}
    filedst=$CANDLEHOME/${sdafilesdst[$i]}
    # create destination directory if doesn't exist
    mkdir -p $(dirname $filedst) 2>/dev/null 
    # copy file
    loginfo "Copying file $file to $filedst"
    cp -f $file $filedst 2>/dev/null
    [[ $? -eq 0 ]] || fail_exit_cleanup $tmpdir "Unable to copy file $file to $filedst" $RC_COPY_FAIL
    i=$i+1 
  done

  # remove temporary directory
  loginfo "Removing temporary directory $tmpdir"
  rm -rf $tmpdir 2>/dev/null  
    
  # if server component is tems, copy selected files to subdirs under CANDLEHOME/tables/<TEMSNAME> directory
  if [[ "$serverpc" = "ms" ]];
  then
    typeset -i i=0
    while [[ $i -lt ${#sdafilesdst[*]} ]]
    do
      file=$CANDLEHOME/${sdafilesdst[$i]}
      if [[ -n $( print $file | grep /tables/cicatrsq/ | grep -v /SQLLIB/ ) ]];
      then
        is_product_file ${sdafilesdst[$i]}
        if [[ $? -eq 1 ]] then
          filedst=$( print $file | sed 's/cicatrsq/'${temsname}'/' )        
          mkdir -p $(dirname $filedst) 2>/dev/null        
          loginfo "Copying file $file to $filedst"
          cp -f $file $filedst 2>/dev/null
          [[ $? -eq 0 ]] || fail_exit_cleanup $tmpdir "Unable to copy file $file to $filedst" $RC_COPY_TEMS_FAIL
        else 
          loginfo "Skipping file $file"
        fi
      fi
      i=$i+1
    done
  fi  
  
else
  loginfo "Newest or same version already installed"  
  print "$RC_OLDER,Newest or same version already installed,$LOGFILE"
  exit $RC_OLDER
fi

loginfo "$PC support installation for $SDATYPE completed successfully"
print "$RC_OK,$PC support installation for $SDATYPE completed successfully,$LOGFILE"
exit $RC_OK
