#!/bin/ksh
#=======================================================================#
# Licensed Materials - Property of IBM                                  #
# 5724-C04                                                              #
# Copyright IBM Corporation 2015 All Rights Reserved                    #
#-----------------------------------------------------------------------#
# itmCANDLEDATA.sh - This conversion utility moves local data from      #
#    CANDLEHOME to a separate user-defined file system known as         #
#    CANDLEDATA. To facilitate normal ITM operation, symbolic links     #
#    from CANDLEHOME to CANDLEDATA are generated. In theory, only       #
#    install.sh and itmCANDLEDATA.sh are fully aware of the split. In   #
#    practice, a few other programs (uninstall.sh, pdcollect. etc.) are #
#    also aware of CANDLEDATA. However, the majority of ITM simply      #
#    accesses local data through the symbolic links.                    #
#                                                                       #
#    We are called at three different points in the life of the         #
#    product:                                                           #
#    1) At the beginning of installation (before CANDLEHOME is          #
#       populated:                                                      #
#       - Create new subdirectories in CANDLEDATA.                      #
#       - Link CANDLEHOME to CANDLEDATA.                                #
#    2) At component start time (to convert that component) or          #
#       for conversion of a specific component or components:           #
#       - Stop the component.                                           #
#       - Set up agent-specific links.                                  #
#       - Move data from CANDLEHOME to CANDLEDATA.                      #
#       - Restart components that were stopped.                         #
#    3) Conversion of a complete existing system:                       #
#       - Stop all components.                                          #
#       - Create new subdirectories in CANDLEHOME.                      #
#       - Move data from CANDLEHOME to CANDLEDATA.                      #
#       - Link CANDLEHOME to CANDLEDATA.                                #
#       - Set up agent-specific links.                                  #
#       - Restart components that were stopped.                         #
#                                                                       #
#-----------------------------------------------------------------------#
# SYNTAX: itmCANDLEDATA.sh                                              #
#            [-h CANDLEHOME] [-l CANDLEDATA]                            #
#            [-b yes|no] [-f no|yes] [-p pc|*] [-r no|yes]              #
#                                                                       #
#    Where:                                                             #
#       -b: indicates build mode. If yes (default), the conversion      #
#           utility builds the CANDLEDATA directory, moves data to it   #
#           from CANDLEHOME, and links CANDLEHOME to it. If no, the     #
#           conversion utility assumes the CANDLEDATA build is          #
#           complete.                                                   #
#       -h: specifies the ITM installation directory.                   #
#       -l: specifies the ITM local data directory.                     #
#       -f: indicates force mode. If no (default), conversion is        #
#           skipped for a given component if it has already been done   #
#           for that component. If yes, conversion is performed even if #
#           it has already been done for the component.                 #
#       -p: indicates conversion for a specific product code. Multiple  #
#           product codes may be specified by separating them with a    #
#           blank space. An asterisk (*) indicates all product          #
#           components are to be converted. Note that this is intended  #
#           for optimization. It is not for picking and choosing which  #
#           components are converted. For example, if a new component   #
#           has just been installed to an existing environment, this    #
#           tells the conversion utility to process just that new one   #
#           vs. all of them.                                            #
#       -r:  indicates restart mode, which controls whether components  #
#            are automatically stopped and restarted. If no (default),  #
#            then affected components must be stopped before the        #
#            conversion utility is run. If yes, the conversion utility  #
#            automatically stops and restarts components as needed.     #
#                                                                       #
# RETURN CODES:                                                         #
#    0 - Success: CANDLEDATA built or migrated.                         #
#    1 - Success: CANDLEDATA not defined, no build/migration needed.    #
#   10 - Syntax error.                                                  #
#   11 - Error creating directory/file.                                 #
#   12 - Error removing directory.                                      #
#   13 - Error moving data.                                             #
#   14 - Error creating symbolic link.                                  #
#   15 - itmCANDLEDATA_pc.sh failed.                                    #
#   16 - itmCANDLEDATA_xx.sh failed.                                    #
#   17 - Components running but '-r yes' not specified.                 #
#   18 - Error checking for running components.                         #
#   19 - Error stopping components.                                     #
#   20 - Error restarting components.                                   #
#   21 - Monitoring server or portal server not supported.              #
#=======================================================================#

function usage { #------------------------------------------------------#
   # Display syntax.                                                    #
   #--------------------------------------------------------------------#
   print -u2 "Usage: ${pgm} ";
   print -u2 "   [-h CANDLEHOME] [-l CANDLEDATA] [-b yes|no] [-f no|yes] [-p pc|*] [-r no|yes]";
}                                           # (usage())                 #

function resolveDir { #-------------------------------------------------#
   # Perform directory canonicalization. In other words, the input      #
   # directory string is converted to a fully qualified name (no        #
   # wildcards, "..", ".", and so on. For example, "/tmp/.." resolves   #
   # to "/". The input directory string must refer to a directory that  #
   # exists.                                                            #
   #                                                                    #
   # AIX and Linux provide the "readlink" command to perform this       #
   # function. If readlink does not exist, we use "cd -P". This         #
   # approach changes the directory *and* sets PWD to be the            #
   # canonicalized directory name. So, we switch to the directory, grab #
   # its name, and switch back to the original working directory.       #
   #                                                                    #
   # Solaris has some sort of buffering issue when the cd shell command #
   # sets $PWD and when the pwd shell command returns the value. In     #
   # in some cases, one or both of these leave extra garbage on the end #
   # of the output value. And of course, this creates problems for us   #
   # later when we try to use it. We get around the problem by using    #
   # the /usr/bin/pwd binary, which always returns the correct value    #
   # without corruption.                                                #
   #--------------------------------------------------------------------#
   eval $(itmsh_trace_func "${0}" "${@}");  # enable function tracing   #
   typeset result;                          # localize output variable  #
   result=$(readlink -f ${1} 2> /dev/null); # try readlink              #
   if [[ ${?} -ne 0 ]]; then                # if no readlink or failed  #
     cd -P ${1} > /dev/null 2>&1;           # use cd to change/resolve  #
     if [[ ${?} -ne 0 ]]; then              # if cd failed...           #
       result="";                           # indicate same to caller   #
     else                                   # get result from cd        #
       if [[ "${thisOS}" = "SunOS" ]]; then # $PWD/pwd shell broken...  #
         result=$(/usr/bin/pwd);            # ...use pwd binary         #
       else                                 # $PWD is ok...             #
         result="${PWD}";                   # ...use its value          #
       fi;                                  # (elif(thisOS="sol"))      #
       cd ${OLDPWD} > /dev/null 2>&1;       # restore working directory #
     fi;                                    # (else($?!=0))             #
   fi;                                      # (if($?!=0))               #
   print "${result}";                       # pass result to caller     #
}                                           # (resolveDir())            #

function do_BUILD { #---------------------------------------------------#
   # Set up CANDLEDATA subdirectories, move any existing CANDLEHOME     #
   # data to them, and link CANDLEHOME subdirectories to CANDLEDATA.    #
   #--------------------------------------------------------------------#
   eval $(itmsh_trace_func "${0}" "${@}");  # enable function tracing   #
   typeset myrc=0;                          # my return code            #

   typeset msg;                             # message to user           #
   typeset x;                               # temporary variable        #
   typeset rc;                              # misc return codes         #
   typeset cmd;                             # command strings           #
   typeset datadir;                         # CANDLEDATA/linkdir        #
   typeset homedir;                         # CANDLEHOME/linkdir        #
   typeset homesource;                      # CANDLEHOME/linkdir/source #
   typeset linkdirs="auditlogs config hist kt1v3depot localconfig logs";#IV77775
           linkdirs="${linkdirs} patchlogs psit tables";                #213940
           linkdirs="${linkdirs} tmaitm6/agentdepot tmp";               #213940
           linkdirs="${linkdirs} ATTRLIB EIFLIB InstallITM";

   #--------------------------------------------------------------------#
   # Create CANDLEDATA if if doesn't already exist.                     #
   #--------------------------------------------------------------------#
   if [[ ! -a "${CANDLEDATA}" ]]; then      # does CANDLEDATA exist?    #
      x=$(mkdir -p "${CANDLEDATA}" 2>&1);   # create it...              #
      rc=${?};
      if [[ ${rc} -ne 0 ]]; then
         #$NLS-E$ "An error occurred while creating $1: rc=$2.\n...xxx"
         msg=$(nls_replace 'KCI1423E' "\"${CANDLEDATA}\"" "${rc}");
         msg="${msg}\n...${x}";
         print -u2 "${pgm}: ${msg}";
         myrc=11;                           # error creating directory  #
      fi;                                   # (if(rc!=0))               #
   fi;                                      # (if(! -a CANDLEDATA))     #
   
   #--------------------------------------------------------------------#
   # Link CANDLEHOME to CANDLEDATA for a back-reference.                #
   #--------------------------------------------------------------------#
   if [[ ${myrc} -eq 0 ]] && [[ ! -L "${CANDLEDATA}/CANDLEHOME" ]]; then
      CD2CHmark="${CANDLEDATA}/CANDLEHOME";
      x=$(ln -fs "${CANDLEHOME}" "${CD2CHmark}" 2>&1);
      rc=${?};
      if [[ ${rc} -ne 0 ]]; then
         #$NLS-E$ "An error occurred while linking $1 to $2: rc=$3.\n...xxx"
         msg=$(nls_replace 'KCI1424E' "\"${CD2CHmark}\"" "\"${CANDLEHOME}\"" "${rc}");
         msg="${msg}\n...${x}";
         print -u2 "${pgm}: ${msg}";
         myrc=14;                           # error linking directory   #
      fi;                                   # (if(rc!=0))               #
   fi;                                      # (if(myrc==0))             #
   
   #--------------------------------------------------------------------#
   # Make sure CANDLEHOME/registry exists and link CANDLEDATA to it.    #
   #--------------------------------------------------------------------#
   if [[ ${myrc} -eq 0 ]] && [[ ! -L "${CH2CDmark}" ]]; then
      if [[ ! -a "${CHregistry}" ]]; then   # if no CANDLEHOME/registry #
         x=$(mkdir "${CHregistry}" 2>&1);   # create it                 #
         rc=${?};
         if [[ ${rc} -ne 0 ]]; then
            #$NLS-E$ "An error occurred while creating $1: rc=$2.\n...xxx"
            msg=$(nls_replace 'KCI1423E' "\"${CHregistry}\"" "${rc}");
            msg="${msg}\n...${x}";
            print -u2 "${pgm}: ${msg}";
            myrc=11;                        # error creating directory  #
         fi;                                # (if(rc!=0))               #
      fi;                                   # (if(! -a CANDLEHOME...))  #

      if [[ ${myrc} -eq 0 ]]; then
         x=$(ln -fs "${CANDLEDATA}" "${CH2CDmark}" 2>&1);
         rc=${?};
         if [[ ${rc} -ne 0 ]]; then
            #$NLS-E$ "An error occurred while linking $1 to $2: rc=$3.\n...xxx"
            msg=$(nls_replace 'KCI1424E' "\"${CH2CDmark}\"" "\"${CANDLEDATA}\"" "${rc}");
            msg="${msg}\n...${x}";
            print -u2 "${pgm}: ${msg}";
            myrc=14;                        # error linking directory   #
         fi;                                # (if(rc!=0))               #
      fi;                                   # (if(myrc==0))             #
   fi;                                      # (if(myrc==0))             #

   #--------------------------------------------------------------------#
   # Create any missing CANDLEHOME and CANDLEDATA subdirectories. For   #
   # CANDLEHOME, these will be replaced with links. However, making     #
   # them ahead of time here ensures parent directories exist (like     #
   # CANDLEHOME/tmaitm6 for CANDLEHOME/tmaitm6/agentdepot).             #
   #--------------------------------------------------------------------#
   if [[ ${myrc} -eq 0 ]]; then
      for datadir in ${linkdirs}; do        # for each subdirectory.... #
         homedir="${CANDLEHOME}/${datadir}";     # localize for home    #
         datadir="${CANDLEDATA}/${datadir}";     # localize for data    #
         
         if [[ ! -a "${datadir}" ]]; then
            x=$(mkdir -p "${datadir}" 2>&1);     # attempt to create    #
            rc=${?};
            if [[ ${rc} -ne 0 ]]; then
               #$NLS-E$ "An error occurred while creating $1: rc=$2.\n...xxx"
               msg=$(nls_replace 'KCI1423E' "\"${datadir}\"" "${rc}");
               msg="${msg}\n...${x}";
               print -u2 "${pgm}: ${msg}";
               myrc=11;                     # error creating directory  #
               break;                       # stop looping              #
            fi;                             # (if(rc!=0))               #
         fi;                                # (if(! -a datadir))        #
         
         if [[ ! -a "${homedir}" ]]; then
            x=$(mkdir -p "${homedir}" 2>&1);     # attempt to create    #
            rc=${?};
            if [[ ${rc} -ne 0 ]]; then
               #$NLS-E$ "An error occurred while creating $1: rc=$2.\n...xxx"
               msg=$(nls_replace 'KCI1423E' "\"${homedir}\"" "${rc}");
               msg="${msg}\n...${x}";
               print -u2 "${pgm}: ${msg}";
               myrc=11;                     # error creating directory  #
               break;                       # stop looping              #
            fi;                             # (if(rc!=0))               #
         fi;                                # (if(! -a datadir))        #
      done;                                 # (for(linkdir...))         #
   fi;                                      # (if(main_rc=0))           #

   #--------------------------------------------------------------------#
   # Move any data from CANDLEHOME to CANDLEDATA, and then link         #
   # CANDLEHOME subdirectories to CANDLEDATA for the future. Note the   #
   # move is not a simple mv command. We don't know if there is any     #
   # content, and a single mv command won't handle both hidden and      #
   # and regular files. What we do instead is roughly the equivalent of #
   # "find... -exec mv". Since the platforms we support don't all have  #
   # a full GNU find command, we use find+tail+grep to simulate the     #
   # mindepth and maxdepth options we'd normally use. Basically, this   #
   # gives a list of everything in the top of the directory... not the  #
   # directory itself and no descending into subdirectories.            #
   #--------------------------------------------------------------------#
   if [[ ${myrc} -eq 0 ]]; then
      for datadir in ${linkdirs}; do        # for each subdirectory.... #
         homedir="${CANDLEHOME}/${datadir}";     # localize for home    #
         datadir="${CANDLEDATA}/${datadir}";     # localize for data    #

         if [[ -a "${homedir}" ]] && [[ ! -L "${homedir}" ]]; then
            cmd="find \"${homedir}\" 2> /dev/null";   # list homedir    #
            cmd="${cmd} | ${tailCMD} +2";   # simulate -mindepth 1 and  #
            cmd="${cmd} | grep -v \"^${homedir}/.*/\""; # -maxdepth 1   #
            eval ${cmd} | while read homesource; do
               x=$(mv "${homesource}" "${datadir}" 2>&1); 
               rc=${?};
               if [[ ${rc} -ne 0 ]]; then
                  #$NLS-E$ "An error occurred while moving $1 to $2: rc=$3.\n...xxx"
                  msg=$(nls_replace 'KCI1425E' "\"${homesource}\"" "\"${datadir}\"" "${rc}");
                  msg="${msg}\n...${x}";
                  print -u2 "${pgm}: ${msg}";
                  myrc=13;                  # error creating directory  #
                  break;                    # stop looping              #
               fi;                          # (else(rc!=0))             #
            done;                           # (while(read homesource))  #
         fi;                                # (if(-a/!-L homedir))      #
         
         if [[ ${myrc} -eq 0 ]]; then
            if [[ ! -L "${homedir}" ]]; then 
               x=$(rmdir "${homedir}" 2>&1);     # remove now-empty dir #
               rc=${?};
               if [[ ${rc} -ne 0 ]]; then
                  #$NLS-E$ "An error occurred while removing $1: rc=$2.\n...xxx"
                  msg=$(nls_replace 'KCI1426E' "\"${homedir}\"" "${rc}");
                  msg="${msg}\n...${x}";
                  print -u2 "${pgm}: ${msg}";
                  myrc=12;                  # error removing directory  #
                  break;                    # stop looping              #
               else
                  x=$(ln -fs "${datadir}" "${homedir}" 2>&1);
                  rc=${?};
                  if [[ ${?} -ne 0 ]]; then
                     #$NLS-E$ "An error occurred while linking $1 to $2: rc=$3.\n...xxx"
                     msg=$(nls_replace 'KCI1424E' "\"${homedir}\"" "\"${datadir}\"" "${rc}");
                     msg="${msg}\n...${x}";
                     print -u2 "${pgm}: ${msg}";
                     myrc=14;               # error linking directory   #
                     break;                 # stop looping              #
                  fi;                       # (if(rc!=0))               #
               fi;                          # (if(rc!=0))               #
            fi;                             # (if(!-L homedir))         #
         fi;                                # (if(myrc==0))             #
      done;                                 # (for(linkdir...))         #
   fi;                                      # (if(myrc==0))             #

   return ${myrc};                          # my rc to caller           #
}                                           # (do_BUILD())              #

function do_MIGRATE { #-------------------------------------------------#
   # Modify a given component (${1}) to use CANDLEDATA.                 #
   #--------------------------------------------------------------------#
   eval $(itmsh_trace_func "${0}" "${@}");  # enable function tracing   #
   typeset myrc=0;                          # my return code            #

   typeset ipc="${1}";                      # product code to convert   #

   typeset need_xx=1;                       # 1=need std process, 0=not #
   typeset msg;                             # message to user           #
   typeset x;                               # temporary variable        #
   typeset rc;                              # misc return codes         #

   #--------------------------------------------------------------------#
   # If a custom itmCANDLEDATA_pc.sh exists, run it.                    #
   #--------------------------------------------------------------------#
   typeset pc_script="${bindir}/${pgmfn}_${ipc}.sh";
   
   if [[ -a "${pc_script}" ]]; then
      "${pc_script}";                                                   #213940
      rc=${?};
      if [[ ${rc} -eq 0 ]]; then            # success...                #
         need_xx=0;                         # ...but skip std process   #
      elif [[ ${rc} -eq 1 ]]; then          # also success...           #
         need_xx=1;                         # ...but need std process   #
      else
         #$NLS-E$ "An error occurred while running $1 to process product code $2: rc=$3.\n...xxx"
         msg=$(nls_replace 'KCI1427E' "\"${pc_script}\"" "\"${ipc}\"" "${rc}");
         print -u2 "${pgm}: ${msg}";
         myrc=15;
      fi;                                   # (else(rc...))             #
   else                                     # no pc-specific process... #
      need_xx=1;                            # ...run std process        #
   fi;                                      # (else(-z pc_script))      #

   #--------------------------------------------------------------------#
   # If we need the standard itmCANDLEDATA_xx.sh process, run it.       #
   #--------------------------------------------------------------------#
   if [[ ${myrc} -eq 0 ]]; then
      if [[ ${need_xx} -eq 1 ]]; then
         pc_script="${bindir}/${pgmfn}_xx.sh";
         "${pc_script}" "${ipc}";                                       #213940
         rc=${?};
         if [[ ${rc} -ne 0 ]]; then
            #$NLS-E$ "An error occurred while running $1 to process product code $2: rc=$3.\n...xxx"
            msg=$(nls_replace 'KCI1427E' "\"${pc_script}\"" "\"${ipc}\"" "${rc}");
            print -u2 "${pgm}: ${msg}";
            myrc=16;
         fi;                                # (if(rc!=0))               #
      fi;                                   # (if(need_xx==1))          #
   fi;                                      # (if(myrc==0))             #

   #--------------------------------------------------------------------#
   # Set CANDLEDATA/registry/CANDLEDATA_pc to show we're done.          #
   #--------------------------------------------------------------------#
   if [[ ${myrc} -eq 0 ]]; then
      typeset markerfile="${CANDLEHOME}/config/.ConfigData/CANDLEDATA_${ipc}";
      x=$(touch "${markerfile}" 2>&1);
      rc=${?};
      if [[ ${rc} -ne 0 ]]; then
         #$NLS-E$ "An error occurred while creating $1: rc=$2.\n...xxx"
         msg=$(nls_replace 'KCI1423E' "\"${markerfile}\"" "${rc}");
         msg="${msg}\n...${x}";
         print -u2 "${pgm}: ${msg}";
         myrc=11;
      fi;                                   # (if(rc!=0))               #
   fi;                                      # (if(myrc==0))             #

   #--------------------------------------------------------------------#
   # Check our overrides and those from the customer and warn if there  #
   # are conflicts.                                                     #
   #--------------------------------------------------------------------#
   if [[ ${myrc} -eq 0 ]]; then
      typeset envfile_ibm="${CANDLEHOME}/config/${ipc}.ibm_environment";
      typeset envfile_cust="${CANDLEHOME}/config/${ipc}.environment";

      if [[ -a "${envfile_ibm}" ]] && [[ -a "${envfile_cust}" ]]; then
         env_ibm=$(grep "^[A-Za-z]" "${envfile_ibm}" | cut -f1 -d"=" );
         env_cust=$(grep "^[A-Za-z]" "${envfile_cust}" | cut -f1 -d"=" );
         for x in ${env_ibm}; do
            if [[ "${env_cust}" = *${x}* ]]; then
               #$NLS-E$ "Warning: $1 is defined in both $2 and $3."
               msg=$(nls_replace 'KCI1428I' "${x}" "\"${envfile_ibm}\"" "\"${envfile_cust}\"");
               print -u2 "${pgm}: ${msg}";
            fi;
         done;                              # (for(x...))               #
      fi;                                   # (if(-a envfile...))       #
   fi;                                      # (if(myrc==0))             #

   return ${myrc};                          # my rc to caller           #
}                                           # (do_MIGRATE())            #

function proc_query { #-------------------------------------------------#
   # Get the list of currently running processes.                       #
   #--------------------------------------------------------------------#
   eval $(itmsh_trace_func "${0}" "${@}");  # enable function tracing   #
   typeset myrc=0;                          # my return code            #
   typeset rc;                              # misc return codes         #

   if [[ -a "${bindir}/itmrestart.sh" ]]; then
      proclist_running=$("${bindir}/itmrestart.sh" "query");
      rc=${?};
      if [[ ${rc} -gt 2 ]]; then
         case ${rc} in                                                  #214745
            10)                                                         #214745
               #$NLS-E$ "An error occurred while trying to determine whether components are running: cinfo failed." #214745
               msg=$(nls_replace 'KCI1434E');                           #214745
              ;;                                                        #214745
            *)                                                          #214745
               #$NLS-E$ "An error occurred while trying to determine whether components are running: rc=$1." #214745
               msg=$(nls_replace 'KCI1429E' "${rc}");
              ;;                                                        #214745
         esac;                                                          #214745
         print -u2 "${pgm}: ${msg}";
         myrc=18;                           # query failed              #
      fi;                                   # (else(rc > 2))            #
   fi;                                      # (if(-a itmrestart.sh))    #

   return ${myrc};                          # my rc to caller           #
}                                           # (proc_query())            #

function proc_stop { #--------------------------------------------------#
   # Stop one or more processes. Our first/only argument is the         #
   # product code to stop. An "*" means to stop everything. The caller  #
   # must have run proc_query() before calling us.                      #
   #--------------------------------------------------------------------#
   eval $(itmsh_trace_func "${0}" "${@}");  # enable function tracing   #
   typeset myrc=0;                          # my return code            #

   typeset pc_stop="${1}";                  # product code to stop      #
   typeset stoplist="";                     # processes to stop         #
   typeset rc;                              # misc return codes         #

   #--------------------------------------------------------------------#
   # Get the proclist entries for the requested components. (The full   #
   # original proclist comes from proc_query().) We remove them from    #
   # the "running" list now on the assumption we can stop them. If we   #
   # can't, the script stops so it doesn't matter if the list is        #
   # shortened prematurely.                                             #
   #--------------------------------------------------------------------#
   if [[ "${pc_stop}" = "*" ]]; then        # if stop all...            #
      stoplist="${proclist_running}";       # get complete list         #
      proclist_running="";
   else
      stoplist=$(print -n "${proclist_running}" | grep "^${pc_stop} ");
      proclist_running=$(print -n "${proclist_running}" | grep -v "^${pc_stop} ");
   fi;                                      # (if(stoplist="*"))        #

   #--------------------------------------------------------------------#
   # Now we stop the requested components. Everything we stop gets      #
   # recorded in a restart list for later. This is so we can try to     #
   # restore the original state.                                        #
   #--------------------------------------------------------------------#
   if [[ -n "${stoplist}" ]]; then          # anything to stop?         #
      if [[ "${iRESTART}" = "yes" ]]; then  # are we allowed?           #
         print "${stoplist}" | "${bindir}/itmrestart.sh" "stop";
         rc=${?};
         
         if [[ "${pc_stop}" = "*" ]]; then  # if stop all...            #214773
            proclist_restart="${stoplist}"; # ...try to restart all     #214773
         elif [[ ${rc} -eq 0 ]]; then       # one: restart if stop ok   #214773
            if [[ -z "${proclist_restart}" ]]; then                     #214773
               proclist_restart="${stoplist}";                          #214773
            else                                                        #214773
               proclist_restart="${proclist_restart}\n${stoplist}";     #214773
            fi;                                                         #214773
         fi;                                                            #214773
                                                                        #214773
         if [[ ${rc} -ne 0 ]]; then                                     #214773
            case ${rc} in                                               #214745
               20)                                                      #214745
                  #$NLS-E$ "An error occurred while trying to stop components: not authorized." #214745
                  msg=$(nls_replace 'KCI1435E');                        #214745
                 ;;                                                     #214745
               21)                                                      #214745
                  #$NLS-E$ "An error occurred while trying to stop components: itmcmd failed." #214745
                  msg=$(nls_replace 'KCI1436E');                        #214745
                 ;;                                                     #214745
               *)                                                       #214745
                  #$NLS-E$ "An error occurred while trying to stop components: rc=$1." #214745
                  msg=$(nls_replace 'KCI1430E' "${rc}");                #214745
                 ;;                                                     #214745
            esac;                                                       #214745
            print -u2 "${pgm}: ${msg}";
            myrc=19;                        # query failed              #
         fi;                                # (else(rc==0))             #
      else
         #$NLS-E$ "You must stop all running components or specify '-r yes' for $1 to stop and restart them."
         msg=$(nls_replace 'KCI1431E' "${pgm}");
         print -u2 "${pgm}: ${msg}";
         myrc=17;                           # components running        #
      fi;                                   # (else(iRESTART=yes))      #
   fi;                                      # (if(-n stoplist))         #

   return ${myrc};                          # my rc to caller           #
}                                           # (proc_stop())             #

function proc_restart { #-----------------------------------------------#
   # Restart anything we stopped in proc_stop().                        #
   #--------------------------------------------------------------------#
   eval $(itmsh_trace_func "${0}" "${@}");  # enable function tracing   #
   typeset myrc=0;                          # my return code            #
   typeset rc;                              # misc return codes         #

   if [[ -n "${proclist_restart}" ]]; then
      print "${proclist_restart}" | "${bindir}/itmrestart.sh" "start";
      rc=${?};
      if [[ ${rc} -ne 0 ]]; then
         case ${rc} in                                                  #214745
            20)                                                         #214745
               #$NLS-E$ "An error occurred while trying to restart components: not authorized." #214745
               msg=$(nls_replace 'KCI1437E');                           #214745
              ;;                                                        #214745
            21)                                                         #214745
               #$NLS-E$ "An error occurred while trying to restart components: itmcmd failed." #214745
               msg=$(nls_replace 'KCI1438E');                           #214745
              ;;                                                        #214745
            *)                                                          #214745
               #$NLS-E$ "An error occurred while trying to restart components: rc=$1." #214745
               msg=$(nls_replace 'KCI1432E' "${rc}");                   #214745
              ;;                                                        #214745
         esac;                                                          #214745
         print -u2 "${pgm}: ${msg}";
         myrc=20;                           # restart failed            #
      fi;                                   # (if(rc!=0))               #
   fi;                                      # (if(-n restart_list))     #

   return ${myrc};                          # my rc to caller           #
}                                           # (proc_restart())          #

#=======================================================================#
# main()                                                                #
#=======================================================================#

#-----------------------------------------------------------------------#
# Establish constants/globals.                                          #
#-----------------------------------------------------------------------#
typeset main_rc=0;                          # overall return code       #
unalias -a;                                 # avert alias interference  #
umask 0022;                                 # create with 755/644 perms #

typeset pgm=$(basename "${0}");             # my full program name      #
typeset pgmfn="${pgm%.sh}";                 # program file name only    #
typeset bindir=`pwd`/$(dirname $0);
[ "$(print $0 | grep '\./')" = "" ] && [ "$(print $0 | grep '^/.*')" = "" ] && [ `pwd` = '/' ] && bindir=`pwd`$(dirname $0);
[ "$(print $0 | grep '^/.*')" != "" ] && bindir=$(dirname $0);

if [[ -a "${bindir}/itmsh_trace_main" ]]; then
   . ${bindir}/itmsh_trace_main;            # embed/start tracing       #
fi;                                         # (if(-a itmsh_trace_main)) #

bindir=$(resolveDir "${bindir}");           # clean up bin path         #214615
if [[ -d ${bindir}/nls ]]; then             # PATH for nls_replace, etc #
   export PATH="${bindir}:${bindir}/nls:${PATH}";  
else
   export PATH="${bindir}:${PATH}";
fi;

typeset thisOS=$(uname -s);                 # get our platform          #
if [[ "${thisOS}" = "SunOS" ]]; then
   tailCMD="tail";                          # Solaris uses old tail     #
else
   tailCMD="tail -n";                       # standard/new tail         #
fi;                                         # (else(thisOS=SunOS))      #
                                                                        #214781
if [[ -a "/usr/xpg4/bin/id" ]]; then        # for older Solaris         #214781
   idCMD="/usr/xpg4/bin/id";                # use POSIX id command      #214781
else                                                                    #214781
   idCMD="id";                              # use standard id command   #214781
fi;                                                                     #214781
typeset thisRUSER=$(${idCMD} -unr);         # get real user name        #214781
typeset needSecure="no";                    # yes==run itmsecure.sh     #214781

#-----------------------------------------------------------------------#
# These are used by proc_query(), proc_stop(), and proc_restart().      #
#-----------------------------------------------------------------------#
typeset proclist_running="";                # procs currently running   #
typeset proclist_restart="";                # procs to restart          #

#-----------------------------------------------------------------------#
# Initialize and parse parameter list. For this exercise, we are just   #
# gathering the options and verifying they are syntactically correct.   #
#-----------------------------------------------------------------------#
typeset -l iBUILD="yes";
typeset -l iFORCE="no";
typeset -l iPC="";
typeset -l iRESTART="no";

while getopts ":b:f:h:l:p:r:" opt > /dev/null 2>&1; do
   case "${opt}" in
      "b") iBUILD="${OPTARG}";;
      "f") iFORCE="${OPTARG}";;
      "h") CANDLEHOME="${OPTARG}";;
      "l") CANDLEDATA="${OPTARG}";;
      "p") iPC="${OPTARG}";;
      "r") iRESTART="${OPTARG}";;
      "?") if [[ "${OPTARG}" = "?" ]]; then # specified "-?" for help?  #
              usage;                        # yes, show it              #
           else                             # must be bad option        #
              #$NLS-E$ "Unrecognized option $1."
              msg=$(nls_replace 'KCI1400E' "\"-${OPTARG}\"");
              print -u2 "${pgm}: ${msg}";
           fi;
           main_rc=10;                      # bad option                #
           break;;                          # stop while/getopts loop   #
      ":") #$NLS-E$ "Value missing for option $1."
           msg=$(nls_replace 'KCI1401E' "\"-${OPTARG}\"");
           print -u2 "${pgm}: ${msg}";
           main_rc=10;                      # missing value             #
           break;;                          # stop while/getopts loop   #
   esac;                                    # (case(opt...))            #
done;                                       # (while(getopts...))       #

if [[ ${main_rc} -eq 0 ]] && [[ ${OPTIND} -le ${#} ]]; then
   #$NLS-E$ "Too many arguments."
   msg=$(nls_replace 'KCI1402E');
   print -u2 "${pgm}: ${msg}";
   main_rc=10;                              # unknown argument          #
fi;                                         # (if(main_rc=0))           #

#-----------------------------------------------------------------------#
# Check parameter values. For this exercise, we're making sure what     #
# they asked for is valid.                                              #
#-----------------------------------------------------------------------#
if [[ ${main_rc} -eq 0 ]]; then
   #--------------------------------------------------------------------#
   # Check -b value.                                                    #
   #--------------------------------------------------------------------#
   if [[ "${iBUILD}" != +(yes|no|1|0) ]]; then
      #$NLS-E$ "Value $1 for $2 is not valid."
      msg=$(nls_replace 'KCI1403E' "\"${iBUILD}\"" "-b")
      print -u2 "${pgm}: ${msg}";
      main_rc=10;                           # bad value                 #
   elif [[ "${iBUILD}" = "1" ]]; then
      iBUILD="yes";
   elif [[ "${iBUILD}" = "0" ]]; then
      iBUILD="no";
   fi;                                      # (elif(iBUILD=0))          #

   #--------------------------------------------------------------------#
   # Check -f value.                                                    #
   #--------------------------------------------------------------------#
   if [[ ${main_rc} -eq 0 ]]; then
      if [[ "${iFORCE}" != +(yes|no|1|0) ]]; then
         #$NLS-E$ "Value $1 for $2 is not valid."
         msg=$(nls_replace 'KCI1403E' "\"${iFORCE}\"" "-f")
         print -u2 "${pgm}: ${msg}";
         main_rc=10;                        # bad value                 #
      elif [[ "${iFORCE}" = "1" ]]; then
         iFORCE="yes";
      elif [[ "${iFORCE}" = "0" ]]; then
         iFORCE="no";
      fi;                                   # (elif(iFORCE=0))          #
   fi;                                      # (if(main_rc=0))           #

   #--------------------------------------------------------------------#
   # Check -h value.                                                    #
   #--------------------------------------------------------------------#
   if [[ ${main_rc} -eq 0 ]]; then
      if [[ -z "${CANDLEHOME}" ]]; then
         export CANDLEHOME=$(cd $(dirname "${0}"); dirname $(pwd) );
      else
         if [[ ! -a "${CANDLEHOME}" ]]; then
            #$NLS-E$ $1 directory $2 does not exist."
            msg=$(nls_replace 'KCI1404E' "CANDLEHOME" "\"${CANDLEHOME}\"");
            print -u2 "${pgm}: ${msg}";
            main_rc=10;                     # bad value                 #
         else
            x=$(resolveDir "${CANDLEHOME}");
            if [[ -z "${x}" ]]; then
               #$NLS-E$ "could not determine the full pathname of $1"
               msg=$(nls_replace 'KCI1081E' "\"${x}\"");
               print -u2 "${pgm}: ${msg}";
               main_rc=10;                  # bad value                 #
            elif [[ "${x}" != "${CANDLEHOME}" ]]; then
               #$NLS-E$ "Directory $1 resolves to $2: using $3 going forward."
               msg=$(nls_replace 'KCI1406I' "\"${resp}\"" "\"${x}\"" "\"${x}\"");
               print "${pgm}: ${msg}";
            fi;                             # (elif(x...))              #
         fi;

         if [[ ${main_rc} -eq 0 ]]; then
            if [[ "${x}" != "${x%%+(#)*}" ]]; then
               #$NLS-E$ "\nDo not use the hash mark (#) character..."
               msg=$(nls_replace 'KCI1360W');
               print -u2 "${pgm}: ${msg}";
               main_rc=10;                  # bad value                 #
            elif [[ "${x}" != "${x%%+([${IFS}])*}" ]]; then
               #$NLS-E$ "\n\$1 - blanks are not permitted..."
               msg=$(nls_replace 'KCI1068I' "\"${x}\"");
               print -u2 "${pgm}: ${msg}";
               main_rc=10;                  # bad value                 #
            elif [[ -a "${x}" ]] && [[ ! -d "${x}" ]]; then
               #$NLS-E$ "$1 already exists but is not a directory."
               msg=$(nls_replace 'KCI1407E' "\"${x}\"");
               print -u2 "${msg}";
               main_rc=10;                  # bad value                 #
            elif [[ ! -a "${x}" ]]; then
               #$NLS-E$ $1 directory $2 does not exist."
               nls_replace 'KCI1404E' "CANDLEHOME" "\"${x}\"";
               print -u2 "${pgm}: ${msg}";
               main_rc=10;                  # bad value                 #
            else
               export CANDLEHOME="${x}";    # use the value             #
            fi;                             # (else(x...))              #
         fi;                                # (if(main_rc=0))           #
      fi;                                   # (else(-z CANDLEHOME))     #
   fi;                                      # (if(main_rc=0))           #

   #--------------------------------------------------------------------#
   # Check -l value.                                                    #
   #--------------------------------------------------------------------#
   if [[ ${main_rc} -eq 0 ]]; then
      CHregistry="${CANDLEHOME}/registry";
      CH2CDmark="${CHregistry}/CANDLEDATA";
      CH2CDlink=$(ls -l "${CH2CDmark}" 2> /dev/null);
      
      if [[ "${CH2CDlink}" = *" -> "* ]]; then
         CH2CDlink="${CH2CDlink##* -> }";   # get link value            #
      else
         CH2CDlink="";                      # ignore garbage            #
      fi;                                   # (if(CD2CHlink...))        #
      
      if [[ -z "${CANDLEDATA}" ]]; then
         export CANDLEDATA="${CH2CDlink}";
         if [[ -z "${CANDLEDATA}" ]]; then
            #$NLS-E$ "$1 is not defined."
            msg=$(nls_replace 'KCI1405E' "CANDLEDATA");
            print -u2 "${pgm}: ${msg}";
            main_rc=10;                     # bad value                 #
         fi;                                # (if(-z CANDLEDATA))       #
      else
         if [[ -a "${CANDLEDATA}" ]]; then
            x=$(resolveDir "${CANDLEDATA}");
            if [[ -z "${x}" ]]; then
               #$NLS-E$ "could not determine the full pathname of $1"
               msg=$(nls_replace 'KCI1081E' "\"${x}\"");
               print -u2 "${pgm}: ${msg}";
               main_rc=10;                  # bad value                 #
            elif [[ "${x}" != "${CANDLEDATA}" ]]; then
               #$NLS-E$ "Directory $1 resolves to $2: using $3 going forward."
               msg=$(nls_replace 'KCI1406I' "\"${CANDLEDATA}\"" "\"${x}\"" "\"${x}\"");
               print "${pgm}: ${msg}";
            fi;                             # (elif(x...))              #
            
            CD2CHmark="${CANDLEDATA}/CANDLEHOME";
            CD2CHlink=$(ls -l "${CD2CHmark}" 2> /dev/null);
            if [[ "${CD2CHlink}" = *" -> "* ]]; then
               CD2CHlink="${CD2CHlink##* -> }";  # get link value       #
            else
               CD2CHlink="";                # ignore garbage            #
            fi;                             # (if(CD2CHlink...))        #
            if [[ -n "${CD2CHlink}" ]]; then
               if [[ "${CANDLEHOME}" != "${CD2CHlink}" ]]; then
                  #$NLS-E$ "Data directory $1 is already in use by installation directory $2."
                  msg=$(nls_replace 'KCI1433E' "\"${CANDLEDATA}\"" "\"${CD2CHlink}\"");
                  print -u2 "${pgm}: ${msg}";
                  main_rc=10;                  # bad value                 #
               fi;
            fi;
         else
            x="${CANDLEDATA}";              # verify value as-is        #
         fi;                                # (else(-a CANDLEDATA))     #
         
         if [[ ${main_rc} -eq 0 ]]; then
            if [[ "${x}" != "${x%%+(#)*}" ]]; then
               #$NLS-E$ "\nDo not use the hash mark (#) character..."
               $(nls_replace 'KCI1360W');
               print -u2 "${pgm}: ${msg}";
               main_rc=10;                  # bad value                 #
            elif [[ "${x}" != "${x%%+([${IFS}])*}" ]]; then
               #$NLS-E$ "\n$1 - blanks are not permitted..."
               msg=$(nls_replace 'KCI1068I' "\"${x}\"");
               print -u2 "${pgm}: ${msg}";
               main_rc=10;                  # bad value                 #
            elif [[ -a "${x}" ]] && [[ ! -d "${x}" ]]; then
               #$NLS-E$ "$1 already exists but is not a directory."
               nls_replace 'KCI1407E' "\"${x}\"";
               print -u2 "${pgm}: ${msg}";
               main_rc=10;                  # bad value                 #
            elif [[ "${x}" = "${CANDLEHOME}" ]]; then
               #$NLS-E$ "Data directory and installation directory are the same. No conversion is required."
               msg=$(nls_replace 'KCI1408I');
               print -u2 "${pgm}: ${msg}";
               main_rc=1;                   # no conversion needed      #
            elif [[ "${x}" = "${CANDLEHOME}"* ]]; then
               #$NLS-E$ "Data directory $1 cannot be a subdirectory of installation directory $2."
               msg=$(nls_replace 'KCI1409E' "\"${x}\"" "\"${CANDLEHOME}\"");
               print -u2 "${pgm}: ${msg}";
               main_rc=10;                  # bad value                 #
            elif [[ "${CANDLEHOME}" = "${x}"* ]]; then
               #$NLS-E$ "Installation directory $1 cannot be a subdirectory of data directory $2."
               msg=$(nls_replace 'KCI1410E' "\"${CANDLEHOME}\"" "\"${x}\"");
               print -u2 "${pgm}: ${msg}";
               main_rc=10;                  # bad value                 #
            elif [[ -n "${CH2CDlink}" ]] && [[ "${CH2CDlink}" != "${x}" ]]; then
               #$NLS-E$ "Input data directory $1 is different than directory $2 already in use by $3."
               msg=$(nls_replace 'KCI1411E' "\"${x}\"" "\"${CH2CDlink}\"" "\"${CANDLEHOME}\"");
               print -u2 "${pgm}: ${msg}";
               main_rc=10;                  # bad value                 #
            else
               export CANDLEDATA="${x}";    # use the value             #
            fi;                             # (else(x...))              #
         fi;                                # (if(main_rc=0))           #
      fi;                                   # (else(-z CANDLEDATA))     #
   fi;                                      # (if(main_rc=0))           #

   #--------------------------------------------------------------------#
   # Check -p value. Note that -f *must* have been checked *before* we  #
   # get here because it affects how we handle -p.                      #
   #--------------------------------------------------------------------#
   if [[ ${main_rc} -eq 0 ]]; then
      typeset pc_process="";                # pc's to process           #

      typeset pc_unsupported="ms";
      pc_unsupported="${pc_unsupported} cq cj cw iu iw kf";
      pc_unsupported="${pc_unsupported} um ul";

      y="";                                 # for padding of pc list    #
      pc_default="";                        # start with empty list     #
      find -L "${CANDLEHOME}/config" -name "??.ini" 2> /dev/null | while read x; do
         x="${x##*/}";                      # strip directory           #
         x="${x%.ini}";                     # strip extension           #
         if [[ "${pc_unsupported}" = *${x}* ]]; then
            #$NLS-E$ "Product code $1 is not eligible to use the data directory."
            msg=$(nls_replace 'KCI1414E' "\"${x}\"");                   #214615
            print -u2 "${pgm}: ${msg}";                                 #214615
            main_rc=10;                     # bad value                 #214615
         elif [[ -a "${CANDLEHOME}/config/.ConfigData/CANDLEDATA_${x}" ]] && [[ "${iFORCE}" != "yes" ]]; then #213940
            : ;                             # already done/no-op==skip  #
         else
            pc_default="${pc_default}${y}${x}";  # add this one         #
            y=" ";                               # start padding        #
         fi;                                     # (if(pc_unsuppor...)) #
      done;                                 # (find|while read...))     #214022
                                                                        #214022
      find "${CANDLEHOME}/bin" -name ".agent.??" 2> /dev/null | while read x; do #214022
         x="${x##*/}";                      # strip directory           #214022
         x="${x#.agent.}";                  # strip fname prefix        #214022
         if [[ "${pc_unsupported}" = *${x}* ]]; then                    #214022
            #$NLS-E$ "Product code $1 is not eligible to use the data directory."
            msg=$(nls_replace 'KCI1414E' "\"${x}\"");                   #214615
            print -u2 "${pgm}: ${msg}";                                 #214615
            main_rc=10;                     # bad value                 #214615
         elif [[ -a "${CANDLEHOME}/config/.ConfigData/CANDLEDATA_${x}" ]] && [[ "${iFORCE}" != "yes" ]]; then #213940
            : ;                             # already done/no-op==skip  #214022
         else                                                           #214022
            pc_default="${pc_default}${y}${x}";  # add this one         #214022
            y=" ";                               # start padding        #214022
         fi;                                     # (if(pc_unsuppor...)) #214022
      done;                                 # (find|while read...))     #214022

      if [[ "${iPC}" = "*" ]]; then
         pc_process="${pc_default}";        # use default list          #
      else
         y="";                              # for padding of pc list    #
         for x in ${iPC}; do                # check each input value    #
            if [[ "${x}" != [a-z0-9][a-z0-9] ]]; then
               #$NLS-E$ "Product code $1 is not valid.";
               msg=$(nls_replace 'KCI1412E' "\"${x}\"");
               print -u2 "${pgm}: ${msg}";
               main_rc=10;                  # bad value                 #
            elif [[ ! -a "${CANDLEHOME}/config/${x}.ini" ]] && [[ ! -a "${CANDLEHOME}/bin/.agent.${x}" ]]; then #214022
               #$NLS-E$ "No configuration file was found for product code $1."
               msg=$(nls_replace 'KCI1413E' "\"${x}\"");
               print -u2 "${pgm}: ${msg}";
               main_rc=10;                  # bad value                 #
            elif [[ "${pc_unsupported}" = *${x}* ]]; then
               #$NLS-E$ "Product code $1 is not eligible to use the data directory."
               msg=$(nls_replace 'KCI1414E' "\"${x}\"");
               print -u2 "${pgm}: ${msg}";
               main_rc=10;                  # bad value                 #
            elif [[ -a "${CANDLEHOME}/config/.ConfigData/CANDLEDATA_${x}" ]] && [[ "${iFORCE}" != "yes" ]]; then
               #$NLS-E$ "Product code $1 skipped because it is already migrated to use the data directory."
               msg=$(nls_replace 'KCI1415I' "\"${x}\"");
               print -u2 "${pgm}: ${msg}";
            elif [[ "${pc_process}" = *${x}* ]]; then
               : ;                          # duplicate/no-op==skip     #
            else
               pc_process="${pc_process}${y}${x}";    # add this one    #
               y=" ";                                 # start padding   #
            fi;                             # (else...))                #
         done;                              # (for(x))                  #
      fi;                                   # (else(iPC=""))            #
   fi;                                      # (if(main_rc=0))           #

   #--------------------------------------------------------------------#
   # Check -r value.                                                    #
   #--------------------------------------------------------------------#
   if [[ ${main_rc} -eq 0 ]]; then
      if [[ "${iRESTART}" != +(yes|no|1|0) ]]; then
         #$NLS-E$ "Value $1 for $2 is not valid.";
         msg=$(nls_replace 'KCI1403E' "\"${iRESTART}\"" "-r")
         print -u2 "${pgm}: ${msg}";
         main_rc=10;                        # bad value                 #
      elif [[ "${iRESTART}" = "1" ]]; then
         iRESTART="yes";
      elif [[ "${iRESTART}" = "0" ]]; then
         iRESTART="no";
      fi;                                   # (elif(iRESTART="0"))      #
   fi;                                      # (if(main_rc=0))           #
fi;                                         # (if(main_rc=0))           #

#-----------------------------------------------------------------------#
# If tracing wasn't started from where itmCANDLEDATA.sh was launched,   #214781
# try now from CANDLEHOME/bin.                                          #214781
#-----------------------------------------------------------------------#
if [[ ${main_rc} -eq 0 ]]; then
   if [[ -z "${ITMSH_TRACE_EMBEDDED}" ]]; then
      if [[ -a "${CANDLEHOME}/bin/itmsh_trace_main" ]]; then
         . "${CANDLEHOME}/bin/itmsh_trace_main"; # embed/start tracing  #
      fi;                                   # (if(-a...))               #
   fi;                                      # (if(-z ITMSH_TRACE_E...)) #
fi;                                         # (if(main_rc=0))           #

#-----------------------------------------------------------------------#
# Make sure TEMS and TEPS are not in the mix. We don't support the      #
# CANDLEDATA split with those components.                               #
#-----------------------------------------------------------------------#
if [[ ${main_rc} -eq 0 ]] && [[ "${iBUILD}" = "yes" ]]; then
   if [[ -a "${CANDLEHOME}/config/ms.ini" ]]; then
      #$NLS-E$ "The monitoring server is installed in $1."
      msg=$(nls_replace 'KCI1416I' "\"${CANDLEHOME}\"");
      print -u2 "${pgm}: ${msg}";
      main_rc=21;
   elif [[ -a "${CANDLEHOME}/config/cq.ini" ]]; then
      #$NLS-E$ "The portal server is installed in $1."
      msg=$(nls_replace 'KCI1417I' "\"${CANDLEHOME}\"");
      print -u2 "${pgm}: ${msg}";
      main_rc=21;
   fi;
   if [[ ${main_rc} -eq 21 ]]; then
      #$NLS-E$ "This component is not supported with the data directory split from the installation directory."
      msg=$(nls_replace 'KCI1418E');
      print -u2 "${pgm}: ${msg}";
   fi;
fi;                                         # (if(main_rc=0))           #

#-----------------------------------------------------------------------#
# Get list of running processes so we know what we may need to stop.    #
#-----------------------------------------------------------------------#
if [[ ${main_rc} -eq 0 ]]; then
   if [[ "${iBUILD}" = "yes" ]] || [[ -n "${pc_process}" ]]; then
      proc_query;
      main_rc=${?};                         # use proc_query's rc as-is #
   fi;                                      # (if(iBUILD...))           #
fi;                                         # (if(main_rc=0))           #

#-----------------------------------------------------------------------#
# Build CANDLEDATA if needed.                                           #
#-----------------------------------------------------------------------#
if [[ ${main_rc} -eq 0 ]]; then
   if [[ "${iBUILD}" = "yes" ]]; then
      proc_stop "*";                        # stop running components   #
      main_rc=${?};                         # use proc_stop's rc as-is  #

      if [[ ${main_rc} -eq 0 ]]; then
         cd /tmp;                           # make sure we're not in CH #214615
         typeset myOLDPWD="${OLDPWD}";      # remember OLDPWD           #214615
                                                                        #214615
         do_BUILD;                          # build CANDLEDATA          #
         main_rc=${?};
         if [[ ${main_rc} -eq 0 ]]; then
            needSecure="yes";               # run itmsecure.sh later    #214781
            #$NLS-E$ "Installation directory $1 successfully configured to use data directory $2."
            msg=$(nls_replace 'KCI1419I' "\"${CANDLEHOME}\"" "\"${CANDLEDATA}\"");
            print "${pgm}: ${msg}";
         else
            #$NLS-E$ "An error occurred while configuring installation directory $1 to use data directory $2: rc=$3."
            msg=$(nls_replace 'KCI1420I' "\"${CANDLEHOME}\"" "\"${CANDLEDATA}\"" "${main_rc}");
            print -u2 "${pgm}: ${msg}";
         fi;                                # (else(main_rc=0))         #
                                                                        #214615
         cd "${myOLDPWD}" > /dev/null 2>&1; # restore working directory #214615
      fi;                                   # (if(main_rc=0))           #
   fi;                                      # (if(iBUILD = yes))        #
fi;                                         # (if(main_rc=0))           #

#-----------------------------------------------------------------------#
# Migrate components to use CANDLEDATA.                                 #
#-----------------------------------------------------------------------#
if [[ ${main_rc} -eq 0 ]]; then
   typeset pc_this;                         # product code to migrate   #
   typeset pc_rc;                           # rc for pc_this            #

   cd /tmp;                                 # make sure we're not in CH #214615
   typeset myOLDPWD="${OLDPWD}";            # remember OLDPWD           #214615
                                                                        #214615
   for pc_this in ${pc_process}; do         # for each product code...  #
      proc_stop "${pc_this}";               # stop running components   #
      main_rc=${?};                         # use proc_stop's rc as-is  #

      if [[ ${main_rc} -eq 0 ]]; then
         do_MIGRATE "${pc_this}";           # migrate the component     #
         pc_rc=${?};

         if [[ ${pc_rc} -eq 0 ]]; then
            needSecure="yes";               # run itmsecure.sh later    #214781
            #$NLS-E$ "Component $1 successfully configured to use data directory $2."
            msg=$(nls_replace 'KCI1421I' "${pc_this}" "\"${CANDLEDATA}\"");
            print "${pgm}: ${msg}";
         else
            #$NLS-E$ "An error occurred while configuring component $1 to use data directory $2: rc=$3."
            msg=$(nls_replace 'KCI1422I' "${pc_this}" "\"${CANDLEDATA}\"" "${pc_rc}");
            print -u2 "${pgm}: ${msg}";

            if [[ ${main_rc} -eq 0 ]]; then # first failure?            #
               main_rc=${pc_rc};            # yes, reflect return code  #
            fi;                             # (if(main_rc=0))           #
         fi;                                # (else(main_rc=0))         #
      fi;                                   # (if(main_rc=0))           #
   done;                                    # (for(pc_this...))         #
                                                                        #214615
   cd "${myOLDPWD}" > /dev/null 2>&1;       # restore working directory #214615
fi;                                         # (if(main_rc=0))           #

#-----------------------------------------------------------------------#214781
# Run itmsecure.sh (aka secureMain) if we did anything to trigger the   #214781
# need and it has been run in the past. Note we do not look for RC==0   #214781
# because itmsecure.sh does not reliably set its return code.           #214781
#-----------------------------------------------------------------------#214781
if [[ "${needSecure}" = "yes" ]] &&         # any conversions succeed?  #214781
   [[ "${thisRUSER}" = "root" ]] &&         # are we root?              #214781
   [[ -a "${CANDLEHOME}/registry/SecureMode" ]]; then # secureMain run? #214781
   export PARM_FILE="x";                    # ==itmsecure.sh noprompt   #214781
   "${CANDLEHOME}/bin/itmsecure.sh" "autorun";        # run it again    #214781
fi;                                         # (if(needSecure="yes"))    #214781
                                                                        #214781
#-----------------------------------------------------------------------#
# Restart whatever we stopped earlier.                                  #
#-----------------------------------------------------------------------#
proc_restart;

exit ${main_rc};                            # exit with return code     #