#!/bin/ksh
#=======================================================================#
# Licensed Materials - Property of IBM                                  #
# 5724-C04                                                              #
# Copyright IBM Corporation 2015 All Rights Reserved                    #
#-----------------------------------------------------------------------#
# itmCANDLEDATA_xx.sh - This script complements ITM's itmCANDLEDATA.sh  #
#    for generic components. We do the following:                       #
#    a) Create subdirectories in CANDLEDATA for agent-specific data.    #213940
#    b) Move existing agent-specific data to the new CANDLEDATA         #213940
#       directories.                                                    #213940
#    c) Link the agent-specific directories in CANDLEHOME to their      #213940
#       mates in CANDLEDATA.                                            #213940
#    c) Set the agent's configuration variables (as needed) to point to #213940
#       the redirected data.                                            #
#-----------------------------------------------------------------------#
# SYNTAX: itmCANDLEDATA_xx.sh <pc>                                      #
#                                                                       #
#    Where:                                                             #
#       pc: Indicates the two-character product code of the component   #
#           to be migrated.                                             #
#                                                                       #
# ENVIRONMENT VARIABLES at launch:                                      #
#    CANDLEHOME: Locates the ITM installation directory.                #
#    CANDLEDATA: Locates the ITM data directory.                        #
#                                                                       #
# RETURN CODES:                                                         #
#    0 - Success.                                                       #
#   10 - Failure: Unable to create subdirectory.                        #
#   11 - Failure: Unable to link CANDLEHOME to CANDLEDATA.              #
#   12 - Failure: Unable to write temp file.                            #
#   13 - Failure: Unable to replace environment override file.          #
#   14 - Failure: Unable to move old data to new location.              #
#=======================================================================#
. ${CANDLEHOME}/bin/itmsh_trace_main;       # embed/start tracing       #
myrc=0;                                     # assume success            #
pc="${1}";                                  # product code in play      #

typeset thisOS=$(uname -s);                 # get our platform          #214586
if [[ "${thisOS}" = "SunOS" ]]; then                                    #214586
   tailCMD="tail";                          # Solaris uses old tail     #214586
else                                                                    #214586
   tailCMD="tail -n";                       # standard/new tail         #214586
fi;                                         # (else(thisOS=SunOS))      #214586
                                                                        #214586
function migrate_dir { #------------------------------------------------#213940
   # Migrate the contents of one directory ($1) to another ($2). Note   #213940
   # the move is not a simple mv command. We don't know if there is     #213940
   # any content (mv will fail if not), a single mv command won't       #213940
   # handle both hidden and regular files, and there's a limit to the   #213940
   # size of a command line (if we used "mv *"). What we do instead is  #213940
   # roughly the equivalent of "find... -mindepth 1 -maxdepth 1 -exec   #213940
   # mv...". Since the platforms we support don't all have a full GNU   #213940
   # find command, we use find+tail+grep to simulate the mindepth and   #213940
   # maxdepth options. Basically, this gives a list of everything in    #213940
   # the top of the directory... not the directory itself and no        #213940
   # descending into subdirectories. This list comprises everything we  #213940
   # need to move. We let mv errors flow to stderr so the user can see  #213940
   # what happened if something went bad.                               #213940
   #--------------------------------------------------------------------#213940
   eval $(itmsh_trace_func "${0}" "${@}");  # enable function tracing   #213940
   typeset myrc=0;                                                      #213940
   typeset srcdir="${1}";                                               #213940
   typeset tgtdir="${2}";                                               #213940
   typeset mvsrc;                                                       #213940
   typeset cmd;                                                         #213940
                                                                        #213940
   cmd="find \"${srcdir}\"";                                            #213940
   cmd="${cmd} | ${tailCMD} +2";                                        #214586
   cmd="${cmd} | grep -v \"^${srcdir}/.*/\"";                           #213940
   eval ${cmd} | while read mvsrc; do                                   #213940
      mv "${mvsrc}" "${tgtdir}";                                        #213940
      if [[ ${?} -ne 0 ]]; then                                         #213940
         myrc=14;                                                       #213940
      fi;                                                               #213940
   done;                                                                #213940
                                                                        #213940
   return ${myrc};                                                      #213940
}                                                                       #213940
                                                                        #214940
function migrate_psit { #-----------------------------------------------#213940
   # Migrate situation data files from one directory ($1) to another    #213940
   # (${2}). The file name patterns used here came from the TEMA team.  #213940
   #--------------------------------------------------------------------#213940
   eval $(itmsh_trace_func "${0}" "${@}");  # enable function tracing   #213940
   typeset myrc=0;                                                      #213940
   typeset srcdir="${1}";                                               #213940
   typeset tgtdir="${2}";                                               #213940
   typeset mvsrc;                                                       #213940
   typeset cmd;                                                         #213940
                                                                        #213940
   cmd="find \"${srcdir}\"";                                            #213940
   cmd="${cmd} \(";                                                     #213940
   cmd="${cmd} -name \"psit_*.str\"";                                   #213940
   cmd="${cmd} -o -name \"*_agntstat.sta\"";                            #213940
   cmd="${cmd} -o -name \"*_autoattr.asa\"";                            #213940
   cmd="${cmd} -o -name \"*_thresholds.xml\"";                          #213940
   cmd="${cmd} \)";                                                     #213940
   eval ${cmd} | while read mvsrc; do                                   #213940
      mv "${mvsrc}" "${tgtdir}";                                        #213940
      if [[ ${?} -ne 0 ]]; then                                         #213940
         myrc=14;                                                       #213940
      fi;                                                               #213940
   done;                                                                #213940
                                                                        #213940
   return ${myrc};                                                      #213940
}                                                                       #213940
                                                                        #213940
                                                                        
#-----------------------------------------------------------------------#
# Create subdirectories in CANDLEDATA for agent-specific data.          #213940
#-----------------------------------------------------------------------#
for subDir in "hist" "psit" "ATTRLIB" "EIFLIB"; do                      #213940
   mkdir -p "${CANDLEDATA}/${subDir}/${pc}";
   if [[ ${?} -ne 0 ]]; then
      myrc=10;
   else
      if [[ ! -L "${CANDLEHOME}/${subDir}" ]]; then
         ln -fs "${CANDLEDATA}/${subDir}" "${CANDLEHOME}/${subDir}";
         if [[ ${?} -ne 0 ]]; then
            myrc=11;
         fi;
      fi;
   fi;
done;

#-----------------------------------------------------------------------#
# Move any existing agent-specific data from CANDLEHOME to our shiny    #213940
# new subdirectories in CANDLEDATA. We also create symbolic links from  #213940
# the agent's subdirectories so as to minimize impact to the agent's    #213940
# configuration.                                                        #213940
#-----------------------------------------------------------------------#
if [[ ${myrc} -eq 0 ]]; then
   for binDir in $(ls -d "${CANDLEHOME}/"*"/${pc}/bin" 2> /dev/null); do
      for subDir in "hist" "psit" "ATTRLIB" "EIFLIB"; do                #213940
         case "${subDir}" in                                            #213940
            "hist")                                                     #213940
               linkDir="${binDir%bin}${subDir}";                        #213940
               if [[ ! -L "${linkDir}" ]]; then                         #213940
                  moveDir="${linkDir}";                                 #213940
                  migrate_dir "${moveDir}" "${CANDLEHOME}/${subDir}/${pc}"; #213940
                  myrc=${?};                                            #213940
               fi;                                                      #213940
            ;;                                                          #213940
            "psit")                                                     #213940
               linkDir="${binDir%bin}${subDir}";                        #213940
               moveDir="${binDir}";                                     #213940
               migrate_psit "${moveDir}" "${CANDLEHOME}/${subDir}/${pc}"; #213940
               myrc=${?};                                               #213940
               CTIRA_SIT_PATH_line="CTIRA_SIT_PATH=${linkDir}";         #213940
            ;;                                                          #213940
            "ATTRLIB"|"EIFLIB")                                         #213940
               linkDir="${binDir%bin}tables/${subDir}";                 #213940
               if [[ ! -L "${linkDir}" ]]; then                         #213940
                  moveDir="${linkDir}";                                 #213940
                  migrate_dir "${moveDir}" "${CANDLEHOME}/${subDir}/${pc}"; #213940
                  myrc=${?};                                            #213940
               fi;                                                      #213940
            ;;                                                          #213940
         esac;                                                          #213940
                                                                        #213940
         if [[ ${myrc} -eq 0 ]]; then                                   #213940
            if [[ ! -L "${linkDir}" ]]; then                            #213940
               [[ -d "${linkDir}" ]] && rmdir "${linkDir}";             #213940
               ln -fs "${CANDLEHOME}/${subDir}/${pc}" "${linkDir}";     #213940
               if [[ ${?} -ne 0 ]]; then                                #213940
                  myrc=11;                                              #213940
               fi;                                                      #213940
            fi;                                                         #213940
         fi;                                                            #213940
      done;
   done;
fi;

#-----------------------------------------------------------------------#
# Change the agent's configuration. Most of the data automatically goes #213940
# to CANDLEDATA by way of the symbolic links. However, we need to use   #213940
# CTIRA_SIT_PATH to redirect persistent situation data out of the       #213940
# agent's bin directory.                                                #213940
#-----------------------------------------------------------------------#
if [[ ${myrc} -eq 0 ]]; then
   envfile="${CANDLEHOME}/config/${pc}.ibm_environment";
   envtemp="${CANDLEHOME}/config/${pc}.temp_environment";
   envrepl="";

   CTIRA_SIT_PATH_done=0;

   if [[ -a "${envfile}" ]]; then
      cat "${envfile}" | while read line; do
         varname="${line%=*}";
         case "${varname}" in
            "CTIRA_SIT_PATH")
               line="${CTIRA_SIT_PATH_line}";
               CTIRA_SIT_PATH_done=1;
               ;;
            *) ;;
         esac
         envrepl="${envrepl}${line}\n";
      done;
   fi;

   if [[ ${CTIRA_SIT_PATH_done} -eq 0 ]]; then
      line="${CTIRA_SIT_PATH_line}";
      envrepl="${envrepl}${line}\n";
   fi;

   if [[ ${myrc} -eq 0 ]]; then
      print -n "${envrepl}" > "${envtemp}";
      if [[ ${?} -ne 0 ]]; then
         myrc=12;
      else
         mv "${envtemp}" "${envfile}";
         if [[ ${?} -ne 0 ]]; then
            myrc=13;
         fi;
      fi;
   fi;
fi;

exit ${myrc};