#!/bin/ksh
#=======================================================================#
# Licensed Materials - Property of IBM                                  #
# 5724-C04                                                              #
# Copyright IBM Corporation 2015 All Rights Reserved                    #
#-----------------------------------------------------------------------#
# itmrestart.sh - This script queries, stops, and restarts ITM          #
#    components. To restart everything that's currently running, the    #
#    call sequence looks like this:                                     #
#                                                                       #
#       proclist=$(itmrestart.sh query)                                 #
#       print "${proclist}" | itmrestart.sh stop                        #
#       print "${proclist}" | itmrestart.sh start                       #
#                                                                       #
#-----------------------------------------------------------------------#
# SYNTAX: itmrestart.sh                                                 #
#            query|stop|start                                           #
#                                                                       #
#    Where:                                                             #
#       query: Requests whether components are running. A list of       #
#              running components is written to STDOUT.                 #
#       stop:  Requests the list of components passed via STDIN be      #
#              stopped.                                                 #
#       start: Requests the list of components passed via STDIN be      #
#              started.                                                 #
#                                                                       #
# RETURN CODES (query)                                                  #
#    0 - No components are running.                                     #
#    1 - One or more components are running.                            #
#   10 - An error occurred running cinfo.                               #
# RETURN CODES (stop|start)                                             #
#    0 - All requested components were stopped or started.              #
#   20 - Not authorized (either not running as root or not running as   #
#        userid that started a process).                                #
#   21 - Command to start or stop a component failed.                   #
# RETURN CODES (all)                                                    #
#    2 - Syntax error.                                                  #
#                                                                       #
# This program intentionally emits no messages itself: we leave it up   #
# to the caller to decide when messages are appropriate. That said,     #
# this program does allow messages from subcomponents (cinfo and        #
# itmcmd) to flow through. It is up to the caller to capture or         #
# surface them as needed.                                               #
#=======================================================================#

function usage { #------------------------------------------------------#
   # Display syntax.                                                    #
   #--------------------------------------------------------------------#
   print -u2 "Usage: ${pgm} ";
   print -u2 "   query|stop|start";
}                                           # (usage())                 #

function locateCMD { #--------------------------------------------------#
   # Find the directory where the executable ${1} resides so we can     #
   # invoke it directly.                                                #
   #--------------------------------------------------------------------#
   eval $(itmsh_trace_func "${0}" "${@}");  # enable function tracing   #
   typeset outCMD="";                       # result                    #
   if [ -n "${1}" ]; then                   # if command passed...      #
      if [ "${thisOS}" = "HP-UX" ] || [ "${thisOS}" = "SunOS" ]; then
         outCMD=`which ${1} 2> /dev/null | grep -v "^no ${1}"`;
      else
         outCMD=`which ${1} 2> /dev/null`;
      fi;                                   # (else(thisOS...))         #
      if [ -z "${outCMD}" ]; then           # still need result...      #
         outCMD=$(find /bin /usr/bin /local/bin /usr/local/bin /sbin /usr/sbin /local/sbin /usr/local/sbin -name ${1} 2> /dev/null | head -n 1);
      fi;                                   # (if(-z outCMD))           #
   fi;                                      # (if(-n ${1}))             #

   print "${outCMD}";                       # result to caller          #
}                                           # (locateCMD())             #

function do_QUERY { #---------------------------------------------------#
   # Obtain and convert "cinfo -r" output into the following form:      #
   #                                                                    #
   #     pc pid instance realuser                                       #
   #                                                                    #
   # Note that cinfo may or may not be there. If we are called during   #
   # a pristine install, it won't be. For that case, we just return     #
   # zero to indicate no running processes.                             #
   #--------------------------------------------------------------------#
   eval $(itmsh_trace_func "${0}" "${@}");  # enable function tracing   #
   typeset myrc=0;                          # assume no processes       #

   typeset x y z;                           # local work variables      #
   typeset irecs;                           # cache for cinfo records   #

   if [[ -a "${CANDLEHOME}/bin/cinfo" ]]; then   # if cinfo is there... #
      irecs=$("${CANDLEHOME}/bin/cinfo" -r);     # get cinfo records    #
      rc=${?};
      
      if [[ ${rc} -eq 0 ]]; then                 # if cinfo ok...       #
         print "${irecs}" | while read x; do     # process each rec...  #
            if [[ "${x}" = *"...running" ]]; then
               myrc=1;                      # we have processes!        #
               set -A y ${x};               # break words into array    #
               z=$(( ${#y[@]} - 2 ));       # penultimate word position #
               z="${y[${z}]}";              # penultimate word/instance #
               if [[ "${y[1]}" = "ud" ]]; then   # for DB2 agent...     #
                  z="${z#${y[0]}_}";        # strip hostname from inst  #214773
               fi;                          # (if(y[1]==ud))            #
               print "${y[1]} ${y[2]} ${z} ${y[3]}";
            fi;                             # (if(x = *...running))     #
         done;                              # (while(read x))           #
      else
         myrc=10;                           # cinfo/grep failed         #
      fi;                                   # (else(${?} != 0))         #
   fi;                                      # (if(-a cinfo))            #

   return ${myrc};                          # rc to caller              #
}                                           # (do_QUERY())              #

function do_STOP { #----------------------------------------------------#
   # Stop all the components found by QUERY. The list of components is  #
   # passed to us via STDIN.                                            #
   #--------------------------------------------------------------------#
   eval $(itmsh_trace_func "${0}" "${@}");  # enable function tracing   #

   typeset myrc=0;                          # my return code            #

   typeset pcnt=0;                          # process count             #
   typeset pPC;                             # ...product code array     #
   typeset pPID;                            # ...identifier array       #
   typeset pPINST;                          # ...instance name array    #
   typeset pPUSER;                          # ...real userid array      #
   typeset p;                               # counter                   #
   typeset cmd;                             # stop command incantation  #

   #--------------------------------------------------------------------#
   # Read the process table from the caller's QUERY. We make sure we    #
   # have the authority to manage the process the way it was originally #
   # established.                                                       #
   #--------------------------------------------------------------------#
   while read pPC[${pcnt}] pPID[${pcnt}] pINST[${pcnt}] pUSER[${pcnt}]; do
      if [[ "${thisEUSER}" != "root" ]] && [[ "${thisEUSER}" != "${pUSER[${pcnt}]}" ]]; then
         myrc=20;
      fi;                                   # (if(thisEUSER...))        #
      pcnt=$((pcnt + 1));                   # count this process        #
   done;                                    # (while(read...))          #

   #--------------------------------------------------------------------#
   # Run the incantation to stop each component. This is adapted from   #
   # install.sh.                                                        #
   #--------------------------------------------------------------------#
   if [[ ${myrc} -eq 0 ]]; then
      p=0;
      while [[ ${p} -lt ${pcnt} ]]; do
         if [[ "${pPC[${p}]}" = "ms" ]]; then
            cmd="${CANDLEHOME}/bin/CandleServer stop ${pINST[${p}]}";
         elif [[ "${pPC[${p}]}" = "kf" ]]; then
            cmd="${CANDLEHOME}/bin/CandleAgent -f stop kf";
         elif [[ "${pINST[${p}]}" = "None" ]]; then
            cmd="${CANDLEHOME}/bin/CandleAgent stop ${pPC[${p}]}";
         elif [[ "${pPC[${p}]}" = "um" ]]; then
            cmd="${CANDLEHOME}/bin/CandleAgent -p ${pINST[${p}]} -f stop um";
         else
            cmd="${CANDLEHOME}/bin/CandleAgent -o ${pINST[${p}]} -f stop ${pPC[${p}]}";
         fi;

         if [[ "${thisRUSER}" != "${pUSER[${p}]}" ]]; then
            cmd="${su_cmd} - ${pUSER[${p}]} -c \"${cmd}\"";
         fi;

         eval "${cmd}";                     # run the command           #
         rc=${?};
         if [[ ${rc} -ne 0 ]]; then         # if command failed...      #
            myrc=21;                        # ...tell our caller        #
         fi;                                # (if($? != 0))             #

         p=$((p + 1));                      # skip to next process      #
      done;                                 # (while(p < pcnt))         #
   fi;                                      # (if(myrc==0))             #

   return ${myrc};                          # rc to caller              #
}                                           # (do_STOP())               #

function do_START { #---------------------------------------------------#
   # Start all the components found by QUERY. The list of components is #
   # passed to us via STDIN.                                            #
   #--------------------------------------------------------------------#
   eval $(itmsh_trace_func "${0}" "${@}");  # enable function tracing   #
   typeset myrc=0;                          # my return code            #

   typeset pcnt=0;                          # process count             #
   typeset pPC;                             # ...product code array     #
   typeset pPID;                            # ...identifier array       #
   typeset pPINST;                          # ...instance name array    #
   typeset pPUSER;                          # ...real userid array      #
   typeset p;                               # counter                   #
   typeset cmd;                             # start command incantation #

   #--------------------------------------------------------------------#
   # Read the process table from the caller's QUERY. We make sure we    #
   # have the authority to manage the process the way it was originally #
   # established.                                                       #
   #--------------------------------------------------------------------#
   while read pPC[${pcnt}] pPID[${pcnt}] pINST[${pcnt}] pUSER[${pcnt}]; do
      if [[ "${thisEUSER}" != "root" ]] && [[ "${thisEUSER}" != "${pUSER[${pcnt}]}" ]]; then
         myrc=20;
      fi;                                   # (if(thisEUSER...))        #
      pcnt=$((pcnt + 1));                   # count this process        #
   done;                                    # (while(read...))          #

   #--------------------------------------------------------------------#
   # Run the incantation to start each component. This is adapted from  #
   # install.sh.                                                        #
   #--------------------------------------------------------------------#
   if [[ ${myrc} -eq 0 ]]; then
      p=0;
      while [[ ${p} -lt ${pcnt} ]]; do
         if [[ "${pPC[${p}]}" = "ms" ]]; then
            cmd="${CANDLEHOME}/bin/CandleServer start ${pINST[${p}]}";
         elif [[ "${pINST[${p}]}" = "None" ]]; then
            cmd="${CANDLEHOME}/bin/CandleAgent start ${pPC[${p}]}";
         elif [[ "${pPC[${p}]}" = "um" ]]; then
            cmd="${CANDLEHOME}/bin/CandleAgent -p ${pINST[${p}]} start um";
         else
            cmd="${CANDLEHOME}/bin/CandleAgent -o ${pINST[${p}]} start ${pPC[${p}]}";
         fi;

         if [[ "${thisRUSER}" != "${pUSER[${p}]}" ]]; then
            cmd="${su_cmd} - ${pUSER[${p}]} -c \"${cmd}\"";
         fi;

         eval "${cmd}";                     # run the command           #
         rc=${?};

         if [[ ${rc} -ne 0 ]]; then         # if command failed...      #
            myrc=21;                        # ...tell our caller        #
         fi;                                # (if($? != 0))             #

         p=$((p + 1));                      # skip to next process      #
      done;                                 # (while(p < pcnt))         #
   fi;                                      # (if(myrc==0))             #

   return ${myrc};                          # rc to caller              #
}                                           # (do_START())              #

#=======================================================================#
# main()                                                                #
#=======================================================================#

#-----------------------------------------------------------------------#
# Establish constants/globals.                                          #
#-----------------------------------------------------------------------#
main_rc=0;                                  # overall return code       #
unalias -a;                                 # avert alias interference  #

pgm=$(basename "${0}");                     # my full program name      #
pgmfn="${pgm%.sh}";                         # program file name only    #
bindir=`pwd`/$(dirname $0)
[ "$(print $0 | grep '\./')" = "" ] && [ "$(print $0 | grep '^/.*')" = "" ] && [ `pwd` = '/' ] && bindir=`pwd`$(dirname $0);
[ "$(print $0 | grep '^/.*')" != "" ] && bindir=$(dirname $0);

. ${bindir}/itmsh_trace_main;               # embed/start tracing       #

thisOS=$(uname -s);                         # get operating system name #
if [[ -a "/usr/xpg4/bin/id" ]]; then        # for older Solaris         #
   id_cmd="/usr/xpg4/bin/id";               # use POSIX id command      #
else
   id_cmd="id";                             # use std id command        #
fi;
thisRUSER=$(${id_cmd} -unr);                # get real user name        #
thisEUSER=$(${id_cmd} -un);                 # get effective user name   #

su_cmd=$(locateCMD su);                     # find su command           #

[[ -n "${CANDLEHOME}" ]] || CANDLEHOME=$(cd $(dirname ${0}) ; dirname $(pwd) );

#-----------------------------------------------------------------------#
# Initialize and parse parameter list. For this exercise, we are just   #
# gathering the options and verifying they are syntactically correct.   #
#-----------------------------------------------------------------------#
typeset iREQUEST="";                        # query|stop|start          #

if [[ ${main_rc} -eq 0 ]] && [[ ${#} -ne 1 ]]; then
   usage;
   main_rc=2;                               # unknown argument          #
else
   iREQUEST="${1}";                         # query|stop|start
fi;                                         # (if(main_rc=0)            #

#-----------------------------------------------------------------------#
# Check parameter values. For this exercise, we're making sure what     #
# they asked for is valid.                                              #
#-----------------------------------------------------------------------#
if [[ ${main_rc} -eq 0 ]]; then
   case "${iREQUEST}" in
      "query")
         do_QUERY;
         main_rc=${?};
         ;;
      "stop")
         do_STOP;
         main_rc=${?};
         ;;
      "start")
         do_START;
         main_rc=${?};
         ;;
      *)
         usage;
         main_rc=2;                         # unknown argument          #
   esac;                                    # (case(iREQUEST))          #
fi;                                         # (if(main_rc==0))          #

exit ${main_rc};                            # exit with return code     #