#!/bin/ksh
# ****************************************************************************
# * Licensed Materials - Property of IBM
# * 
# * OCO Source Materials - IBM Confidential
# * 5724-C04
# *
# * Copyright IBM Corporation 2007-2011 All Rights Reserved
# *
# * The source code for this program is not published or other-
# * wise divested of its trade secrets, irrespective of what has
# * been deposited with the U.S. Copyright Office.
# *****************************************************************************


###################################################################
#                         Constants                               #
###################################################################

LOGFILE=$CANDLEHOME/logs/itmsecure_$(date +%m%d%y%H%M%S).log
SECUREFILE=$CANDLEHOME/registry/SecureMode
DEBUG=

###################################################################
#                         Functions                               #
###################################################################

# Display message
# $1 - message type
# $2 - message to display
Err_report() {
   [[ -n "$DEBUG" ]] && set -x
   case $1 in
      #$NLS-E$ "$PROGNAME warning: $2,continuing ..."
	  warn) print -u2 "`nls_replace 'KCI0250I' \"$PROGNAME\" \"$2\"`"
            sleep 2
            return
            ;;
      #$NLS-E$ "$PROGNAME failure: $2."
	  fail) print -u2 "`nls_replace 'KCI0251I' \"$PROGNAME\" \"$2\"`"
            exit 2
            ;;
      info) print -u2 "$PROGNAME        : $2."
            return
            ;;
   esac
}

# Reads common group from SecureMode file
# it returns group name only if environment is secured
# example line from SecureMode - lock:itmuser:itmgroup
get_common_group()
{
  [[ -n "$DEBUG" ]] && set -x
  if [[ -f "$SECUREFILE" ]];
  then
    line=$(cat $SECUREFILE 2>/dev/null)
    mode=$(print $line | cut -f1 -d:)
    group=$(print $line | cut -f3 -d:)
    [[ "$mode" = "lock" ]] && [[ "$group" != "@none@" ]] && print "$group"
  fi
}

# Direct execution from install.sh
mode_install()
{
  [[ -n "$DEBUG" ]] && set -x
  # if PARM_FILE variable is set then we are in silent mode
  [[ -n "$PARM_FILE" ]] && SILENT_MODE=yes
  
  # secure mode enabled in install.sh
  if [[ -z "$SECURE_GROUP" && ! -f "$SECUREFILE" && -z "$SILENT_MODE" ]];
  then
  	
    #$NLS-E$ Product installation process creates the majority of directories and files with world write permissions. IBM Tivoli Monitoring provides...
    nls_replace 'KCI1344I'
  	
    while true
    do
      #$NLS-E$ "Do you want to run secureMain [ 1-yes, 2-no ; 2 is default]? \c"
      print "`nls_replace 'KCI1339I'` [ 1-`nls_replace 'KCI0197I'`, 2-`nls_replace 'KCI0198I'`; \"2\" `nls_replace 'KCI0199I'` ]? \c"
      read  
      [ -z "$REPLY" ] && REPLY=2
      case $REPLY in
        n*|N*|2) break
                 ;;
        y*|Y*|1) while true               
                 do
                   #$NLS-E$ "Enter common group name: \c"
                   print "`nls_replace 'KCI1337I'` \c"
                   read
                   if [[ -z $(print "$REPLY" | tr -s ' ') ]];
                   then
                     #$NLS-E$ "Group name can't be blank"
                     nls_replace 'KCI1338E'
                   else
                     # verify if group name is valid
                     testFile=/tmp/secureTest.$$
                     touch "$testFile" > /dev/null 2>&1
                     if [[ $? = 0 ]];
                     then
                       # testFile created without any problem, use chgrp to validate group                       
                       chgrp "$REPLY" "$testFile" > /dev/null 2>&1
                       rcGroup=$?
                       rm -f "$testFile" > /dev/null 2>&1
                       [[ "$rcGroup" = 0 ]] && valid=yes
                     else
                       # use alternative method when testFile can't be created, this check is only for non root user
                       [[ $userid != 0 ]] && [[ -n "$(groups | grep -w $REPLY)" ]] && valid=yes
                     fi
                     [[ -z "$valid" ]] && nls_replace 'KCI1346E' "$REPLY" $username && break # $NLS-E$ The specified group $1 is not a valid group or user $2 is not a member of this group
                     SECURE_GROUP=$REPLY
                     break                   
                   fi
                 done
                 
                 if [[ -z "$valid" ]] ; then
                   continue
                 fi
                 
                 break     
                 ;;
              *) #$NLS-E$ invalid response, please enter $1 or $2
                 nls_replace 'KCI1075I' "1" "2"
                 ;;
      esac
    done	     	      
  else
    if [[ -f "$SECUREFILE" ]];
    then
      SECURE_GROUP=$(get_common_group)
    fi
  fi

}

# UpdateAutoRun.sh execution
mode_autorun()
{
  [[ -n "$DEBUG" ]] && set -x
  SECURE_GROUP=$(get_common_group)
}

# Log message to log file
# $1 - message to append
log()
{
  print "$(date +%m.%d.%Y\ %T) $1" >> $LOGFILE
}

###################################################################
#                         Main Routine                            #
###################################################################

[[ -n "$DEBUG" ]] && set -x

bindir=`pwd`/$(dirname $0)
[ "$(print $0 | grep '\./')" = "" ] && [ "$(print $0 | grep '^/.*')" = "" ] && [ `pwd` = '/' ] && bindir=`pwd`$(dirname $0)
[ "$(print $0 | grep '^/.*')" != "" ] && bindir=$(dirname $0)

[ -z "$(whoami 2>/dev/null)" ] && alias whoami='id | cut -d"(" -f2 | cut -d")" -f1'

export TEXTDOMAINDIR=$bindir/../config/nls/msg
export PATH=$bindir:$PATH

PROGNAME=$(basename $0)

if [ ! -d "$CANDLEHOME" ] 
then
   #$NLS-E$ "Cannot set CANDLEHOME environment variable"
   Err_report fail "`nls_replace 'KCI0252E'`"
fi

# log execution - script name and parameters
log "Starting $0 $@"

MODE=$1

# check execution mode - install, when this script is executed from install.sh or autorun for UpdateAutoRun.sh
[[ "$MODE" = "install" ]] || [[ "$MODE" = "autorun" ]] || Err_report fail "`nls_replace 'KCI1335E' \"$MODE\"`" #$NLS-E$ Invalid option $MODE

# get user id
userid=$(id 2> /dev/null | cut -d"=" -f2 2> /dev/null | cut -d"(" -f1 2> /dev/null)
username=$(whoami)

log "UID: $userid"
log "USERNAME: $username"
log "CDNAME: $CDNAME"

# if id doesn't equal 0 set root mode variable 
[[ "$userid" = "0" ]] || [[ -z "$PARM_FILE" ]] || ROOT_MODE="-n"

case $MODE in
  "install") mode_install;;
  "autorun") mode_autorun;;
esac

if [[ -n "$SECURE_GROUP" ]];
then	
  command="$CANDLEHOME/bin/secureMain $ROOT_MODE -g $SECURE_GROUP lock"
  log "Executing command: $command"
  #$NLS-E$ Executing command: 
  nls_replace 'KCI1336I' "$command"  
  
  if [[ -z "$PARM_FILE" ]];
  then
    #$NLS-E$ "Enter the root password if prompted"
    nls_replace 'KCI0529I'
  fi  
  
  eval "$command" >> "$LOGFILE"
  rc=$?
  log "Return code = $rc"
  if [[ $rc -eq 0 ]];
  then
    #$NLS-E$ secureMain completed successfully
    nls_replace 'KCI1342I'
  else
    #$NLS-E$ secureMain completed with errors. Check $1 log for more details.
    nls_replace 'KCI1343E' "$LOGFILE"
    
    # display additional information how to fix this problem when running as non root user
    command_fix="$CANDLEHOME/bin/secureMain -g $SECURE_GROUP lock"
    [[ "$userid" = "0" ]] || nls_replace 'KCI1345I' "$command_fix" #$NLS-E$ This might be because you are running installation as non root user. Execute command ...
  fi  
fi
