#!/bin/ksh

# ****************************************************************************
# * Licensed Materials - Property of IBM
# *
# * 5724-C04
# *
# * Copyright IBM Corporation 2015 All Rights Reserved
# *
# ****************************************************************************
#
# Description:
#
# Extract all situations
# Touch embedded situations so that they will be recompiled
# Import only touched situations
#-----------------------------------------------------------------------------
# modifyEmbeddedSits.sh / modifyEmbeddedSits.cmd must be changed together.
#-----------------------------------------------------------------------------

Err_report() {
   case $1 in
      #$NLS-E$ "$pgm warning: $2,continuing ..."
      warn) print -u2 "$pgm `nls_replace 'KCI0520I'`: $2, `nls_replace 'KCI0521I'` ..."
            sleep 2
            return
            ;;
          #$NLS-E$ "$pgm failure: $2."
      fail) print -u2 "$pgm `nls_replace 'KCI0522I'`: $2."
            exit 2
            ;;
      info) print -u2 "$pgm        : $2."
            return
            ;;
   esac
}

Use_report() {
    print -u2 "\n$pgm [ -h install_dir ]"
    exit 2
}

###################################################################
#                         Main Routine                            #
###################################################################

# Remove any aliases defined.
unalias -a

pgm=$(basename "${0}");                     # my full program name      #
pgmfn="${pgm%.sh}";                         # program file name only    #
bindir=`pwd`/$(dirname $0)
[ "$(print $0 | grep '\./')" = "" ] && [ "$(print $0 | grep '^/.*')" = "" ] && [ `pwd` = '/' ] && bindir=`pwd`$(dirname $0);
[ "$(print $0 | grep '^/.*')" != "" ] && bindir=$(dirname $0);

. ${bindir}/itmsh_trace_main;               # embed/start tracing       #

export TEXTDOMAINDIR=$bindir/../config/nls/msg
export PATH=$bindir:$PATH

thisOS=$(uname -s);                         # get operating system name #

###################################################################
# Process the parameters.
###################################################################
while getopts :h: opt
do
  case $opt in
    h) CANDLEHOME=$OPTARG
       CANDLEHOME=$(eval print $CANDLEHOME)  # Process ~ if it was entered.
       ;;
    ?) Use_report
       ;;
  esac
done
shift $((OPTIND-1))

###################################################################
# Set and validate the installation directory.
###################################################################
[[ -n "$CANDLEHOME" ]] || CANDLEHOME=$(dirname $bindir)
if [[ ! -d "$CANDLEHOME" ]]; then
   #$NLS-E$ "Cannot set CANDLEHOME environment variable"
   Err_report fail "`nls_replace 'KCI0523E' CANDLEHOME`"
fi
if [[ $(print "$CANDLEHOME" | wc -w) -ne 1 ]]; then
   #$NLS-E$ "Blanks are not permitted in install_dir"
   Err_report fail "`nls_replace 'KCI0524E' install_dir`"
fi
if [[ ! (-d "$CANDLEHOME/bin" && -d "$CANDLEHOME/config" &&
      -d "$CANDLEHOME/logs" && -d "$CANDLEHOME/registry") ]]; then
   #$NLS-E$ "$CANDLEHOME is not a valid installation"
   Err_report fail "`nls_replace 'KCI0525E' $CANDLEHOME`"
fi
# Export the installation directory.
export CANDLEHOME

###################################################################
# Set and create the temporary directory.
###################################################################
TEMP=/tmp/$pgmfn.$$
mkdir -p $TEMP > /dev/null 2>&1
if [[ ! -d "$TEMP" ]]; then
   Err_report fail "Cannot create $TEMP directory"
fi

###################################################################
# Validate that tacmd is executable.
###################################################################
TACMD=$CANDLEHOME/bin/tacmd
if [[ ! -x "$TACMD" ]]; then
   Err_report fail "Cannot execute $TACMD"
fi


###################################################################
# See if we are logged in to the HUB TEMS
###################################################################
$TACMD listsystems -t em > /dev/null
RC=$?
[[ $RC -gt 0 ]] && Err_report fail " You must log into the server before running \"$pgm\"."

###################################################################
# Export all situations with distribution information.
###################################################################
$TACMD bulkexportsit -p $TEMP -d -f
RC=$?
[[ $RC -gt 0 ]] && Err_report fail " tacmd bulkexportsit failed: $RC \n\t\"$pgm\" terminated"

###################################################################
# Determine which situations need to be selected.
# Modify the selected situations.
###################################################################
# The following embedded situations can potentially be affected by
# the expand embedded situations change and are selected below:
# *IF *SIT SITA *EQ *TRUE
# *IF *SIT SITB *EQ *TRUE *AND *VALUE Process.User_Name *NE bozo
# *IF *SIT SITC *EQ *TRUE *AND *SIT SITB *EQ *TRUE 
# *IF *SIT SITA *EQ *TRUE *UNTIL ( *SIT SITB *EQ *TRUE )
#------------------------------------------------------------------
# The following embedded situations can NOT be affected by
# the expand embedded situations change and are NOT selected below:
# *IF *VALUE Process.User_Name *NE bozo
# *IF *VALUE Process.User_Name *NE bozo *UNTIL ( *SIT SITB *EQ *TRUE )
###################################################################
set -A SITLIST
SITexcludingUntil=0
LISTFILE=$TEMP/LISTFILE
rm -f $LISTFILE
for sitfile in $TEMP/Bulk/SITUATION/*/*.xml ; do

   # Check for *SIT outside of *UNTIL.  Only ignore *SIT inside of *UNTIL.
   check=`sed 's/\*UNTIL.*$//' $sitfile | grep '\*SIT'`

   if [[ -n "$check" ]]; then
      # No need to modify the sitFile. Just importing it marks it as changed.
      print $(basename $sitfile .xml) >> $LISTFILE
      SITexcludingUntil=$((SITexcludingUntil + 1))
   fi
done
Err_report info " Exported situation files with *SIT not inside *UNTIL: $SITexcludingUntil"

###################################################################
# Import the modified situations
###################################################################
if [[ -f "$LISTFILE" ]]; then
   $TACMD bulkimportsit -p $TEMP -l $LISTFILE -d -f
   RC=$?
   [[ $RC -gt 0 ]] && Err_report fail "tacmd bulkimportsit failed: $RC \n\t\"$pgm\" terminated"
fi

###################################################################
# Clean up
###################################################################
[[ -d "$TEMP" ]] && rm -rf $TEMP
