#!/bin/sh

#****************************************************************//
#* COPYRIGHT_COMMENT_TAG_BEGIN
#* Licensed Materials - Property of IBM (C)
#* Copyright IBM Corp. 2013. All Rights Reserved.
#* US Government Users Restricted Rights - Use, duplication,
#* or disclosure restricted by GSA ADP Schedule Contract with
#* IBM Corp
#* COPYRIGHT_COMMENT_TAG_END
#****************************************************************//

myname=`basename $0`
curdate=`date +"%Y%m%d_%H%M"`
verbose=0

runcmd() {
    if [ $verbose -eq 1 ]; then
        eval $*
        retval=$?
    else
        eval $* > /dev/null 2>&1
        retval=$?
    fi

    return $retval
}

usage() {
    echo "Usage: $myName [-v] [-o <path_to_output_directory>] -i <path_to_installation_media_directory> -d <path_to_candle_home> -p <product_code1> [-p <product_code2>]"
    echo "       $myName -h"
}

cleanup() {
    runcmd rm -f $prstmplog
}

if [ $# -eq 0 ]; then
    usage
    exit 1
fi

# Solaris has different path for POSIX-compliant grep
GREPCMD=grep
if [ `uname -s` = "SunOS" ]; then
	GREPCMD="/usr/xpg4/bin/grep"
fi

while getopts "vo:i:d:p:h" opt
do
    case $opt in
        v)  verbose=1
            ;;
        o)  userprsdir=$OPTARG
            ;;
        i)  installdvd=$OPTARG
            ;;
        d)  candlehome=$OPTARG
            ;;
        p)  # Three types of product codes are accepted: 2-letter, 2-letter
			# concatenated with 32 to indicate 32-bit version and 2-letter with
			# 3264 concatenated to indicate that both 32 and 64-bit versions
			# should be checked for prerequisites
            if [ `echo $OPTARG | wc -c` -eq 3 ]; then
                productcodes="$productcodes $OPTARG"
        	elif [ `echo $OPTARG | $GREPCMD -E '^[a-zA-Z0-9]{2}32$' >/dev/null 2>&1; echo $?` -eq 0 ]; then
        		pc=`echo $OPTARG | cut -c 1,2 | tr [a-z] [A-Z] 2>/dev/null`
        		productcodes="$productcodes $pc"
        		eval K${pc}_32=true
        		export K${pc}_32
        	elif [ `echo $OPTARG | $GREPCMD -E '^[a-zA-Z0-9]{2}3264$' >/dev/null 2>&1; echo $?` -eq 0 ]; then
        		pc=`echo $OPTARG | cut -c 1,2 | tr [a-z] [A-Z] 2>/dev/null`
        		productcodes="$productcodes $pc"
        		eval K${pc}_3264=true
        		export K${pc}_3264
        	fi
            ;;
        h)  usage
            exit 0
            ;;
        *)  usage
            exit 1
            ;;
    esac
done

# Validate input arguments

# Invalid installation media directory path disables prereqchecker usage.
if [ ! -d "$installdvd" ]; then
    runcmd usage
    exit 1
fi

if [ ! -d "$candlehome" ]; then
    runcmd usage
    exit 1
fi

# This is the default location for storing Prerequisite Scanner's output.
prsdir=${candlehome}/tmp/ITM_PRS_${curdate}

if [ -n "$userprsdir" ]; then
	# Search for relative paths (those with ./ at the beginning). They need
	# special care as prsdir is directly supplied to prereq_checker.sh and it
	# will create output directory in different current working dir than
	# prereqlauncher.sh thus confusing user as to the location of the output dir.
	echo $userprsdir | grep "^\.\/" > /dev/null 2>&1
	if [ $? -eq 0 ]; then
		userprsdir=`pwd -P`/${userprsdir}
	fi
    	prsdir=$userprsdir
fi

mkdir -p ${prsdir}
if [ ! -d ${prsdir} ]; then
	echo "Unable to create directory $prsdir. Check permissions." 1>&2 # $NON-NLS-1$
    exit 1
fi

prereqchecker=${installdvd}/unix/prereqchecker/prereq_checker.sh
prstmplog=`echo "${prsdir}/.prstmp.log" | tr -s "/"`
prsresult=`echo "${prsdir}/prs_result.txt" | tr -s "/"`
productcodes=`echo $productcodes | tr [a-z] [A-Z] 2>/dev/null`

# Call prereq_checker.sh for all components at once. This allows cumulative
# checking for disk space and memory
pcs=""
excludedpcs=""
for pc in $productcodes;
do
	# Check if PRS configuration file exists for a given component. If it doesn't
	# then exclude that component from the check.
	if [ -f ${installdvd}/unix/prereqchecker/K${pc}_codename.cfg ]; then
		pcs="${pcs}K${pc},"
	else
		excludedpcs="${excludedpcs} ${pc}"
	fi
	
done
# Remove last comma
pcs=`echo $pcs | sed 's/,$//'`

pcoutputdir=`echo "${prsdir}/COMMON/" | tr -s "/"`

# CMVC defect 206742: prereq checker will return an error when run with empty pcs
if [ -n "$pcs" ]; then
	$prereqchecker "${pcs}" PATH=$candlehome outputdir="${pcoutputdir}" > ${prstmplog} 2>&1
	prsres=$?

	if [ ! -f ${pcoutputdir}/result.txt ]; then
		echo "ERR;ERROR;${pcoutputdir}"
		cleanup
		exit 0
	fi
	overallstring=`grep -i "Overall Result" ${pcoutputdir}/result.txt | cut -d "(" -f2 | cut -d ")" -f1 | tr -d [0-9] | tr -d " " 2>/dev/null`
	overallresult=`grep -i "Overall Result" ${pcoutputdir}/result.txt | cut -d ":" -f 2 | cut -d "(" -f1 | tr -d " "`

	while [ -n "$overallstring" ]; do
		pair=`echo $overallstring | cut -d "," -f1 2>/dev/null`
		comp=`echo $pair | cut -d ":" -f1 | sed s'/^[kK]//' 2>/dev/null`
		result=`echo $pair | cut -d ":" -f2 2>/dev/null`
		echo "${comp};${result};${pcoutputdir}" | tee -a $prsresult
		overallstring=`echo $overallstring | sed s/$pair// | sed s'/^,//' 2>/dev/null`
	done
else
	# In case there are no PRS config files then fake overall PASS result so
	# CandleInstall.java can go further.
	overallresult="PASS"
fi

for pc in $excludedpcs; do
	echo "$pc;N/A;${pcoutputdir}" | tee -a $prsresult
done

if [ "$overallresult" = "FAIL" ]; then
	echo "pc;OVERALLFAIL;${pcoutputdir}"
fi
if [ "$overallresult" = "PASS" ]; then
	echo "pc;OVERALLPASS;${pcoutputdir}"
fi

cleanup
exit 0