#!/bin/ksh

# ****************************************************************************
# * Licensed Materials - Property of IBM
# *
# * 5724-C04
# *
# * Copyright IBM Corporation 2015 All Rights Reserved
# *
# ****************************************************************************
#
# Description:
#
# Set the TEMS variable KMS_EXPAND_EMBEDDED_SIT to Y or N
# Reconfigure the TEMS
# Recycle the TEMS
#-----------------------------------------------------------------------------
# setEmbeddedSitsFlag.sh / setEmbeddedSitsFlag.cmd must be changed together.
#-----------------------------------------------------------------------------

Err_report() {
   case $1 in
      #$NLS-E$ "$pgm warning: $2,continuing ..."
      warn) print -u2 "$pgm `nls_replace 'KCI0520I'`: $2, `nls_replace 'KCI0521I'` ..."
            sleep 2
            return
            ;;
          #$NLS-E$ "$pgm failure: $2."
      fail) print -u2 "$pgm `nls_replace 'KCI0522I'`: $2."
            exit 2
            ;;
      info) print -u2 "$pgm        : $2."
            return
            ;;
   esac
}

Use_report() {
    print -u2 "\n$pgm [ -h install_dir ] {enable|disable}"
    exit 2
}

###################################################################
#                         Main Routine                            #
###################################################################

# Remove any aliases defined.
unalias -a

pgm=$(basename "${0}");                     # my full program name      #
pgmfn="${pgm%.sh}";                         # program file name only    #
bindir=`pwd`/$(dirname $0)
[ "$(print $0 | grep '\./')" = "" ] && [ "$(print $0 | grep '^/.*')" = "" ] && [ `pwd` = '/' ] && bindir=`pwd`$(dirname $0);
[ "$(print $0 | grep '^/.*')" != "" ] && bindir=$(dirname $0);

. ${bindir}/itmsh_trace_main;               # embed/start tracing       #

export TEXTDOMAINDIR=$bindir/../config/nls/msg
export PATH=$bindir:$PATH

thisOS=$(uname -s);                         # get operating system name #

###################################################################
# Process the parameters.
###################################################################
while getopts :h: opt
do
   case $opt in
      h) CANDLEHOME=$OPTARG
         CANDLEHOME=$(eval print $CANDLEHOME)  # Process ~ if it was entered.
         ;;
      ?) Use_report
         ;;
   esac
done
shift $((OPTIND-1))

ACTION=$1

case $ACTION in
   enable) FLAG=Y
      ;;
   disable) FLAG=N
      ;;
   "") FLAG=""
      Err_report info "\"enable\" or \"disable\" must be specified"
      Use_report
      ;;
   *) FLAG=""
      Err_report info "Invalid action specified: $ACTION"
      Use_report
      ;;
esac

###################################################################
# Set and validate the installation directory.
###################################################################
[[ -n "$CANDLEHOME" ]] || CANDLEHOME=$(dirname $bindir)
if [[ ! -d "$CANDLEHOME" ]]; then
   #$NLS-E$ "Cannot set CANDLEHOME environment variable"
   Err_report fail "`nls_replace 'KCI0523E' CANDLEHOME`"
fi
if [[ $(print "$CANDLEHOME" | wc -w) -ne 1 ]]; then
   #$NLS-E$ "Blanks are not permitted in install_dir"
   Err_report fail "`nls_replace 'KCI0524E' install_dir`"
fi
if [[ ! (-d "$CANDLEHOME/bin" && -d "$CANDLEHOME/config" &&
      -d "$CANDLEHOME/logs" && -d "$CANDLEHOME/registry") ]]; then
   #$NLS-E$ "$CANDLEHOME is not a valid installation"
   Err_report fail "`nls_replace 'KCI0525E' $CANDLEHOME`"
fi
# Export the installation directory.
export CANDLEHOME

###################################################################
# Set and create the temporary directory.
###################################################################
TEMP=/tmp/$pgmfn.$$
mkdir -p $TEMP > /dev/null 2>&1
if [[ ! -d "$TEMP" ]]; then
   Err_report fail "Cannot create $TEMP directory"
fi

###################################################################
# Validate that itmcmd is executable.
###################################################################
ITMCMD=$CANDLEHOME/bin/itmcmd
if [[ ! -x "$ITMCMD" ]]; then
   Err_report fail "Cannot execute $ITMCMD"
fi

###################################################################
# Validate that kmsenv exists.
###################################################################
KMSENV=$CANDLEHOME/config/.ConfigData/kmsenv
[[ -f "$KMSENV" ]] || Err_report fail "No TEMS is installed"

###################################################################
# Get the TEMS name
###################################################################
CELLNAME=`grep '|CELLNAME|' $KMSENV | grep -v '^default'`
TEMS=""
oldIFS="$IFS"
for cell in $CELLNAME ; do
   IFS="|"
   set -A parts $cell
   if [[ ${parts[0]} == ${parts[2]} ]]; then
      TEMS=${parts[2]}
      break
   fi
done
IFS="$oldIFS" 
[[ -z "$TEMS" ]] && Err_report fail "Unable to determine TEMS name"

###################################################################
# Build the response file to enable or disable IV74529
###################################################################
RESPONSE=$TEMP/enable.rsp
print "CUSTOM#KMS_EXPAND_EMBEDDED_SIT=$FLAG" >$RESPONSE

###################################################################
# Reconfigure the TEMS
###################################################################
$ITMCMD config -S -t $TEMS -p $RESPONSE
RC=$?
[[ $RC -gt 0 ]] && Err_report fail " $TEMS configuration failed: $RC \n\t\"$pgm\" terminated"

###################################################################
# Recycle the TEMS
# Don't call "Err_report fail" so that we still try to start the TEMS even if it is not running.
# Don't call "Err_report fail" so that we still clean up even if the TEMS takes too long to start.
###################################################################
$ITMCMD server stop $TEMS
$ITMCMD server start $TEMS

###################################################################
# Clean up
###################################################################
[[ -d "$TEMP" ]] && rm -rf $TEMP
