#!/bin/ksh

#------------------------------------------
# Enable support for TEC EIF event emitter
#
# - create directory CANDLEHOME/platform/pc/tables/EIFLIB
# - create directory CANDLEHOME/platform/pc/tables/EIFLIB/rb
# - add variable EIFLIB to agent .ini file
#------------------------------------------

enableEif()
{
  pc=$1

  # determine agent binarch
  for pcVerFile in `ls $candlehomedir/registry/$pc*.ver 2>/dev/null | grep -v ${pc}t.*\.ver`;
  do
    pcVerFile=${pcVerFile##*/}
    
    binArch=${pcVerFile%.ver}  # Remove the extension.
    binArch=$(print $binArch | cut -c3-)    # Remove the product code.
  
    # create eif directories
    if [ ! -d "$candlehomedir/$binArch/$pc/tables/EIFLIB" ];
    then
      # $NLS-E$ Creating directory '$1' for component '$2'
      nls_replace 'KCI1282I' "$candlehomedir/$binArch/$pc/tables/EIFLIB" "${pc}"
      mkdir -p "$candlehomedir/$binArch/$pc/tables/EIFLIB" 2>/dev/null
      # $NLS-E$ Error creating directory '$1'
      [ $? -eq 0 ] || nls_replace 'KCI1285E' "$candlehomedir/$binArch/$pc/tables/EIFLIB";
    else
      # $NLS-E$ Directory '$1'  for agent '$2' already exists
      nls_replace 'KCI1288I' "$candlehomedir/$binArch/$pc/tables/EIFLIB" "${pc}"
    fi
  done

  # update agent .ini file
  eifEnabled=$(grep EIFLIB= $candlehomedir/config/$pc.ini 2>/dev/null)
  if [ -z "$eifEnabled" ];
  then
    # just in case, make .ini file backup
    cp -f $candlehomedir/config/$pc.ini $candlehomedir/config/$pc.ini.bak 2>/dev/null
    print >> $candlehomedir/config/$pc.ini 2>/dev/null
    print "EIFLIB=\$CANDLEHOME\$/\$BINARCH\$/\$PRODUCTCODE\$/tables/EIFLIB" >> $candlehomedir/config/$pc.ini 2>/dev/null
  fi
}

# These components do not use 'localconfig' directory
exclusionList="ms cq cj cw iw iv kf jr gs ci ax as ii"

# These components do not use 'kt1depot' directory
kt1exclusionList="ms cq cj cw iw iv kf jr gs ci ax as ii"

# ------------------------
# Initialize variables
#-------------------------

SYNCLOCALCONFSCRIPTNAME=$0

bindir=`pwd`/$(dirname $SYNCLOCALCONFSCRIPTNAME)
[ "$(print $SYNCLOCALCONFSCRIPTNAME | grep '\./')" = "" ] \
            && [ "$(print $SYNCLOCALCONFSCRIPTNAME | grep '^/.*')" = "" ] \
            && [ `pwd` = '/' ] \
            && bindir=`pwd`$(dirname $SYNCLOCALCONFSCRIPTNAME)
[ "$(print $SYNCLOCALCONFSCRIPTNAME | grep '^/.*')" != "" ] && bindir=$(dirname $SYNCLOCALCONFSCRIPTNAME)

PATH=$PATH:"${bindir}"

candlehomedir=$(cd $bindir/..; pwd)
regdir="${candlehomedir}/registry"
confdir="${candlehomedir}/config"
localconfdir="${candlehomedir}/localconfig"
kt1depotdir="${candlehomedir}/kt1v3depot"

retcode=0

# ------------------------
# Find installed products
# ------------------------

installedProducts=$(cd $regdir; ls *.ver | grep -v "^ci" | grep -v "tms.ver" | grep -v "tpd.ver" | grep -v "tps.ver" | grep -v "tpw.ver" | grep -v "tpa.ver" | cut -c 1-2 | sort -u)

# -----------------------
# Remove localconfig directories for agents already uninstalled if the dirs are empty
# -----------------------

# $NLS-E$ ... Checking for configuration directories to be removed
nls_replace 'KCI1286I'

if [[ -d "${localconfdir}" ]]
then
  for pcDir in `ls "${localconfdir}" | grep "^..$"`
  do
    # Skip if the component was found on exclusion list
    print "${exclusionList}" | grep "${pcDir}" >/dev/null 2>&1 && continue

    if [[ -d "${localconfdir}/${pcDir}" ]]
    then
      print "${installedProducts}" | grep "${pcDir}" >/dev/null 2>&1 && 
      { 
        # $NLS-E$ Component '$1' is still installed, directory '$2' will not be removed.
        nls_replace 'KCI1279I' "${pcDir}" "${localconfdir}/${pcDir}";
        continue; 
      }
    
      [[ $(find "${localconfdir}/${pcDir}" ! -type d) = "" ]] && 
      { 
        # $NLS-E$ Directory '$1' will be removed due to no files found and component '$2' not installed.
        nls_replace 'KCI1280I' "${localconfdir}/${pcDir}" "${pcDir}";
      
        # Try removing the directory and report error if failed
        # $NLS-E$ Error occurred when removing directory '$1'
        rm -rf "${localconfdir}/${pcDir}" || 
        {
          retcode=1;
          nls_replace 'KCI1284E' "${localconfdir}/${pcDir}";
        }
        continue; 
      }

      # $NLS-E$ Directory '$1' will not be removed due to files found.
      nls_replace 'KCI1281I' "${localconfdir}/${pcDir}"
    fi
  done
fi

# ------------------------
# Remove kt1depot directories for agents already uninstalled if the dirs are empty
# ------------------------

if [[ -d "${kt1depotdir}" ]];
then
  for pcDir in `ls "${kt1depotdir}" 2>/dev/null`
  do
    # Skip if the component was found on exclusion list
    print "${kt1exclusionList}" | grep "${pcDir}" >/dev/null 2>&1 && continue
	    
    if [[ -d "${kt1depotdir}/${pcDir}" ]]
    then
      print "${installedProducts}" | grep "${pcDir}" >/dev/null 2>&1 && 
      { 
        # $NLS-E$ Component '$1' is still installed, directory '$2' will not be removed.
        nls_replace 'KCI1279I' "${pcDir}" "${kt1depotdir}/${pcDir}";
        continue; 
      }
    
      [[ $(find "${kt1depotdir}/${pcDir}" ! -type d) = "" ]] &&
      { 
        # $NLS-E$ Directory '$1' will be removed due to no files found and component '$2' not installed.
        nls_replace 'KCI1280I' "${kt1depotdir}/${pcDir}" "${pcDir}";
      
        # Try removing the directory and report error if failed
        # $NLS-E$ Error occurred when removing directory '$1'
        rm -rf "${kt1depotdir}/${pcDir}"
        [[ $? -eq 0 ]] || nls_replace 'KCI1284E' "${kt1depotdir}/${pcDir}";
        continue; 
      }

      # $NLS-E$ Directory '$1' will not be removed due to files found.
      nls_replace 'KCI1281I' "${kt1depotdir}/${pcDir}"
    fi
  done
fi



# ------------------------
# Create directory structure for installed components
# ------------------------

# $NLS-E$ ... Checking for configuration directories to be created
print
nls_replace 'KCI1287I'

for pcCode in `print "${installedProducts}"`
do
  # Prepare kt1depot directory for component
  if [[ -f "${confdir}/${pcCode}.ini" ]];
  then
    # Skip components from exclusion list
    excludepc="$(print "${kt1exclusionList}" | grep "${pcCode}" 2>/dev/null)"
    if [[ -z "$excludepc" ]]
    then	
      # Skip if required directories already exist
      if [[ -d "${kt1depotdir}/${pcCode}" ]];
      then
        nls_replace 'KCI1288I' "${kt1depotdir}/${pcCode}" "${pcCode}"
      else		    
        # $NLS-E$ Creating directory '$1' for component '$2'
        nls_replace 'KCI1282I' "${kt1depotdir}/${pcCode}" "${pcCode}"	    
        # Try creating the directory and report error if failed
        # $NLS-E$ Error creating directory '$1'
        mkdir -p "${kt1depotdir}/${pcCode}"
        [[ $? -eq 0 ]] || nls_replace 'KCI1285E' "${kt1depotdir}/${pcCode}";
      fi
    fi  	      
  fi

  # Skip if the component was found on exclusion list
  print "${exclusionList}" | grep "${pcCode}" >/dev/null 2>&1 && continue 
 
  # Skip if the component is not a runnable agent
  [[ -f "${confdir}/${pcCode}.ini" ]] || continue

  # Enable TEC EIF event emitter support
  enableEif $pcCode

  # Skip if required directories already exist
  [[ -d "${localconfdir}/${pcCode}/html" ]] && 
  {
    nls_replace 'KCI1288I' "${localconfdir}/${pcCode}/html" "${pcCode}"
    continue;
  }

  # $NLS-E$ Creating directory '$1' for component '$2'
  nls_replace 'KCI1282I' "${localconfdir}/${pcCode}/html" "${pcCode}"

  # Try creating the directory and report error if failed
  # $NLS-E$ Error creating directory '$1'
  mkdir -p "${localconfdir}/${pcCode}/html" || 
  {
    retcode=1;
    nls_replace 'KCI1285E' "${localconfdir}/${pcCode}/html";
  }
    
done

exit "${retcode}"
