#!/bin/ksh
#
# ****************************************************************************
# * Licensed Materials - Property of IBM
# * 
# * 5724-C04
# *
# * Copyright IBM Corporation 2007-2015 All Rights Reserved
# *****************************************************************************
#
# Script removes installed products
# Return codes:
#   0 = OK
#   1 = no products to delete
#   2 = failed
#   3 = failed due to running processes found
#

PROGNAME=$(basename $0)
UsrCmd="$0 $*"

typeset AWK="awk"
# need to use nawk if it's available
type nawk >/dev/null 2>&1
[[ $? -eq 0 ]] && AWK=nawk

Err_report() {
   case $1 in
      #$NLS-E$ warn) print -u2 "$PROGNAME warning: $2,continuing ..."
      warn) print -u2 "`nls_replace 'KCI0200I' $PROGNAME \"$2\"`"
            sleep 2
            return
            ;;
      #$NLS-E$ fail) print -u2 "$PROGNAME failure: $2."
      fail) print -u2 "$PROGNAME `nls_replace 'KCI0201I'`: $2";         #214546
            clean_up 2
            ;;
      info) print -u2 "$PROGNAME        : $2";                          #214546
            return
            ;;
   esac
}

Use_report() {
    print -u2 "$PROGNAME"
    print -u2 "$PROGNAME [-f] [-i] [-h install_dir] product platformCode"
    print -u2 "$PROGNAME [-h install_dir] REMOVE EVERYTHING"
    #$NLS-E$ "\t-f Force delete, suppress confirmation messages and prompts."
    print -u2 "\t-f `nls_replace 'KCI0202I'`"
    #$NLS-E$ "\t-i Ignore all running processes."
    print -u2 "\t-i `nls_replace 'KCI0203I'`"
    #$NLS-E$ "\t'product' A two-letter code for the product to be uninstalled."
    print -u2 "\t'product' `nls_replace 'KCI0204I'`"
    #$NLS-E$ "\t'platformCode' Platform code (such as aix513, sol286, hp11, etc.) for the product."
    print -u2 "\t'platformCode' `nls_replace 'KCI0205I'`"
    #$NLS-E$ "\tREMOVE EVERYTHING are special keywords."
    print -u2 "\tREMOVE EVERYTHING `nls_replace 'KCI1266I'`"
    #$NLS-E$ "\t WARNING, specifying these keywords will remove the entire contents of the ITM home directory without further prompting or confirmation."
    print -u2 "\t\t`nls_replace 'KCI1267I'`"
    exit 2
}

#get product description
get_desc() {
  pc=$1
  desc=$(grep "^desc" $(ls ${CANDLEHOME}/registry/${pc}+(ai|hp|li|lp|ls|lx|so)*.ver | head -1) | cut -d"=" -f2 | sed 's/^ //g'); #IV27849
  [[ -z "$desc" ]] && desc=$(grep "^$pc" $CANDLEHOME/registry/proddsc.tbl | head -1 | cut -f2 -d"|")
  [[ -z "$desc" ]] && desc=$pc
  print $desc
}

# Uninstall a product and its prereqs
#
# This function is called recursively and we have to be sure the variables are used locally,
# but for SLES10 the variables 'pc', 'platform' are not local and are rewritten when the function is called recursively.
# The function is executed in its own shell by placing parenthesis within the function brackets.
# The function gets its own copy of the environment when is called and any changes are not reflected in the calling script which is its own function.
uninstall_product() { (
  typeset pc=$1
  typeset platform=$2
  typeset skipComp=$3  # skip these components when checking for shared prereq
  typeset sharedPrereq="" 

  [[ "$skipComp" = "" ]] && skipComp=$pc$platform

  # Nuke CandleHome!
  if [[ "$pc" = "REMOVE" && "$platform" = "EVERYTHING" ]]
  then
    #$NLS-E$ "Removing ITM home: $CANDLEHOME\n"
	print "`nls_replace 'KCI0206I'`: $CANDLEHOME\n"
    cd $originalDir
    cmdHome=$(cd $(dirname $0) ; dirname $(pwd) )
    if [[ $(pwd) = "$CANDLEHOME/bin" || "$CANDLEHOME" = "$cmdHome" ]]
    then
      #$NLS-E$ "$CANDLEHOME/bin may have to be removed manually after this script completes"
	  Err_report warn "`nls_replace 'KCI0207I' $CANDLEHOME/bin `"
    fi
    #$NLS-E$ Err_report warn "Purging $CANDLEHOME"
	Err_report warn "`nls_replace 'KCI0208I' $CANDLEHOME`"


    if [ -f $CANDLEHOME/registry/AutoStart ]
    then
       remove_autostart
    fi

    hp_forcewait

    rm -rf $CANDLEHOME
    if [[ -n "${CANDLEDATA}" ]]; then                                   #IV67523
       rm -rf "${CANDLEDATA}";                                          #IV67523
    fi;                                                                 #IV67523

    remove_links
    clean_up
  fi

  # Not installed if there's no VER file
  cd $regDir
  vFile=$pc$platform.ver
  #$NLS-E$ "product \"$pc\" for platform \"$platform\" not installed in $CANDLEHOME"
  if [ ! -f $vFile ] 
  then 
    Err_report warn "`nls_replace 'KCI0209I' "$pc" "$platform" "$CANDLEHOME"`"
    return
  fi

  # Remove Watchdog default configuration files and pas.dat
  if [ "$pc" = "ux" ] || [ "$pc" = "lz" ] 
  then
    rm -f $CANDLEHOME/config/CAP/k"$pc"_default.xml 2> /dev/null
    rm -f $CANDLEHOME/config/CAP/kca_default.xml 2> /dev/null
    rm -f $CANDLEHOME/config/CAP/pas.dat 2> /dev/null
    rm -f $CANDLEHOME/config/CAP/kwgcap.xsd 2> /dev/null
  else
    rm -f $CANDLEHOME/config/CAP/k"$pc"_default.xml 2> /dev/null
  fi
                                                                        #IV67523
  #---------------------------------------------------------------------#IV67523
  # Clean up CANDLEDATA-specific stuff.                                 #IV67523
  #---------------------------------------------------------------------#IV67523
  if [[ -n "${CANDLEDATA}" ]]; then                                     #IV67523
    if [ "$pc" = "ux" ] || [ "$pc" = "lz" ]; then                       #IV67523
       unlink "${CANDLEHOME}/kca" 2> /dev/null;                         #IV67523
       rm -rf "${CANDLEDATA}/kca" 2> /dev/null;                         #IV67523
    fi;                                                                 #IV67523
    rm -rf "${CANDLEDATA}/ATTRLIB/${pc}" 2> /dev/null;                  #213940
    rm -rf "${CANDLEDATA}/EIFLIB/${pc}" 2> /dev/null;                   #213940
    rm -rf "${CANDLEDATA}/hist/${pc}" 2> /dev/null;                     #IV67523
    rm -rf "${CANDLEDATA}/psit/${pc}" 2> /dev/null;                     #IV67523
  fi;                                                                   #IV67523
                                                                        #IV67523
  #---------------------------------------------------------------------#IV67523
  # Clean up agent application exits in CANDLEHOME/bin.                 #IV67523
  #---------------------------------------------------------------------#IV67523
  rm -rf "${CANDLEHOME}/bin/"*"_${pc}.sh" 2> /dev/null;                 #IV67523

  # Get each prereq for the product (except ci and jr)
  prereqs=$(grep "^preReq" $vFile | grep -v " ci" | grep -v " jr" | grep -v " ui" | grep -v " none" |cut -f1 -d"|"|cut -f3 -d " ")

  # remove as with TEMS
  if [[ "$pc" = "ms" && -n "$(ls $CANDLEHOME/registry/as$platform.ver 2>/dev/null)" ]];
  then
    asdesc=$(get_desc as)
    msdesc=$(get_desc ms)
    #$NLS-E$ $asdesc will be removed with $msdesc
    nls_replace 'KCI1361I' "$asdesc" "$msdesc"
    prereqs="as$platform $prereqs"
  fi

  # Delete prereqs
  for pre in $(print $prereqs)
  do
    prePC=$(print $pre|cut -c1-2)
    preArch=${pre#$prePC}
    # check if this prereq component are shared as prereq by other component
    sharedPrereq=""
    for comp in $(ls *.ver | grep -v "^ci" | grep -v "^jr" | grep -v "^ui" )
    do
      comp=${comp%.ver}
      skip=$(print $skipComp | grep "$comp")
      if [ "$skip" != "" ]
      then
        # this component should be skipped,
        # because it is going to be removed
        continue
      fi
      tmpSharedPrereq=$(grep " = $pre" ${comp}.ver)
      [[ -n "$tmpSharedPrereq" ]] && sharedPrereq="$sharedPrereq $comp"
    done
    #sharedPrereq=$(grep " = $pre" $(ls *.ver|grep -v "$vFile"))
    # remove this prereq component recursively
    if [ -z "$sharedPrereq" ]
    then
      # add this prereq component to the skip list,
      # for it is going to be removed
      have=$(print $skipComp | grep  "^$prePC$preArch")
      [[ "$have" = "" ]] && skipComp="$skipComp $prePC$preArch"
      uninstall_product $prePC $preArch "$skipComp"
    fi
  done

  # Remove the product
  # OS agent is slow to down go, wait for it
  if [ "$pc" = "ux" ]
  then
    hp_forcewait
  fi
   
  remove_product $pc $platform 
  update_autostart $pc

  # Synchronize/remove 'localconfig' directories for product(s) just removed.
  # Keep stderr printing to the screen to catch possible cricital errors
  $CANDLEHOME/bin/syncLocalConfDirs.sh >/dev/null
) }


update_autostart()
{
   if [ "$(whoami)" = "root" ]; then
    if [ -f $CANDLEHOME/registry/AutoStart ]; then
	editString1="start $pc"
	editString2="stop $pc"
	filenum=`cat $CANDLEHOME/registry/AutoStart`
	CurRegPath=$CANDLEHOME/registry
	ArchTbl=$CurRegPath/archdsc.tbl
	DynShl=$CANDLEHOME/bin/dynarch.shl
	. $DynShl $ArchTbl 2>/dev/null
  
	os=$(print $cArch|cut -c1-2)
	[ "$os" = li ] && rcFile="/etc/init.d/ITMAgents$filenum"
	[ "$os" = lx ] && rcFile="/etc/init.d/ITMAgents$filenum"
	[ "$os" = lp ] && rcFile="/etc/init.d/ITMAgents$filenum"
	[ "$os" = ls ] && rcFile="/etc/init.d/ITMAgents$filenum"
	[ "$os" = hp ] && rcFile="/sbin/init.d/ITMAgents$filenum"
	[ "$os" = so ] && rcFile="/etc/init.d/ITMAgents$filenum"
	[ "$os" = ai ] && rcFile=/etc/rc.itm$filenum

	  # construct a sed script for deleting lines                       #214546
	  editString="";                                                    #214546
	  if [[ -n "$(grep "$editString1" $rcFile 2>/dev/null)" ]]; then    #214546
	     editString="/${editString1}/d;";                               #214546
	  fi                                                                #214546
	  if [[ -n "$(grep "$editString2" $rcFile 2>/dev/null)" ]]; then    #214546
	     editString="${editString}/${editString2}/d";                   #214546
	  fi                                                                #214546
	  if [[ -n "${editString}" ]]; then                                 #214546
	     update_rcFile
	     if [[ $checkit != 0 ]]; then
		  #$NLS-E$ Delete of agent start/stop records failed.
	      Err_report fail "`nls_replace 'KCI1168E'`"
	     else
		  #$NLS-E$ Delete of agent start/stop records successful.
	      Err_report info "`nls_replace 'KCI1169I'`"
	     fi
	  fi
  
      fi
   fi
}

update_rcFile()
{
   SEDscript=$CANDLETMP/SEDscript
   print "$editString" > $SEDscript
   checkit=0
   tmpRcFile=$CANDLETMP/rcFile.new.$$
   cp -f $rcFile $tmpRcFile
   checkit=$?
   if [[ $checkit = 0 ]]; then
     sed -f $SEDscript $tmpRcFile > $rcFile
     checkit=$?
     if [[ $checkit != 0 ]]; then
       cp -f $tmpRcFile $rcFile
     fi
   fi
   rm -f $tmpRcFile
   rm -f $SEDscript
}

# Remove a product or a prereq (component)
remove_product() {
  # set -x                                  # enable shell tracing      #IZ92223
  product=$1
  binarch=$2

  name=$(get_desc $product)

  typeset regdir="${CANDLEHOME}/registry";                              #IZ92223
  typeset vfile="${regdir}/${product}${binarch}.ver";                   #IZ92223
  typeset varch;                            # arch from ver file        #IZ92223
  typeset varchdir;                         # CANDLEHOME/varch/product  #IZ92223
  typeset rm_varch;                         # 1=rm needed,0=skip        #IZ92223
  typeset rm_binarch;                       # 1=rm needed,0=skip        #IZ92223
                                                                        #IZ92223
  #---------------------------------------------------------------------#IZ92223
  # If there's a ver file, do two things:                               #IZ92223
  #                                                                     #IZ92223
  # a) Remove the directory that corresponds to each runArch that is    #IZ92223
  #    defined in the ver file. But ONLY do that if that runArch is not #IZ92223
  #    represented by its own ver file.                                 #IZ92223
  # b) Remove the ver file itself.                                      #IZ92223
  #---------------------------------------------------------------------#IZ92223
  # For the general case, xx<arch>.ver has a one-to-one relationship    #IZ92223
  # with its directory CANDLEHOME/<arch>/xx. However, for at least one  #IZ92223
  # case (kf=Eclipse Help Server), this is not true. The kf component   #IZ92223
  # can have multiple directories for a given kf<arch>.ver file.        #IZ92223
  #                                                                     #IZ92223
  # For example, when package kfaix533 is installed, files are created  #IZ92223
  # in CANDLEHOME/aix533/kf. But, other components (the TEPS plus any   #IZ92223
  # TEPS application support) create files in CANDLEHOME/aix536/kf. The #IZ92223  
  # runArch scan is designed to address this case. Our example          #IZ92223
  # kfaix533.ver has the following runArch entries:                     #IZ92223
  #                                                                     #IZ92223
  #    a) runArch = aix533                                              #IZ92223
  #    b) runArch = aix536                                              #IZ92223
  #    c) runArch = open                                                #IZ92223
  #                                                                     #IZ92223
  # As we pass through these entries, we do the following:              #IZ92223
  #                                                                     #IZ92223
  #    a) CANDLEHOME/aix533/kf exists because it was created when the   #IZ92223
  #       kfaix533 component was installed. It is the same architecture #IZ92223
  #       as was requested to be uninstalled, so we remove it.          #IZ92223
  #    b) On a 32-bit system, CANDLEHOME/aix536/kf does not exist, so   #IZ92223
  #       we ignore the entry. On a 64-bit system, this directory is    #IZ92223
  #       populated by the TEPS and by any TEPS application support. If #IZ92223
  #       there is no ver file that corresponds to (owns) this          #IZ92223
  #       architecture, we remove it as part of the request.            #IZ92223
  #    c) There is never any CANDLEHOME/open/kf, so we ignore this      #IZ92223
  #       entry.                                                        #IZ92223
  #                                                                     #IZ92223
  # While the kf product is the only one known to be affect in this     #IZ92223
  # manner, the code below is written to be generic.                    #IZ92223
  #---------------------------------------------------------------------#IZ92223
  rm_binarch=1;                             # assume rm needed later    #IZ92223
  if [ -f ${vfile} ]; then                  # if we have a ver file...  #IZ92223
     grep "^runArch =" ${vfile} | cut -f3 -d" " | while read varch; do  #IZ92223
        varchdir=${CANDLEHOME}/${varch}/${product};                     #IZ92223
        if [ -d ${varchdir} ]; then         # runArch directory exist?  #IZ92223
           rm_varch=1;                      # yes, assume rm is needed  #IZ92223
           if [ "${varch}" = "${binarch}" ]; then                       #IZ92223
              rm_binarch=0;                 # rm'ing now, skip later    #IZ92223
           elif [ -e ${regdir}/${product}${varch}.ver ]; then           #IZ92223
              rm_varch=0;                   # not ours to rm            #IZ92223
           fi;                              # (elif(-e...))             #IZ92223
           if [ ${rm_varch} -eq 1 ]; then                               #IZ92223
              rm -rf ${varchdir} 2> /dev/null;                          #IZ92223
              if [ ${?} -ne 0 ]; then                                   #IZ92223
                 #$NLS-E$ "unable to remove directory for $product"     #IZ92223
                 Err_report warn "`nls_replace 'KCI0211I' $product`";   #IZ92223
              fi;                           # (if($?!=0))               #IZ92223
           fi;                              # (if($rm_varch==1)         #IZ92223
        fi;                                 # (if(-e $varchdir)         #IZ92223
     done;                                  # (while(read varch))       #IZ92223
                                                                        #IZ92223
     rm ${regdir}/${product}${binarch}.ver 2> /dev/null;                #IZ92223
     if [ ${?} -ne 0 ]; then                                            #IZ92223
        #$NLS-E$ "unable to remove version file for $product"           #IZ92223
        Err_report warn "`nls_replace 'KCI0212I' $product`";            #IZ92223
     fi;                                    # (if($?!=0))               #IZ92223
  fi;                                       # (if(-f $vfile))           #IZ92223
                                                                        #IZ92223
  #---------------------------------------------------------------------#IZ92223
  # If we didn't remove the binarch directory as part of ver file       #IZ92223
  # processing above, then make sure it is gone here. This code comes   #IZ92223
  # into play if we were asked to remove something that didn't have a   #IZ92223
  # ver file to start with or if the ver file did not reference the     #IZ92223
  # requested platform.                                                 #IZ92223
  #---------------------------------------------------------------------#IZ92223
  if [ ${rm_binarch} -eq 1 ]; then                                      #IZ92223
     rm -rf ${CANDLEHOME}/${binarch}/${product} 2> /dev/null;           #IZ92223
     if [ ${?} -ne 0 ]; then                                            #IZ92223
        #$NLS-E$ "unable to remove directory for $product"              #IZ92223
        Err_report warn "`nls_replace 'KCI0211I' $product`";            #IZ92223
     fi;                                    # (if($?!=0))               #IZ92223
  fi;                                       # (if($rm_binarch==1))      #IZ92223
                                                                        #IZ92223
  #---------------------------------------------------------------------#IZ92223
  # TEP/TEPS support files reside in the corresponding TEP/TEPS         #IZ92223
  # directory. So, if we just removed a TEP/TEPS component, we also     #IZ92223
  # removed those support files. So, we need to remove the ver files    #IZ92223
  # that go with them.                                                  #IZ92223
  #                                                                     #IZ92223
  # This logic does not apply to TEMS support files. Those reside in    #IZ92223
  # CANDLEHOME/tables/cicatrsq, which means they are not removed when   #IZ92223
  # the TEMS component is removed. Since the TEMS support files exist   #IZ92223
  # after the TEMS is removed, we don't want to remove the ver files    #IZ92223
  # that go with them.                                                  #IZ92223
  #---------------------------------------------------------------------#IZ92223
  case ${product} in                                                    #IZ92223
     cj)                                    # TEP desktop client        #IZ92223
        find ${regdir} -name "??tpd.ver" -exec rm {} 2> /dev/null \; ;  #IZ92223
     ;;                                     # (cj)                      #IZ92223
     cq)                                    # TEPS                      #IZ92223
        find ${regdir} -name "??tps.ver" -exec rm {} 2> /dev/null \; ;  #IZ92223
     ;;                                     # (cq)                      #IZ92223
     cw)                                    # TEP browser client        #IZ92223
        find ${regdir} -name "??tpw.ver" -exec rm {} 2> /dev/null \; ;  #IZ92223
     ;;                                     # (cw)                      #IZ92223
     pa)                                    # TEP perf extensions       #IZ92223
        find ${regdir} -name "??tpa.ver" -exec rm {} 2> /dev/null \; ;  #IZ92223
     ;;                                     # (pa)                      #IZ92223
  esac;                                     # (case($product))          #IZ92223
                                                                        #IZ92223
  # set +x                                  # disable shell tracing     #IZ92223
    
  #---------------------------------------------------------------------#IZ85668
  # If the product is completely removed, then also remove all of its   #IZ85668
  # configuration files:                                                #IZ85668
  #    CANDLEHOME/config/xx.*    (ini, ini.bak, config, etc.)           #IZ85668
  #    CANDLEHOME/config/xx_*.config                                    #IZ85668
  #    CANDLEHOME/config/xx_dd*  (xml, properties, etc.)                #IZ85668
  #    CANDLEHOME/config/kxx_config.ini                                 #IZ85668
  #    CANDLEHOME/config/.ConfigData/kxxenv                             #IZ85668
  #    CANDLEHOME/config/CAP/kxx_default.ini                            #IZ85668
  #    CANDLEHOME/config/<hostname>_xx.cfg     (instance)               #IZ85668
  #    CANDLEHOME/config/<hostname>_xx_*.cfg   (instance)               #IZ85668
  #    CANDLEHOME/config/<hostname>.*_xx_*.cfg (instance)               #IZ85668
  #                                                                     #IZ85668
  # Note: Without removing these, a reinstall of the same product can   #IZ85668
  # pick up old configuration values.                                   #IZ85668
  #---------------------------------------------------------------------#IZ85668
  ls ${CANDLEHOME}/registry/${product}+(ai|hp|li|lp|ls|lx|so)*.ver > /dev/null 2>&1  #IV27849
  rc=$?
  if [ $rc -ne 0 ] && [ -d $CANDLEHOME/config ]; then
     rm -f $CANDLEHOME/config/${product}.* 2> /dev/null 
     rm -f $CANDLEHOME/config/${product}_*.config 2> /dev/null          #IZ85668
     rm -f $CANDLEHOME/config/${product}_dd* 2> /dev/null 
     rm -f $CANDLEHOME/config/k${product}_config.ini 2> /dev/null
     rm -f $CANDLEHOME/config/.ConfigData/k${product}env 2> /dev/null
     rm -f $CANDLEHOME/config/CAP/k${product}_default.xml 2> /dev/null
                                                                        #IZ85668
     MChost=$(hostname | cut -d. -f1);      # mixed case host name      #IZ85668 
     typeset -l LChost="${MChost}";         # lower case host name      #IZ85668 
     typeset -u UChost="${MChost}";         # upper case host name      #IZ85668 
     for host in "${MChost}" "${LChost}" "${UChost}"; do                #IZ85668 
        rm -f ${CANDLEHOME}/config/${host}_${product}.cfg 2> /dev/null;     #IZ85668
        rm -f ${CANDLEHOME}/config/${host}_${product}_*.cfg 2> /dev/null;   #IZ85668
        rm -f ${CANDLEHOME}/config/${host}.*_${product}_*.cfg 2> /dev/null; #IZ85668
     done;                                                              #IZ85668 
  fi 
  
  # remove swg tagging files
  if [[ -d "$CANDLEHOME/properties/version/" ]];
  then
    if [[ $rc -ne 0 ]];
    then
      # product removed, just remove all related swg tagging files	
      rm -f $CANDLEHOME/properties/version/k${product}.* > /dev/null 2>&1
    else  
      # get base architecture code ex. li from li6263 binarch
      baseArch=${binarch%%[0-9]*}
      ls $CANDLEHOME/registry/$product$baseArch*.ver 2> /dev/null | egrep "$product$baseArch[0-0]+\.ver" > /dev/null 2>&1
      if [[ $? -ne 0 ]];
      then
        for file in `ls $CANDLEHOME/properties/version/k${product}.${baseArch}* 2>/dev/null | egrep "k$product\.$baseArch[0-9]+\..*"`;
        do
          rm -rf $file > /dev/null 2>&1	
        done
      else
        rm -f $CANDLEHOME/properties/version/k${product}.${binarch}* > /dev/null 2>&1
      fi
    fi  
  fi
    
  #defect 55109, append "|" with $binarch to avoid hp11 & hp116 return together.
  platformDesc=$(grep "^$binarch|" $regDir/archdsc.tbl|cut -f2 -d"|")

  #$NLS-E$ "$name for platform $platformDesc removed from $CANDLEHOME.
  nls_replace 'KCI0213I' "$name" "$platformDesc" "$CANDLEHOME"

  # Log the deletion also
  logfile=$CANDLEHOME/logs/candle_installation.log
  print "uninstall.sh  : $(date)" >> $logfile
  print "uninstall.sh  : $name for platform $platformDesc removed from $CANDLEHOME" >> $logfile
  
}

# Delete old token files, and warn if there is another install running in this <candlehome>.

procTokFiles()
{
  typeset -i maxAge=5  # Remove "remote" token files older than 5 days.
  typeset -i age=0 dot=0 doy=0
  typeset PID name
  
  ls ${baseTok}* | grep -v $thisTok > /dev/null 2>&1  # Other installs?
  [ $? -eq 0 ] || return
  
  doy=$(date +"%j")  # Day of the year (001 - 366)
  
  # Remove token files:
  # 1) created on this machine that are no longer connected to any process.
  # 2) created on other machines that are older than maxAge.
  
  for name in $(ls ${baseTok}* | grep -v $thisTok)
  do
    cat $name | grep "$thisMach" > /dev/null 2>&1 
    if [ $? -eq 0 ]  # The token file was created on this machine.
    then
      PID=${name##*_}
      ps -p $PID | grep $PID > /dev/null 2>&1
      [ $? -ne 0 ] && rm -f $name
    else  
      cat $name | grep "|" > /dev/null 2>&1  # Ensure the day of year field exists.
      [ $? -eq 0 ] || continue
      
      dot=$(cat $name | cut -d"|" -f2 | cut -d"|" -f1)  # Day the token file was created (001 - 366).

      if [ doy -ge dot ]
      then
        age=doy-dot
      else  # This could happen towards the end of a year, so "normalize" the day of the year.
        doy=366+doy
        age=doy-dot
      fi

      [ age -gt maxAge ] && rm -f $name
    fi
  done
  
  # Warn the user about possible simultaneous installs.
  
  ls ${baseTok}* | grep -v $thisTok > /dev/null 2>&1  # Check for other installs.
  if [ $? -eq 0 ] 
  then
  #$NLS-E$ "\nAn install may currently be running in \"$CANDLEHOME\" from"
  #$NLS-E$ "the following machine(s): \n"
	print " "
	nls_replace 'KCI0215I' \"$CANDLEHOME\"
	print " "

    for name in $(ls ${baseTok}* | grep -v $thisTok)
    do
      print "  \c"; cat $name
    done
  
    #$NLS-E$ "\nContinue with this uninstallation [ y or n; \"n\" is default ]?  \c"
	print "\n`nls_replace 'KCI0217I'` [ 1-`nls_replace 'KCI0197I'`, 2-`nls_replace 'KCI0198I'`; \"2\" `nls_replace 'KCI0199I'` ]? \c"
    read
    case $REPLY in
      y*|Y*|1) ;;
    
          *) clean_up
             ;;
    esac
  fi
}
# Get the running process list for this host

cinfoRunning()
{
  $CANDLEHOME/bin/cinfo -r |
     $AWK '/\.\.\.running/ { A=NF-1; print $1"|"$2"|"$3"|"$A"|"$4 }'
}
# Check for running processes before uninstall

checkProcesses()
{
  typeset rFile=$CANDLEHOME/config/.ConfigData/RunInfo
  typeset PID desc fieldSep fld23 msg pc proc 
  typeset thisMachShort 

  # No RunInfo means nothing's running
  [ ! -f $rFile ] && return
  
  # Determine whether it's BigK or dinkySQL.
  
  fieldSep=$(grep "FIELDSEP" $rFile | cut -d "=" -f2 2> /dev/null)
  
  # See if there are any agents/CMS's running from this <candlehome> on this machine.
  
  fld123=$(cinfoRunning)  # fld123 -> HOST|ms|35594|TEMS|itmuser
  fld23=$(print "$fld123" | cut -d "$fieldSep" -f2-5)  # fld23 -> ms|35594|TEMS|itmuser

  # Check to see if the processes are actually running.
  [ -n "$fld23" ] || return

  #$NLS-E$ "\\nThe following processes must be stopped before attempting uninstall: \\n"
  msg="\\n`nls_replace 'KCI0218I'` \\n"

  print "$msg"
  for proc in $fld23
  do
    pc=$(print "$proc" | cut -d "$fieldSep" -f1)

    desc=$(get_desc $pc) 

    PID=$(print "$proc" | cut -d "$fieldSep" -f2)

    #$NLS-E$ Product
    print "  `nls_replace 'KCI1170I'` = $desc  PID = $PID"
  done
  print
  exit 3
}

hp_forcewait()
{

#  OS agent is not cleaning up its children so we have to wait on HP for 
#  them to die

    CurRegPath=$CANDLEHOME/registry
    ArchTbl=$CurRegPath/archdsc.tbl
    DynShl=$CANDLEHOME/bin/dynarch.shl


    . $DynShl $ArchTbl 2>/dev/null
    os=$(print $cArch|cut -c1-2)
 
    if [ "$os" = hp ]
    then
	   #$NLS-E$ Waiting for agent to shutdown	
       print "`nls_replace 'KCI1171I'`"
       sleep 70
       if [ -d "$CANDLEHOME/../" ]
       then
           cd $CANDLEHOME/../
       fi
    fi

    return

}
remove_autostart()
{

    CurRegPath=$CANDLEHOME/registry
    ArchTbl=$CurRegPath/archdsc.tbl
    DynShl=$CANDLEHOME/bin/dynarch.shl


    . $DynShl $ArchTbl 2>/dev/null

    os=$(print $cArch|cut -c1-2)
  typeset -i filenum=$(cat $CANDLEHOME/registry/AutoStart 2> /dev/null);  #IZ58055
                                                                          #IZ58055
  #-----------------------------------------------------------------------#IZ58055
  # Define the commands to undo autostart for our platform.               #IZ58055
  #-----------------------------------------------------------------------#IZ58055
  case "$os" in                                                           #IZ58055
    li|lx|lp|ls)                                                          #IZ58055
      cmd="chkconfig --del ITMAgents$filenum > /dev/null 2>&1"            #IZ58055
      cmd="$cmd;rm -f /etc/init.d/ITMAgents$filenum"                      #IZ58055
      cmd="$cmd;rm -f /etc/rc.d/rc[0-6].d/K10ITMAgents$filenum"           #IZ58055
      cmd="$cmd;rm -f /etc/rc.d/rc[0-6].d/S99ITMAgents$filenum"           #IZ58055
      cmd="$cmd;rm -f /etc/init.d/rc[0-6].d/K10ITMAgents$filenum"         #IZ58055
      cmd="$cmd;rm -f /etc/init.d/rc[0-6].d/S99ITMAgents$filenum"         #IZ58055
      cmd="$cmd;rm -f /var/lock/subsys/ITMAgents$filenum 2> /dev/null"
      ;;                                                                  #IZ58055
    hp)                                                                   #IZ58055
      cmd="rm -f /sbin/rc0.d/K100ITMAgents$filenum"                       #IZ58055
      cmd="$cmd;rm -f /sbin/rc1.d/K100ITMAgents$filenum"                  #IZ58055
      cmd="$cmd;rm -f /sbin/rc2.d/S500ITMAgents$filenum"                  #IZ58055
      cmd="$cmd;rm -f /sbin/init.d/ITMAgents$filenum"                     #IZ58055
      ;;                                                                  #IZ58055
    so)                                                                   #IZ58055
      cmd="rm -f /etc/rc0.d/K10ITMAgents$filenum"                         #IZ58055
      cmd="$cmd;rm -f /etc/rc1.d/K10ITMAgents$filenum"                    #IZ58055
      cmd="$cmd;rm -f /etc/rc2.d/S99ITMAgents$filenum"                    #IZ58055
      cmd="$cmd;rm -f /etc/init.d/ITMAgents$filenum"                      #IZ58055
      ;;                                                                  #IZ58055
    ai)                                                                   #IZ58055
      rcFile="/etc/rc.itm${filenum}";                                     #IV23322
      cmd="rmitab rcitm$filenum"                                          #IZ58055
      cmd="$cmd;rm -f ${rcFile}";                                         #IV23322
      ;;                                                                  #IZ58055
    *)                                           # should never happen... #IZ58055
      cmd="print -u2 'internal error: os=$os is not valid'"               #IZ58055
      ;;                                                                  #IZ58055
  esac                                           # (case(os))             #IZ58055
                                                                          #IZ58055
  #-----------------------------------------------------------------------#IZ58055
  # If we're already root, run the command and return. Otherwise, we try  #IZ58055
  # to run the command interactively and let the user control how we      #IZ58055
  # proceed.                                                              #IZ58055
  #-----------------------------------------------------------------------#IZ58055
  if [ "$(whoami)" = "root" ]; then                                       #IZ58055
    eval "$cmd"                                                           #IZ58055
  else
     #$NLS-E$ print "\nroot user access is required to remove automatic restart files"
     print "\n`nls_replace 'KCI0219I'`";                                  #IV23322
     # If we're not root and we're in force mode, bail out
     if [ "$force" = "yes" ]; then                                        #IV23322
        loop="no";                                                        #IV23322
     else                                                                 #IV23322
        loop="yes";                                                       #IV23322
     fi;                                                                  #IV23322

    while [ $loop = "yes" ]
    do
       su root -c "$cmd"
       rc=$?
       if [ $rc -ne 0 ]  # su failed
       then
          #$NLS-E$ print "\nIgnore [ i ] or Try again [ y, n; \"n\" is default ]?  \c"
          print "\n`nls_replace 'KCI0220I'` [2-`nls_replace 'KCI0197I'`, 3-`nls_replace 'KCI0198I'`; \"3\" `nls_replace 'KCI0199I'`]? \c"; #IV23322
          read 
          case $REPLY in
            y*|Y*|2) 
              continue
            ;;
            i*|I*|1) 
              loop="no";                                                  #IV23322
            ;;
            *) 
              #$NLS-E$ "Failing, Commands to be executed were:"
			  nls_replace 'KCI0221E'
              print "$cmd"
              loop="no";                                                  #IV23322
            ;;
         esac
       else                                                               #IV23322
          loop="no";                                                      #IV23322
       fi;                                                                #IV23322
    done
  fi
                                                                          #IV23322
  if [[ "${os}" = "ai" ]]; then                                           #IV23322
    #$NLS-E$ "You must manually disable automatic stop at system          #IV23322
    #$NLS-E$ shutdown. Modify /etc/rc.shutdown so it no longer            #IV23322 
    #$NLS-E$ invokes the command '$1 stop'.                               #IV23322
    print "";                                                             #IV23322
    nls_replace 'KCI0120I' "${rcFile}";                                   #IV23322
  fi;                                                                     #IV23322
}

uninstall_process()
{
   TEMP_pc=$1
   TEMP_platform=$2

   # If exists, then we can run this script if JAVA is also installed.
   # This script was added as a special case for a specific agent.
   # Generally, it is not installed. And it is not used by the SMA style agents.
   if [ -f $CANDLEHOME/bin/uninstall-process.sh ]
   then

      # and is executable
      if [ -x $CANDLEHOME/bin/uninstall-process.sh ]
      then

         print ""
         #$NLS-E$ "Processing pre-uninstall operations for component:"
         print "`nls_replace 'KCI0248I'` $TEMP_pc"
    
         #TEST print "$CANDLEHOME/bin/uninstall-process.sh \"$JAVA_HOME\" \"$CANDLEHOME\" \"$TEMP_pc\" \"preuninstall\" \"$TEMP_platform\" \"uninstall\""
         #$CANDLEHOME/bin/uninstall-process.sh "$JAVA_HOME" "$CANDLEHOME" "$TEMP_pc" "preuninstall" "$TEMP_platform" "uninstall" 2>$CANDLEHOME/logs/uninstall-process.err

         $CANDLEHOME/bin/uninstall-process.sh "$JAVA_HOME" "$CANDLEHOME" "$TEMP_pc" "preuninstall" "$TEMP_platform" "uninstall" "$force" 2>$CANDLEHOME/logs/uninstall-process.err			
         typeset result=$?
         if [ $result -eq 0 ]
         then
            #$NLS-E$ "Processing pre-uninstall complete."
            print "`nls_replace 'KCI0249I'`"
            print ""
         fi
         return $result

      else

         #TEST print "OK, the permissions are off, lets try and set execute"
         #TEST print "chmod a+x $CANDLEHOME/bin/uninstall-process.sh"
         chmod a+x $CANDLEHOME/bin/uninstall-process.sh 2>/dev/null

         # Now check again
         if [ -x $CANDLEHOME/bin/uninstall-process.sh ]
         then

            # We fixed the problem, good

            print ""
            #$NLS-E$ "Processing pre-uninstall operations for component:"
            print "`nls_replace 'KCI0248I'` $TEMP_pc"
    
            #TEST print "$CANDLEHOME/bin/uninstall-process.sh \"$JAVA_HOME\" \"$CANDLEHOME\" \"$TEMP_pc\" \"preuninstall\" \"$TEMP_platform\" \"uninstall\""
            #$CANDLEHOME/bin/uninstall-process.sh "$JAVA_HOME" "$CANDLEHOME" "$TEMP_pc" "preuninstall" "$TEMP_platform" "uninstall" 2>$CANDLEHOME/logs/uninstall-process.err

            $CANDLEHOME/bin/uninstall-process.sh "$JAVA_HOME" "$CANDLEHOME" "$TEMP_pc" "preuninstall" "$TEMP_platform" "uninstall" "$force" 2>$CANDLEHOME/logs/uninstall-process.err			
            typeset result=$?
            if [ $result -eq 0 ]
            then
               #$NLS-E$ "Processing pre-uninstall complete."
               print "`nls_replace 'KCI0249I'`"
               print ""
            fi
            return $result

         else

            # OK, we tried once, now we tell the user for more advanced steps
            unProcessFile = "$CANDLEHOME/bin/uninstall-process.sh"
            print ""
            #$NLS-E$ "The file [$CANDLEHOME/bin/uninstall-process.sh] exists, but is not executable by this user."
            print "`nls_replace 'KCI1260I' "$unProcessFile"`"
            #$NLS-E$ "Please change the permission of this file to be executable by this user."
            print "`nls_replace 'KCI1261I'`"
            print ""

            # This is a bold step to exit the entire unintsaller, but warranted since they need to try and fix this,
            # and it's easy enough to re-enter the tool. Plus, the user might not see the message

            clean_up 1

         fi
      fi

   else
      # The file does not exist, which will often happen - we silently skip this
      print ""
   fi

}

# remove /opt/IBM/ITM/tmaitm6/links if ITM is installed to a non-default location
# if there is only the directory links under /opt/IBM/ITM/tmaitm6, then we remove the whole /opt/IBM/ITM
# if there are other files/directories under /opt/IBM/ITM or /opt/IBM/ITM/tmaitm6, then we just remove /opt/IBM/ITM/tmaitm6/links
remove_links()
{
  typeset defaultCH="/opt/IBM/ITM"
  typeset link_target=""
  typeset tmp=""

  if [[ "$CANDLEHOME" != "$defaultCH" && -d $defaultCH ]]
  then
    # search for symblink files under /opt/IBM/ITM
    for linkF in $(find ${defaultCH}/tmaitm6/links -type l 2>/dev/null)
    do
      link_target=`ls -l $linkF`
      link_target=${link_target##*-\> }

      # link_target=/opt/IBM/ITMxyz/tmaitm6/... -> tmp=xyz/tmaitm6/...
      # link_target=/opt/IBM/xyz/tmaitm6/... -> tmp=/opt/IBM/xyz/tmaitm6/...
      tmp=${link_target##$CANDLEHOME}
      if [ "$tmp" = "$link_target" ]
      then
        # this link doesn't point to this ITM
        continue 
      fi

      # tmp=xyz/tmaitm6/... -> tmp=xyz
      tmp=${tmp%%/*}
      if [ "$tmp" != ""  ]
      then
        # this link doesn't point to this ITM
        continue
      fi

      # this symblink file links to this ITM, it should be removed
      rm -f $linkF 1>/dev/null 2>&1
    done
    
    # remove /opt/IBM/ITM if it's empty (if it doesn't contains any kind of file)
    typeset -i file_num=`find $defaultCH ! -type d 2>&1 | wc -l`
    if [ $file_num = 0 ]
    then
      rm -rf $defaultCH 1>/dev/null 2>&1
    fi
  fi
}

uninstall_callpoint()
{
	productCode=$1
	
	export saveLang=$LANG
	
	CurRegPath=$CANDLEHOME/registry
	ArchTbl=$CurRegPath/archdsc.tbl
	DynShl=$CANDLEHOME/bin/dynarch.shl

	. $DynShl $ArchTbl 2>/dev/null
	
	baseArch=${cArch%%[0-9]*}
	pcInstalled=$(ls $CANDLEHOME/registry/$productCode$baseArch*.ver 2>/dev/null)
	[ -z "$pcInstalled" ] && return
	
	useJreArch=$cArch
	
	# in case we are running platform lx we need to modify java arch prerequisite
	if [ "$baseArch" = "lx" ]; then
	  export LANG=C
	  cArchJava=$(grep "^preReqALL = jr" $pcInstalled 2>/dev/null | cut -d"=" -f2 | cut -d "|" -f1 | cut -c 4-)
	  export LANG=$saveLang
	  #use cArchJava only if component has jre prereq	
	  [ -n "$cArchJava" ] && useJreArch=$cArchJava
	fi

	setJavaEnv $useJreArch
	
	setBinLibArchs $cArch $productCode
	# echo "Product arch = $binArch"
	baseArch=${cArch%%[0-9]*}
	export LANG=C
	pcVRMF=$(grep "^VRMF =" $CANDLEHOME/registry/$productCode$baseArch*.ver 2>/dev/null | cut -d"=" -f2)
	export LANG=$saveLang
	plugin_type=install
	plugin_action=uninstall
	
	# if there is no VRMF in registry/<pc><arch>.ver file - create VRMF based on "vel" and "ver"
	# VRMF = 0+ver+rel+0
	if [ -z "$pcVRMF" ];
	then
	 export LANG=C
	 pcVER=$(grep "^ver =" $CANDLEHOME/registry/$productCode$baseArch*.ver 2>/dev/null | cut -d"=" -f2)
	 pcREL=$(grep "^rel =" $CANDLEHOME/registry/$productCode$baseArch*.ver 2>/dev/null | cut -d"=" -f2)
	 export LANG=$saveLang
		# Check if ver and rel can be read from ver file
		if [ -z "$pcVER" ] || [ -z "$pcREL" ]
		then
		  # Unable to determine product version
		  return
		fi
	 pcVRMF="0""$pcVER""$pcREL""0"
	fi
	
	$JREEXE -classpath $CLASSPATH $SAFE_MODE -DGlobalDbgLevel="DEBUG_MAX" -DInstallRASConfig="$CANDLEHOME/config/ITMConfigRAS.properties" ITMinstall.InstallComponentPluginCall $plugin_type $plugin_action $CANDLEHOME $cArch $productCode $binArch $pcVRMF
	rc=$?

	# echo "Return code is $rc"
}

clean_up()
{
  typeset -i rc=0

  [[ "$1" = +([0-9]) ]] && rc=$1

  rm -f "$thisTok" > /dev/null 2>&1
  
  # Our plugin work dir
  rm -rf /tmp/plugin_tmp > /dev/null 2>&1
  #on HP-IA, the temporary directory is /var/tmp
  if [ -d /var/tmp/plugin_tmp ]
  then
      rm -rf /var/tmp/plugin_tmp > /dev/null 2>&1
  fi

  exit $rc
}

###################################################################
#                         Main Routine                            #
###################################################################

if [[ -f /usr/xpg4/bin/grep ]]; then                                    #IV27849
   grepCmd="/usr/xpg4/bin/grep";            # Solaris 10 & earlier      #IV27849
else                                                                    #IV27849
   grepCmd="grep";                          # Solaris 11+ & Linux/Unix  #IV27849
fi;                                                                     #IV27849
                                                                        #IV27849
bindir=`pwd`/$(dirname $0)
[ "$(print $0 | grep '\./')" = "" ] && [ "$(print $0 | grep '^/.*')" = "" ] && [ `pwd` = '/' ] && bindir=`pwd`$(dirname $0)
[ "$(print $0 | grep '^/.*')" != "" ] && bindir=$(dirname $0)

export TEXTDOMAINDIR=$bindir/../config/nls/msg
export PATH=/bin:/usr/bin:/usr/sbin:/usr/ucb:$bindir:$PATH              #IZ97998
Mode=""
Menu="yes"
force="no"
ignore="no"
[ -z "$(whoami 2>/dev/null)" ] && alias whoami='id | cut -d"(" -f2 | cut -d")" -f1'

typeset -i ver=0 vr=0

typeset baseTok thisMach thisOS thisRec thisTok thisUser  # Variables used for the token file.

while getopts ":h:fi" OPTS
do case $OPTS in
   h) CANDLEHOME=$OPTARG
      ;;
   f) force="yes"
      ;;
   i) ignore="yes"
      ;;
   ?) Use_report
      ;;
   esac
done

shift $((OPTIND-1))

# Allow only product/platform combo or nothing
if [ $# -ge 2 ]
then
   pc="$1"; shift; platform="$1"
   Menu="no"
elif [ $# -eq 1 ]
then
   Use_report
else
   Menu="yes"
fi

[ -n "$CANDLEHOME" ] || CANDLEHOME=$(cd $(dirname $0) ; dirname $(pwd) )
#$NLS-E$ "specified ITM home is not a valid directory: $CANDLEHOME"
[ -d "$CANDLEHOME" ] || Err_report fail "`nls_replace 'KCI0222E'` $CANDLEHOME"
#$NLS-E$ "You have to be a root user or have the write access to directory $CANDLEHOME if you want to uninstall ITM."
[ -w "$CANDLEHOME" ] || Err_report fail "`nls_replace 'KCI1205E' $CANDLEHOME`"
export CANDLEHOME
originalDir=$(pwd)

if [ $(print "$CANDLEHOME" | wc -w) -ne 1 ]
then
  #$NLS-E$ "Blanks are not permitted in CANDLEHOME"
  Err_report fail "`nls_replace 'KCI0223E'`"
fi

export CANDLETMP=$CANDLEHOME/tmp

CANDLEDATA="";                              # assume no CANDLEDATA      #IV67523
CH2CDlink="${CANDLEHOME}/registry/CANDLEDATA";   # get the link marker  #IV67523
if [[ -L "${CH2CDlink}" ]]; then                 # is link set?         #IV67523
  CH2CDlink=$(ls -l "${CH2CDlink}" 2> /dev/null);                       #IV67523
  if [[ "${CH2CDlink}" = *" -> "* ]]; then  # is it valid?              #IV67523
     CANDLEDATA="${CH2CDlink##* -> }";      # yes, use it               #IV67523
  fi;                                       # (if(CH2CDlink=* -> *))    #IV67523
fi;                                         # (if(-L...))               #IV67523
                                                                        #IV67523
# LL - start token file stuff.

baseTok=$CANDLETMP/candle_install_

thisOS=$(uname -s)  # thisOS -> AIX
thisUser=$(whoami)

# Create the token file.

thisMach=$(hostname)  # thisMach -> vulcan
thisTok=$baseTok${thisMach}_$$
touch $thisTok > /dev/null 2>&1
#$NLS-E$ "could not create token file"
[ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI0224E'`"

trap "clean_up" 1 2 3 15

# Build the description record, and add it to the token file.  This record will look like: 
# "AIX vulcan billy 17440 Friday Jun 08 14:21 2001 |159|".  The number between pipes (|)
# is the day of the year the file was created.

thisRec="$thisOS $thisMach $thisUser $$ $(date +"%A %B %d %H:%M %Y |%j|")"
print "$thisRec" > $thisTok

# Delete old token files, and warn if there is another install running in this <candlehome>.

procTokFiles 

# LL - end token file stuff.

if [ "$ignore" = "no" ]
then
  # Make sure nothing's running in this CandleHome
  checkProcesses
fi

# Get installer version and release number
ciVFile=cienv.ver
typeset curRel="" curVer="" regFile=""

regFile=$CANDLEHOME/registry/$ciVFile
#$NLS-E$ "could not read file: $regFile"
[ -r $regFile ] || Err_report fail "`nls_replace 'KCI0225E' $regFile`"

curVrmf=$(grep "^VRMF =" $regFile | cut -d"=" -f2)
if [ -z "$curVrmf" ]
then
    curVer=$(grep "^ver =" $regFile | cut -d"=" -f2)
    #$NLS-E$ "could not determine the version number from $regFile"
	[ -n "$curVer" ] || Err_report fail "`nls_replace 'KCI0226E' $regFile`"
    curRel=$(grep "^rel =" $regFile | cut -d"=" -f2)
    #$NLS-E$ "could not determine the release number from $regFile"
	[ -n "$curRel" ] || Err_report fail "`nls_replace 'KCI0227E' $regFile`"
else
    curVer1=$(print $curVrmf| cut -c 1,2)
    curVer2=$(print $curVrmf| cut -c 3,4)
    curVer3=$(print $curVrmf| cut -c 5,6)
    curVer4=$(print $curVrmf| cut -c 7,8)
fi

# Show banner
print " " 
print "*********** $(date) ******************"
if [[ "$(uname -s)" != "NONSTOP_KERNEL" ]]
then
  print "User      : $(whoami)\t Group: $(groups)"
  if [ -z "$curVrmf" ]
  then
    #$NLS-E$ "Host name : $(hostname)\t Installer Lvl:$curVer /$curRel" 
	print "`nls_replace 'KCI0228I'` : $(hostname)\t `nls_replace 'KCI0229I'` Lvl:$curVer /$curRel" 
  else
    #$NLS-E$ "Host name : $(hostname)\t Installer Lvl:$curVer1.$curVer2.$curVer3.$curVer4"
	print "`nls_replace 'KCI0228I'` : $(hostname)\t `nls_replace 'KCI0229I'` Lvl:$curVer1.$curVer2.$curVer3.$curVer4"
  fi
else
  #$NLS-E$ "User      : $(whoami)\t "
  print "`nls_replace 'KCI0230I'`      : $(whoami)\t "
  if [ -z "$curVrmf" ]
  then
    #$NLS-E$ "Installer Lvl:$curVer /$curRel" 
	print "`nls_replace 'KCI0229I'` Lvl:$curVer /$curRel" 
  else
    #$NLS-E$ " Installer Lvl:$curVer1.$curVer2.$curVer3.$curVer4"
	print "`nls_replace 'KCI0229I'`  Lvl:$curVer1.$curVer2.$curVer3.$curVer4"
  fi
fi		# end uname -s
print "ITM home  : $CANDLEHOME" 
if [[ -n "${CANDLEDATA}" ]]; then                                       #IV67523
   print "ITM data  : ${CANDLEDATA}";                                   #IV67523
fi;                                                                     #IV67523
print "***********************************************************" 

infoTmp=$CANDLEHOME/tmp/info.tmp_$$
infoTmp2=$CANDLEHOME/tmp/info.tmp2_$$
regDir=$CANDLEHOME/registry

# This returns JAVA_HOME to us on the command line - set to variable and use
if [ -f $CANDLEHOME/bin/CandleGetJavaHome ]
then
  export JAVA_HOME=`$CANDLEHOME/bin/CandleGetJavaHome`
else
  export JAVA_HOME=
fi

# User entered a product code / platform combo in the args
if [ "$Menu" = "no" ]
then

  cd $regDir
  typeset sharedPrereq=""
  for comp in $(ls *.ver | grep -v "^ci")
  do
    comp=${comp%.ver}
    sharedPrereq=$(grep " = $pc$platform" ${comp}.ver)
    [[ -n "$sharedPrereq" ]] && break
  done
  if [ -n "$sharedPrereq" ]
  then
    Err_report fail "`nls_replace 'KCI1231E' \"$pc$platform\" \"$comp\"`"
  fi

  typeset result=0

  cd $regDir
  tmpVerFile=$pc$platform.ver
  if [ -f $tmpVerFile ]; then 
    isProdInstalled="yes"
  fi
 
  if [ "$isProdInstalled" = "yes" ]; then 
    if [ -n "$JAVA_HOME" ]; then 
      # Call the uninstall-process handler
      uninstall_process $pc $platform
      result=$?
      # Call the uninstall callpoint funtion
      if [ $result -eq 0 ]; then
        uninstall_callpoint $pc
        result=$?
      fi
    fi
  elif [[ -n "$JAVA_HOME" && "$pc" = "REMOVE" && "$platform" = "EVERYTHING" ]]
  then
    cd $regDir
    codes="$(ls *.ver 2>/dev/null | grep -v "^ci")"

    for verfile in $(ls ??+(ai|hp|li|lp|ls|lx|so)*.ver 2> /dev/null)    #IV27849
    do
      pc=$(print $verfile | cut -c 1-2)

      if [ "$pc" != "gs" ] && [ "$pc" != "t1" ] && [ "$pc" != "jr" ] && [ "$pc" != "ui" ]
      then
        platform="$(print $verfile | cut -c 3-)"
        platform=${platform%.ver}

        for prereq in $codes; do
          prereq=${prereq%.ver}
          sharedPrereq=$(grep " = $pc$platform" ${prereq}.ver)
          [[ -n "$sharedPrereq" ]] && break
        done

        if [ -z "$sharedPrereq" ]
        then
          # Call the uninstall-process handler
          uninstall_process $pc $platform
          result=$?
          # Call the uninstall callpoint funtion
          if [ $result -eq 0 ]; then
            uninstall_callpoint $pc
            result=$?
          fi
        fi
      fi
    done

    pc="REMOVE"
    platform="EVERYTHING"
  fi
    
  if [ $result -eq 1 ]
  then
    #$NLS-E$ "Cancel Uninstall..."
    print "`nls_replace 'KCI1290I'`"
  elif [ $result -eq 0 ]
  then
	# The uninstall_product function will check if the package is installed and display a message if not found.
	uninstall_product $pc $platform
	# if installed product was successfully uninstalled with all prerequisites and was executed by non-root, show additional message to the user
	if [ "$isProdInstalled" = "yes" ]; then 
		if [ "$(whoami)" != "root" ]; then
			#$NLS-E$ You have to be a root user to update auto restart script
			Err_report warn "`nls_replace 'KCI1167I'`"
		fi
	fi
  elif [ $result -eq 2 ]
  then
    # uninstall.sh failed.
	clean_up 2
  fi

  # cm12160 - for silent install, need to remove top level directory
  # if no more products are installed and the force flag (-f) is specified
  remaining=$(find $regDir -name '*.ver' ! -exec $grepCmd -q 'preReqALL = none' {} \; -print 2>/dev/null)

  if [[ -z "$remaining" && "$force" = "yes" ]]; then
    #$NLS-E$ "Removing ITM home: $CANDLEHOME\n"
    print "`nls_replace 'KCI0231I'`: $CANDLEHOME\n"
  
    # remove the agent autostart files
    if [ -f $CANDLEHOME/registry/AutoStart ]
    then
      remove_autostart
    fi
    
    cd $originalDir
 
    hp_forcewait
 
    rm -rf $CANDLEHOME

    remove_links
  fi

# User entered nothing on command-line so we show the list of installed prods
else 
  rm -f $infoTmp2
  Response=""
  delPC=""

  while true
  do
    cd $regDir
    # Get a list of installed product version files. Show only version files that have prereqs
    for pc in $(ls ??+(ai|hp|li|lp|ls|lx|so)*.ver 2> /dev/null)         #IV27849
    do
      pc=$(print $pc|cut -c 1-2)
      # GSKit should not be printed on the list of components available for uninstall and
      # should not be uninstalled until everything is removed, 
      # it must get removed along with CANDLEHOME directory
      # T1 should not be listed as an option on the menu since it is not a true component, 
      # but now integrated into the TEMS package.
      # JR should not be listed as an option on the uninstall menu since it is not a true ITM component.
       if [ "$pc" != "gs" ] && [ "$pc" != "t1" ] && [ "$pc" != "jr" ] && [ "$pc" != "ui" ]
      then
        print $pc >> $infoTmp2
      fi
    done
    
    if [ -z "$Response" ] && [ -f $infoTmp2 ]
    then
      #$NLS-E$ "\n...Products available to uninstall\n"
	  print "\n...`nls_replace 'KCI0232I'`\n"
      #$NLS-E$ "Num\tProduct [ Code  Platform Version:Release Description ]"
	  print "`nls_replace 'KCI0233I'`"
    fi

    # No products to list ... ask about removing CANDLEHOME
    if [ ! -f $infoTmp2 ]
    then
      print
      #$NLS-E$ "No products found to uninstall. Preparing to remove ITM home ..."
      nls_replace 'KCI0234I'
      if [ "$force" = "no" ]
      then
        #$NLS-E$ "Confirm: CANDLEHOME=$CANDLEHOME  ... OK to delete? [y or n; \"n\" is default]: "
		print -n "`nls_replace 'KCI0235I'`: CANDLEHOME=$CANDLEHOME  ... `nls_replace 'KCI0236I'` [ 1-`nls_replace 'KCI0197I'`, 2-`nls_replace 'KCI0198I'`; \"2\" `nls_replace 'KCI0199I'` ]: "
          read answer
        else
          answer="y"
      fi
      case "$answer" in
        #$NLS-E$ y|Y|yes|YES) print "Removing ITM home: $CANDLEHOME\n"
		y|Y|yes|YES|1) print "`nls_replace 'KCI0237I'`: $CANDLEHOME\n"
	cd $originalDir
        cmdHome=$(cd $(dirname $0) ; dirname $(pwd) )
	if [[ $(pwd) = "$CANDLEHOME/bin" || "$CANDLEHOME" = "$cmdHome" ]]
	then
	  #$NLS-E$ "$CANDLEHOME/bin may have to be removed manually after this script completes"
	  Err_report warn "$CANDLEHOME/bin `nls_replace 'KCI0238I'`"
	fi
        #$NLS-E$ "Purging $CANDLEHOME"
		Err_report warn "`nls_replace 'KCI0239I'` $CANDLEHOME"
        if [ -f $CANDLEHOME/registry/AutoStart ]
        then
           remove_autostart
        fi
 
        hp_forcewait
 
        rm -rf $CANDLEHOME
        if [[ -n "${CANDLEDATA}" ]]; then                               #214587
          rm -rf "${CANDLEDATA}";                                       #214587
        fi;                                                             #214587

        remove_links
        ;;
      esac
      clean_up 1
    fi

    sort -u -o $infoTmp $infoTmp2
    typeset -i index=0
    while read pc
    do
      name=$(get_desc $pc)
      for vFile in $(ls ${pc}+(ai|hp|li|lp|ls|lx|so)*.ver 2> /dev/null);#IV27849
      do
        comp=${vFile%.ver}
        ${grepCmd} -q "preReqALL = ${comp}" ??+(ai|hp|li|lp|ls|lx|so)*.ver; #IV27849
        if (( ${?} == 0 )); then                                        #IV27849
          # this component is a prereq of other component,
          # it shouldn't be selected to uninstall
          # just skip it
          continue
        fi

        index=$index+1
        if [ "$Response" = "" ]
        then
          platform=${vFile#$pc}
          platform=${platform%.ver}
          vrmf=$(grep "^VRMF =" $vFile|cut -f3 -d" ")
          if [ -z "$vrmf" ]
          then
            version=$(grep "^ver =" $vFile|cut -f3 -d" ")
            release=$(grep "^rel =" $vFile|cut -f3 -d" ")
            print "$index\t$pc  $platform\t v$version:r$release  $name"
          else
            vrmf1=$(print $vrmf| cut -c 1,2)
            vrmf2=$(print $vrmf| cut -c 3,4)
            vrmf3=$(print $vrmf| cut -c 5,6)
            vrmf4=$(print $vrmf| cut -c 7,8)
            print "$index\t$pc  $platform\t v$vrmf1.$vrmf2.$vrmf3.$vrmf4 $name"
          fi
        elif [ "$Response" = $index ]
        then
          platform=${vFile#$pc}
          platform=${platform%.ver}
          vrmf=$(grep "^VRMF =" $vFile|cut -f3 -d" ")
          if [ -z "$vrmf" ]
          then
            version=$(grep "^ver =" $vFile|cut -f3 -d" ")
            release=$(grep "^rel =" $vFile|cut -f3 -d" ")
            delVersion=$version
            delRelease=$release
          else
            vrmf1=$(print $vrmf| cut -c 1,2)
            vrmf2=$(print $vrmf| cut -c 3,4)
            vrmf3=$(print $vrmf| cut -c 5,6)
            vrmf4=$(print $vrmf| cut -c 7,8)
            delVersion=$vrmf1.$vrmf2.$vrmf3.$vrmf4
            delRelease=""
          fi
          delPC=$pc
          delPlatform=$platform
          delName=$name
          break
        fi
      done
    done < $infoTmp
    rm -f $infoTmp $infoTmp2

    # List displayed and user responded ... now process response

    # User entered something
    if [ "$Response" != "" ]
    then
      # User picked a valid product to uninstall
      if [ "$delPC" != "" ]
      then
	if [ "$force" = "no" ]
	then
	  if [ -n "$delRelease" ]
	  then
            #$NLS-E$ "Confirm: $delPC  $delPlatform  v$delVersion:r$delRelease $delName ... OK to delete? [ y or n; \"n\" is default ]: "
			print -n "`nls_replace 'KCI0240I'`: $delPC  $delPlatform  v$delVersion:r$delRelease $delName ... `nls_replace 'KCI0241I'` [ 1-`nls_replace 'KCI0197I'`, 2-`nls_replace 'KCI0198I'`; \"2\" `nls_replace 'KCI0199I'` ]: "
	  else
            #$NLS-E$ print -n "Confirm: $delPC  $delPlatform  v$delVersion $delName ... OK to delete? [ y or n; \"n\" is default ]: "
			print -n "`nls_replace 'KCI0240I'`: $delPC  $delPlatform  v$delVersion $delName ... `nls_replace 'KCI0241I'` [ 1-`nls_replace 'KCI0197I'`, 2-`nls_replace 'KCI0198I'`; \"2\" `nls_replace 'KCI0199I'` ]: "
	  fi
          read answer
	else
	  answer="y"
	fi
        case "$answer" in
	  #$NLS-E$  y|Y|yes|YES|1) print "Uninstall product $delPC $delPlatform"
	  y|Y|yes|YES|1) print "`nls_replace 'KCI0247I' "$delPC" "$delPlatform"`"
	  
	  typeset result=$?

	  # check if product is installed based on product ver file and set isProdInstalled variable appropriately
	  isProdInstalled="no"
	  cd $regDir
	  tmpVerFile=$delPC$delPlatform.ver
	  if [ -f $tmpVerFile ]; then 
	    isProdInstalled="yes"
	  fi

	  # This returns JAVA_HOME to us on the command line - set to variable and use
	  if [ -f $CANDLEHOME/bin/CandleGetJavaHome ]
	  then
	    export JAVA_HOME=`$CANDLEHOME/bin/CandleGetJavaHome`
	  else
	    export JAVA_HOME=
	  fi

	  if [ "$isProdInstalled" = "yes" ]; then 
	    if [ -n "$JAVA_HOME" ]; then 
	      # Call the uninstall-process handler
	      uninstall_process $delPC $delPlatform
	      result=$?
	      # Call the uninstall callpoint funtion
	      if [ $result -eq 0 ]; then
	        uninstall_callpoint $delPC
	        result=$?
	      fi
	    fi
	  fi	

	  if [ $result -eq 1 ]
	  then
		#$NLS-E$ "Cancel Uninstall..."
        	print "`nls_replace 'KCI1290I'`"
	  elif [ $result -eq 0 ]
	  then
		# The uninstall_product function will check if the package is installed and display a message if not found.
		uninstall_product $delPC $delPlatform
		# if installed product was successfully uninstalled with all prerequisites and was executed by non-root, show additional message to the user
		if [ "$isProdInstalled" = "yes" ]; then 
			if [ "$(whoami)" != "root" ]; then
				#$NLS-E$ You have to be a root user to update auto restart script
				Err_report warn "`nls_replace 'KCI1167I'`"
			fi
		fi
	  elif [ $result -eq 2 ]
	  then
		# uninstall.sh failed.
		clean_up 2
	  fi
	  delPC=""
             ;;
        esac
      else
	#$NLS-E$ "$Response was not a valid selection, please try again"
	nls_replace 'KCI0242I' $Response
      fi
      Response=""

    # User entered nothing, or first time thru .. just prompt for a selection (or exit)
    else
      #$NLS-E$ "\nEnter number for a product to uninstall or \"EXIT\" to exit: "
	  print -n "\n`nls_replace 'KCI0243I' \\"99\\"` "
      read Response
      case "$Response" in
        x|X|exit|EXIT|99) clean_up
           ;;
      esac
    fi
  done
fi

clean_up
